/*   File Header   ---------------------------------
     File Name:        misc_intf.c
     Author and Date:  Paul B. Anderson
     Description:      Client interface routines for 
                       miscellaneous functions
     End Header   ---------------------------------- */


/*  INCLUDES  */

#include <geodata.h>
#include <geodata_private.h>
#include <stdio.h>


/*  PROTOTYPES  -- Export to geodata_private.h?  */

void GB_message();


/*   ========== Internal routines , definitions, and private data ==========   */

static char etext[50];

/*   ========== Routines past this point are for client use ==========   */

/*   Function Header   -----------------------------
     Description:      Initialize the GeoData software
     Parameters:       user_file  a pointer to a character string
                                 giving the name of a user GOB LISP
                                 file.  If pointer is NULL, a user
                                 file is not loaded
     Preconditions:    The Geodata software must not have been 
                       initialized prior to the call.
     Postconditions:   GeoData is initialized
     Return Value:     None
     Logic:
       1.   Initialize the DB component using GB_init.
       2.   Load reference data objects.
       3.   Load defined symbols.
       4.   Load user file if specified.
     End Header   ---------------------------------- */

void GI_init(user_file)
  char *user_file;
{
  char *place = "GI_init";
  char pathname[128];
  GB_init();
  
  /*  Load stored configuration data  */
  sprintf (pathname,"%s/line_styles.gl",GEODATA_LIB);
  GB_load_object_file(pathname);
  sprintf (pathname,"%s/symbols.gl",GEODATA_LIB);
  GB_load_object_file(pathname);
  sprintf (pathname,"%s/data_sets.gl",GEODATA_LIB);
  GB_load_object_file(pathname);
  sprintf (pathname,"%s/usgs.gl",GEODATA_LIB);
  GB_load_object_file(pathname);
  sprintf (pathname,"%s/representations.gl",GEODATA_LIB);
  GB_load_object_file(pathname);
  sprintf (pathname,"%s/scenes.gl",GEODATA_LIB);
  GB_load_object_file(pathname);
  if(user_file != NULL) GB_load_object_file(user_file);

}
