#include <geodata.h>
#include <geodata_private.h>
#include "types.h"
#include <stdio.h>

extern GB_structure GB_structures[];
extern GB_constant GB_constants[];
int GB_parse();

int GB_skip_whitespace(frame)
     GB_parse_frame_t frame;
{
  int state=1;
  char temp;

  while (state){
    temp=fgetc(frame->in);
    if (temp == '\n') frame->line_count++;
    if (temp == ';') state=2;
    if ((state ==2) && (temp == '\n')) state=1;
    if ((state ==1) && (temp != ' ') && (temp != '\n') && (temp != '\t'))
      state =0;
  }
  if (temp != EOF) ungetc(temp,frame->in);
  return (temp);
}


int GB_open_paren(frame)
     GB_parse_frame_t frame;
{
  char temp;
  
  GB_skip_whitespace(frame);
  temp=getc(frame->in);

  if (temp == '(') return (1);
  if (temp != ')') 
    fprintf (frame->err,"Expected open or close on line %d, got '%c'\n",
	     frame->line_count,temp);
  return (0);
}


int GB_decode_constant(name)
     char *name;
{
  int i;
  for (i=0;GB_constants[i].value != END_OF_CONSTANTS;i++)
    if (strncmp(name,GB_constants[i].name,GB_LABEL_LENGTH)==0)
      return(GB_constants[i].value);
  return(-1);
}


void GB_scan_float (frame,dest,type,size)
     GB_parse_frame_t frame;
     char **dest;
     int type,size;
{
  if (fscanf (frame->in,"%f",*dest) != 1)
    fprintf (frame->err,"Expected float on line %d.\n",frame->line_count);
}

void GB_scan_quoted_char_array(frame,dest,type,size)
     GB_parse_frame_t frame;
     char **dest;
     int  type,size;
{
  int i; char c;
  c = getc(frame->in);  i = 0; **dest == '\0';
  if(c != EOF) {
    if(c != '"') {
      fprintf(frame->err,"Expected quote mark but encountered %c on line %d\n",
             c, frame->line_count); }
    else {
      while ((c = getc(frame->in)) != EOF) {
        if(c == '"') { break; }
        else {
          if(i < size-1) { *(*dest+i++) = c; *(*dest+i) = '\0'; }
        }
      }
    }
  }
}


void GB_scan_string (frame,dest,type,size)
     GB_parse_frame_t frame;
     char **dest;
     int type,size;
{
  char temp[20];
  int i=0;
  if (fscanf (frame->in,"%[^) (;\n]",*dest) == 0){
    while (((temp[i++]=getc(frame->in)) != EOF) && (i < 20));
    fprintf (frame->err,"Expected string on line %d, got %s.\n",
	     frame->line_count,temp);
  }
}

void GB_scan_int (frame,dest,type,size)
     GB_parse_frame_t frame;
     char **dest;
     int type,size;
{ 
  char temp[GB_LABEL_LENGTH];
  char *hack = temp;
  int i;
  if (fscanf (frame->in,"%d",*dest) == 0){
    GB_scan_string(frame,&hack);
    for (i=0;temp[i];i++)
      if ((temp[i] <= 'z') && (temp[i] >= 'a')) temp[i]-='a'-'A';
    if ((*(int *)(*dest)=GB_decode_constant(temp))==-1){
      fprintf (frame->err,"Expected int on line %d, got %s\n",
	       frame->line_count,temp);
    }
  }
}

void GB_clean_list(where,type)
     char *where;
     unsigned type;
{
  if (where != (char *)0)
    GB_clean_list((*(char **)GB_next_instance(where,type)),type);
  (char *)GB_delete_instance(where,type);
}


void GB_scan_list(frame,dest,type,size)
     GB_parse_frame_t frame;
     char ***dest;
     int type,size;
{
  int list_type = GB_structures[type-1].list_type;
  char *kludge_and_a_half;
  char **kludge_and_three_quarters;
  char ***tnext;
  tnext=dest;
  
  kludge_and_three_quarters = (&kludge_and_a_half);
  GB_clean_list(**dest,list_type);
  while (GB_open_paren(frame)){
    GB_skip_whitespace(frame);
    **tnext = (char *)GB_create_instance(list_type);
    while (GB_open_paren(frame)) GB_parse (frame,**tnext);
    kludge_and_three_quarters = (char **)GB_next_instance(**tnext,list_type);
    tnext=(&kludge_and_three_quarters);
  }
}

/* a kludge, both this and the exceptions in 'GB_parse'*/
void GB_scan_object(frame,dest,type,size)
     GB_parse_frame_t frame;
     char **dest;
     int  type,size;
{
  int id;
  char object_name[GB_LABEL_LENGTH];
  char *hack = object_name;

/* was supposed to be existing GB_id scan...screwed up top level*/
/*  if (fscanf (in,"%d",**dest) == 0){*/
/* } */
  GB_scan_string(frame,&hack);
  if ((id = GB_get_id(object_name)) == 0)
    *dest = (char *)GB_add_object(object_name,type);
  else {
    if (*dest) **(int **)dest = id;
    else *dest=(char *)GB_get_object(id);
  }

}

/* does this need to be here?*/
void GB_scan_char(frame,dest,type,size)
     GB_parse_frame_t frame;
     char **dest;
     int  type,size;
{
  fread(*dest,1,1,frame->in);
  if (fscanf (frame->in,"%[^) (;\n]",*dest) == 0)
    if ((**dest == ')') || (**dest == '(') ||
	(**dest == ';') || (**dest == '\n')) **dest=0;
}


static GB_type types[]={
  {GB_scan_float,sizeof(float)},
  {GB_scan_int,sizeof(int)},
  {GB_scan_string,sizeof(char *)},
  {GB_scan_list,sizeof(char *)},
  {GB_scan_object,sizeof(GB_id)},
  {GB_scan_quoted_char_array,sizeof(char)}};


int GB_parse (frame,context)
    GB_parse_frame_t frame;
    char *context;
{
  unsigned type;
  int i,required,execute,size;
  char *new_context;
  char temp;
  char type_name[20];

  if (GB_skip_whitespace(frame)==EOF) return(0);

  if ((temp=getc(frame->in)) != '(') 
    ungetc(temp,frame->in);
  
  GB_skip_whitespace(frame);

  new_context=(char *)(&type);
  (*(types[GB_INT-GB_FLOAT].scanner))(frame,&new_context);
  GB_skip_whitespace(frame);

  new_context = (*(GB_structures[type-1].accessor))(context);
  required = GB_structures[type-1].number;
  for (i=0;i<GB_structures[type-1].number;i++){
    size = GB_structures[type-1].size;
    (*(types[GB_structures[type-1].type-GB_FLOAT].scanner))
             (frame,&new_context,type,size);
    if (i < required - 1)
      new_context+=types[GB_structures[type-1].type-GB_FLOAT].length*size;
    GB_skip_whitespace(frame);
  }
  while (GB_open_paren(frame)) GB_parse (frame,new_context);
  GB_execute_instance(new_context,type,frame);

  if (type == GB_HANDSHAKE) return(0);
  return(1);
}


void GB_parse_file (in,out,err)
     FILE *in,*out,*err;
{
  int c;
  struct GB_parse_frame frame;
  frame.in=in;
  frame.out=out;
  frame.err=err;
  frame.line_count=1;
  
  while (GB_parse (&frame,0));
}


void GB_load_object_file(filename)
     char *filename;
{
  FILE *in;
  char error[128];

  if ((in=fopen(filename,"r")) == 0){
    sprintf (error,"Couldn't open file %s",filename);
    GB_message (GB_NOTIFY,"GB_load_object_file",error);
  } else {
    GB_parse_file (in,stdout,stderr);
  }
}
