/******************************************************************************
*                                                                             *
*   BEAMTABLE: Calculates the weighting tables required to form beams in the  *
*               frequency domain for an array partitioned into two sub-arrays *
*               Beams are equally spaced in sin space.                        *
*                                                                             *
*  beam_file_name - Front-end file containing description of array            *
*  real_table     - Pointer to slice of CM memory to contain beam weights     *
*  c1             - Number of phones in first sub-array                       *
*  c2             - Number of phones in second sub-array                      *
*  cp             - Number of arrays per PE                                   *
*  i              - Beam number                                               *
*  nbeams         - Number of beams                                           *
*  cnst           - Bins between subarray centers                             *
*  cal_file_name  - Front-end file containing array calibration data          *
*                                                                             *
******************************************************************************/

#include <stdio.h>
#include <cm/paris.h>
#include <math.h>


    BEAMTABLE (beam_file_name,real_table, c1,c2,cp,j,nbeams,cnst,cal_file_name)
    char beam_file_name[128],cal_file_name[128];
    unsigned real_table, c1,c2,cp,j,nbeams;
    float *cnst;
 {
  static unsigned iflag=0;
  static unsigned i, k, p ;
  static unsigned temp_poly,temp,name,send_name,addr,cal;
  static float ratio,constant,dtemp1,dtemp2,d[128],w[128];
  static int pcal,n,num,ctot;
  static FILE *fopen(), *fp;
  static CM_geometry_id_t current_geometry;

  if (iflag == 0) {
    iflag=1;
    num=(c1+c2)*cp;
    ctot=c1+c2;
    ratio=1.;
    d[0]=0.;
    if ( ctot > 1) {
      fp=fopen (beam_file_name,"r");
      if (fp == NULL) { fprintf (stderr,"Couldn't open file \n"); exit(0);}
      fscanf (fp,"%f",&ratio);
      printf (" %f \n ",ratio);
      for (i=0;i<ctot;i++) {
        fscanf (fp,"%f",&d[i]);
        printf (" %f \n",d[i]);
       }
    }

    dtemp1=0.;
    if ( c1 > 1) dtemp1 = .75*(d[0]+d[c1-1]) - .25*(d[1]+d[c1-2]);
    for (i=0;i<c1; i++) d[i] -= dtemp1;
    dtemp2=0.;
    if (c2 > 1) dtemp2 = .75*(d[c1]+d[ctot-1]) - .25*(d[c1+1]+d[ctot-2]);
    for (i=c1;i<ctot; i++) d[i] -= dtemp2;

    *cnst = ratio * (dtemp2-dtemp1);
    printf (" cnst = %f \n ", *cnst);
     w[0]=1.;
    if (c1 >1) TSHADE (d,w,c1,5,25.);
    w[c1]=1.;
    if (c2>1) TSHADE (&d[c1],&w[c1],c2,5,25.);

    temp_poly=CM_allocate_stack_field(32);
    temp=CM_allocate_stack_field(32);
    cal=CM_allocate_stack_field(64*ctot);
    for (i=0; i<ctot*64; i += 64) {
      CM_f_move_constant_1L (cal+i,1.,23,8);
      CM_f_move_zero_1L (cal+i+32,23,8);
    }
    if (*cal_file_name != '\0') READCMFILE (cal_file_name,cal,64*ctot);
    current_geometry=CM_vp_set_geometry(CM_current_vp_set);
    n= CM_geometry_axis_length(current_geometry,0);
    p= CM_geometry_send_address_length(current_geometry);

    name = CM_allocate_stack_field (32) ;
    send_name = CM_allocate_stack_field (32) ;
    CM_u_move_zero_1L (name,32);
    CM_u_move_zero_1L (send_name,32);
    CM_my_send_address_1L(send_name);
    CM_extract_news_coordinate_1L (CM_vp_set_geometry(CM_current_vp_set),
             name,0,send_name,32) ;
    CM_f_u_float_2_2L ( temp, name, 32, 23, 8) ;
  }

  for (k=0; k< num; k += ctot) {
   for (i=0 ; i<ctot ; i++ ) {
     addr=real_table+64*(k+i);
     constant=0.;
     if ( nbeams != 1) {
        constant=-1.+2.*j/(nbeams-1.);
        constant=constant*d[i]*(0.0003834929*ratio*8192./n);
      }
     CM_f_multiply_constant_3_1L ( temp_poly,temp,constant,23,8);
     CM_f_cos_2_1L (addr,temp_poly,23,8);
     CM_f_sin_2_1L (addr+32,temp_poly,23,8);
     CM_f_negate_1_1L (addr+32,23,8);
     CM_f_multiply_constant_2_1L ( addr,w[i],23,8);
     CM_f_multiply_constant_2_1L ( addr+32,w[i],23,8);
     CM_c_multiply_2_1L(addr,cal+64*i,23,8); 
     CM_set_context();
   }
  }

  if ( j+1 == nbeams) CM_deallocate_stack_through (temp_poly);
}
