/*

  BITREV2D - Perfroms the bit reverse scrambling associated with the FFT.
             This will work for either the case when the FFT is across the
             whole CM or when multiple FFTs are performed simultaneously.
             The routine can also shift the data ( representing a time delay
             if the data is time series or a phase shift if the data is 
             in the frequency domain).  The method used is to set up a location
             in each PE which will contain the PE number, subtract the shift
             desired from the contents of this location, bit reverse the
             bottom p ( where 2^p is the FFT size) bits of the contents of
             this location, and then use the contents as the detination PE of
             a router request.  The calling sequence is

     BITREV2D (slice, p, num,offset, geometry)

  where

    slice     - pointer to the slice of data on the CM to be scrambled
    p         - log to the base 2 of the FFT size
    num       - number of bits per PE in the data slice
    offset    - number of points to shift the data - a right shift)
    geeometry - CM configuaration - required only for multiple FFTs
            
 */

#include <cm/fast-graph.h>
#include <cm/paris.h>
#include <math.h>

void BITREV2D (real_in,  p, num, offset)
unsigned real_in, p, num;
int offset;
{
  unsigned i, pcal, name, save_context_flag;
  unsigned i1,i2,channel ;
  unsigned n = 1 << p;
  static unsigned last_value_of_p = 0;
  static char *digit[] = { "0","1","2","3","4","5","6","7","8","9" };
  static char out_file[80]=" this is a test";
  static CMFG_trace_id_t trace;
  static CM_geometry_id_t current_geometry;

/* ---------------- INITIALIZATION ---------------- */

 current_geometry = CM_vp_set_geometry(CM_current_vp_set);
 pcal = CM_geometry_send_address_length(current_geometry);
 save_context_flag = CM_allocate_stack_field(1);
 CM_store_context(save_context_flag);
 CM_set_context();
 name = CM_allocate_stack_field(pcal);
 CM_u_move_zero_1L(name, pcal);
 CM_my_send_address_1L(name);

/* --------------------- BEGIN BIT REVERSAL --------------------- */

{ unsigned numb, temp;
  unsigned sa,x,y;

 numb=CM_allocate_stack_field(pcal);
 temp=CM_allocate_stack_field(num);
    
/* --------- DETERMINE DESTIONATION PE NUMBER ------------------ */
  if (pcal!= last_value_of_p) {
    last_value_of_p=pcal;
    strcpy (out_file,"FG/brvtrace");
    channel=CM_geometry_coordinate_length(current_geometry,0);
    i1=channel/10;
    i2=channel-i1*10;
    strcat (out_file,digit[i1]);
    strcat (out_file,digit[i2]);
    channel=CM_geometry_coordinate_length(current_geometry,1);
    strcat (out_file,digit[channel]);
    trace=CMFG_dvpath_read_trace(out_file);
 }
    CMFG_deliver_1L ( temp,trace,real_in,num);  

/* --------------------- BIT REVERSE ORDER DATA --------------------- */

 CM_u_move_1L (real_in,temp,num);
}

CM_load_context(save_context_flag);
CM_deallocate_stack_through(save_context_flag);
}
