/******************************************************************************
*                                                                             *
*   BMMACHELL - Merges the braodband correlations from a set of               *
*                beam BB correlations using the method described              *
*               in Section 5.2  of Fred Machell's paper, "Algorithms          *
*               for Broadband Processing and Display", ARL, University        *
*               of Texas, Feb 1990                                            *
*                                                                             *
*         out  - Slice in CM memory to contain merged beams                   *
*         in   - Slice in CM memory containing beam BB                        *
*                correlations                                                 *
*         num  - Number of BB beam correlations                               *
*         size - 64                                                           *
*         cnst - Number of bins of time delay betwwen subarray centers        *
*                                                                             *
******************************************************************************/

#include <cm/paris.h>
#include <math.h>


  void BMMACHELL ( out,in,num,size,cnst)
   unsigned in, out;
   int num,size;
   float cnst;

{
  int i,k;
  int p,pcal,n;
  float c,d,e,temp;
  static float tau[128];
  static iflag=0;
  static CM_memaddr_t alpha,fname;
  static CM_memaddr_t ctx ;
  CM_memaddr_t name,name_send;
  CM_geometry_id_t current_geometry;

  if ( iflag == 0) {
    iflag =1;
    printf (" This statement should appear only once ");
    current_geometry=CM_vp_set_geometry(CM_current_vp_set);
    p= CM_geometry_coordinate_length(current_geometry,0);
    n=1<<p;
    pcal= CM_geometry_send_address_length(current_geometry);
    for (k=0; k< num; k++){
      tau[k]=(cnst*(2*(k+1)-num-1)/(num-1))/2+n/2;
     }
    alpha = CM_allocate_stack_field(128);
    fname = CM_allocate_stack_field(32);
    ctx = CM_allocate_stack_field (1);
    name_send = CM_allocate_stack_field(pcal);
    name = CM_allocate_stack_field(p);
    CM_store_context (ctx);
    CM_set_context();
    CM_u_move_zero_1L(name, p);
    CM_u_move_zero_1L(name_send, pcal);
    CM_my_send_address(name_send);
    CM_extract_news_coordinate_1L(current_geometry,name,0,name_send,p);
    CM_f_u_float_2_2L (fname,name,p,23,8);
    CM_u_move_zero_1L (alpha,128);
    for (i=1; i< num; i++ ) {
     CM_f_gt_constant_1L(fname, tau[i-1], 23,8);
     CM_logand_context_with_test();
     CM_f_le_constant_1L(fname, tau[i], 23,8);
     CM_logand_context_with_test();
     CM_f_move_1L (alpha,fname,23,8);
     CM_f_subtract_constant_2_1L(alpha,tau[i-1],23,8);
     CM_f_divide_constant_2_1L ( alpha,tau[i]-tau[i-1],23,8);
     CM_f_move_1L (alpha+32,alpha,23,8);
     CM_f_subfrom_constant_3_1L (alpha+64,alpha,1.,23,8);
     CM_f_move_1L (alpha+96,alpha+64,23,8);
     CM_set_context();
   }
     CM_deallocate_stack_through (name);
  }
  CM_load_context(ctx);

/******************************************************************************
*                                                                             *
*       Merge in BB correlation of -90 end fire beam                          *
*                                                                             *
******************************************************************************/
   CM_f_le_constant_1L(fname, tau[0], 23,8);
    CM_logand_context_with_test();
    CM_u_move_1L (out,in,size);
    CM_set_context();

/******************************************************************************
*                                                                             *
*       Merge beams i-1 and i from ntau[i-1] to ntau[i]                       *
*                                                                             *
******************************************************************************/
  for (i=1; i< num; i++ ) {
   CM_f_gt_constant_1L(fname, tau[i-1], 23,8);
   CM_logand_context_with_test();
   CM_f_le_constant_1L(fname, tau[i], 23,8);
   CM_logand_context_with_test();
   CM_f_multiply_3_1L (out,in+i*size,alpha,23,8);
   CM_f_multiply_3_1L (out+32,in+i*size+32,alpha+32,23,8);
   CM_f_mult_add_1L(out,in+(i-1)*size,alpha+64,out,23,8);
   CM_f_mult_add_1L(out+32,in+(i-1)*size+32,alpha+96,out+32,23,8);
   CM_set_context();
  }
/******************************************************************************
*                                                                             *
*       Merge+90 end fire beam                                                *
*                                                                             *
******************************************************************************/
  CM_f_ge_constant_1L(fname, tau[num-1], 23,8);
  CM_logand_context_with_test();
  CM_u_move_1L (out,in+size*(num-1),size);

  CM_load_context(ctx);
 }


