/*
  FRAMEBUFFER - Displays 8-bit data stored in send address order to a 
                framebuffer.  Calling sequence is

       FRAMEBUFFER ( amplitude, step,copy, start_bin)


  where

      amplitude - Pointer to slice containg data to be displayed
      step      - Line on framebuffer data to start writing data
      copy      - Number of times to output line
      start_bin - Processing element to start displaying data

*/



#include <cm/paris.h>
#include <cm/cm_file.h>
#include <cm/cmfb.h>

#define NULL 0

#define DVFILE int
#define MIN(a,b) (((a) < (b)) ? (a) : (b))

  void init_frame_buffer();
  void plot_x_y();
  void plot_from_grid();
  void show_plot_x_y();
     
  static CM_memaddr_t save_buffer;
  static struct CMFB_display_id fb_display;
  static CM_vp_set_id_t fb_vp_set;
  static CM_geometry_id_t fb_geometry;
  static CM_memaddr_t fb_color;


  FRAMEBUFFER(amplitude,step,copy,start_bin)
  unsigned amplitude;
  int step,start_bin,copy;
{
  int bits, n, i, pos, dbu, buf_index; 
  CM_memaddr_t save_offset, amp_offset, fbx, fby, entry_context, addre;
  unsigned entry_vp_set = CM_current_vp_set;

  entry_context = CM_allocate_stack_field (1);
  CM_store_context (entry_context);
  addre = CM_allocate_stack_field (16);
  CM_u_move_zero_always_1L(addre, 16);

  CM_set_context();

  fbx = CM_allocate_stack_field(32);
  fby = CM_allocate_stack_field(32);
  CM_u_move_zero_always_1L(fbx, 32);
  CM_my_send_address_1L (fbx, 32);
  CM_u_move_zero_always_1L(fby, 32);
  for (i=0; i<copy; i++) {
  plot_x_y(fbx, fby, amplitude, start_bin, step+i );
  show_plot_x_y();
  }

  CM_set_vp_set (entry_vp_set);
  CM_load_context (entry_context);
  CM_deallocate_stack_through (entry_context);
}
/********************************************************************/

void INIT_FRAME_BUFFER(x_dim, y_dim)
     int x_dim, y_dim;
{
  unsigned dims[2];
  int physical_x, physical_y, zoom, c;
  char *displays[16], ctab[256];
  CM_memaddr_t entry_context;
  CM_memaddr_t entry_vp_set ;

  entry_vp_set = CM_current_vp_set;
  entry_context = CM_allocate_stack_field (1);
  CM_store_context (entry_context);
  CM_set_context();
  
  dims[0] = x_dim;
  dims[1] = y_dim;
  fb_geometry = CM_create_geometry(dims,2);
  fb_vp_set = CM_allocate_vp_set(fb_geometry);
  CM_start_timer(1);
  CMFB_available_displays(displays);
  if (CMFB_attach_display(NULL, &fb_display) < 0) {
    printf("init_frame_buffer: Could not attach to frame buffer: %s.\n", 
       displays[0]);
  }
  else{
    printf("Initializing fb %s.\n", displays[0]);
    CMFB_initialize_display(&fb_display, 24, 1);
    physical_x = CMFB_width(&fb_display);
    physical_y = CMFB_height(&fb_display); 
    zoom = MIN((physical_x / x_dim), (physical_y / y_dim)) - 1;
    CMFB_set_zoom(&fb_display, zoom, zoom, 0);
    CM_set_vp_set(fb_vp_set);
    fb_color = (CM_memaddr_t) CM_allocate_heap_field(24);
    CM_u_move_zero_always_1L(fb_color, 24);
  }
  CM_set_vp_set (entry_vp_set);
  CM_load_context (entry_context);
  CM_deallocate_stack_through (entry_context);
  CM_stop_timer(1);
}



  void plot_x_y( x, y, color, xoff, yoff)
  CM_memaddr_t x, y, color;
  int xoff, yoff;
{
  CM_memaddr_t a_send_address = CM_allocate_stack_field(32),
               xfb = CM_allocate_stack_field(32),
               yfb = CM_allocate_stack_field(32);
  CM_memaddr_t ctx = CM_allocate_stack_field (1);
  
  CM_memaddr_t entry_vp_set ;

  entry_vp_set = CM_current_vp_set;
  CM_store_context (ctx);
  CM_set_context();
  CM_u_move_zero_always_1L(a_send_address, 32);
  CM_u_move_1L(xfb, x, 32);
  CM_u_add_constant_3_1L(yfb, y, (long) yoff, 32);
  CM_u_subtract_constant_3_1L(xfb, xfb, (long) xoff, 32);
  CM_u_lt_constant_1L (xfb, 1024, 16);
  CM_logand_context_with_test();

  CMFB_shuffle_from_x_y(a_send_address, xfb, yfb, fb_geometry);
  CM_send_1L(fb_color, a_send_address, color, 24, CM_no_field);
  CM_load_context (ctx);
  CM_set_vp_set (entry_vp_set);
  CM_deallocate_stack_through (a_send_address);
}



 void show_plot_x_y()
{
  int i;
  CM_memaddr_t ctx = CM_allocate_stack_field (1);
  CM_memaddr_t entry_vp_set ;

  entry_vp_set = CM_current_vp_set;
  CM_store_context (ctx);
  CM_set_vp_set(fb_vp_set);
  CM_set_context();  
  CMFB_write_preshuffled_always(&fb_display,
				CMFB_current_buffer(&fb_display),
				fb_color, 0, 0);
  CM_set_vp_set (entry_vp_set);
  CM_load_context (ctx);
  CM_deallocate_stack_through (ctx);  
}


