/*

  GFILTBP - Generates square pass band filters for broadband correlations.
            Leaves filter iin first coordinate untouched.  Calling sequence 
            is

      GFILTBP ( gfilt ,nlo, nhi,bitrev)

  where
    
       gfilt   - Pointer to slice of filter amplitudes in CM memory
       bitrev - Flag that indicates whether or not filter is in normal order 

*/
#include <cm/paris.h>

  void GFILTBP (gfilt,nlo,nhi,bitrev)
  unsigned gfilt;
  int nlo,nhi;
  int bitrev;

{
  int pcal,i,p,mult;
  unsigned name,name_send,name1, name2, save_context_flag;
  CM_geometry_id_t default_geometry_id;
  default_geometry_id = CM_vp_set_geometry(CM_current_vp_set);
  pcal = CM_geometry_send_address_length(default_geometry_id);
  p = CM_geometry_coordinate_length(default_geometry_id,0);
  save_context_flag = CM_allocate_stack_field(1);
  CM_store_context(save_context_flag);
  name_send = CM_allocate_stack_field(pcal);
  name1 = CM_allocate_stack_field(pcal);
  name2 = CM_allocate_stack_field(pcal);
  CM_my_send_address_1L(name_send);
  CM_u_move_zero_1L (name1,pcal);
  CM_u_move_zero_1L (name2,pcal);
  CM_extract_news_coordinate_1L(default_geometry_id,name1,0,name_send,pcal);
  if (p !=pcal)
    CM_extract_news_coordinate_1L(default_geometry_id,name2,1,name_send,pcal);

  if ( bitrev == 1) CM_move_reversed_1L (name1,name1,p);

    CM_f_move_zero_1L (gfilt,23,8);
    CM_u_gt_constant_1L (name1,nlo,pcal);
    CM_logand_context_with_test();
    CM_u_lt_constant_1L (name1,nhi,pcal);
    CM_logand_context_with_test();
    CM_f_move_constant_1L (gfilt,1.,23,8);
    CM_load_context(save_context_flag);

  CM_deallocate_stack_through(save_context_flag);
}
 
