/*

  GS_LINEAR - Maps the broad band correlations to a 16 level gray
                 scale using the nonlinear mapping given in Table 5.1
                 of Fred Machell's report.  Calling sequence is 

            GS_LINEAR ( out,in)

  where 

       out - Pointer to 8-bit slice in CM memory containing
             grey scale levels
       in  - Pointer to 32-bit slice in CM memory containing
             floating point correlation values.  

*/
#include <cm/paris.h>


void   GS_LINEAR ( out,in)
   unsigned in, out;

{
  unsigned temp;
  CM_memaddr_t ctx = CM_allocate_stack_field (1);
  float alpha=1.;

  CM_store_context (ctx);
  temp=CM_allocate_stack_field(32);
  CM_u_move_1L(temp,in,32);
  CM_f_ge_constant_1L(temp, alpha, 23, 8);
  CM_logand_context_with_test();
  CM_f_move_constant_1L(temp, alpha, 23, 8);
  CM_load_context(ctx);
  CM_f_le_constant_1L(temp, -alpha, 23, 8);
  CM_logand_context_with_test();
  CM_f_move_constant_1L(temp, -alpha, 23, 8);
  CM_load_context(ctx);
  CM_f_add_const_mult_const_1L (temp,temp,alpha,127./alpha,23,8);
  CM_f_add_constant_2_1L (temp,1.,23,8);  
  CM_u_f_floor_2_2L(out,temp,8,23,8); 
  CM_deallocate_stack_through(ctx);
  
}
