/*

  GSMACHELL - Maps the broad band correlations to a 16 level gray
                 scale using the nonlinear mapping given in Table 5.1
                 of Fred Machell's report.  Calling sequence is 

            GSMACHELL ( out,in, sigma)

  where 

       out - Pointer to 8-bit slice in CM memory containing
             grey scale levels
       in  - Pointer to 32-bit slice in CM memory containing
             floating point correlation values.  
       sigma - Standard deviation of bb correlation
*/
#include <cm/paris.h>


void   GSMACHELL ( out,in,sigma)
   unsigned in, out;
   float sigma;

{
/*  Sigma set for 400Hz data and 4K nonoverlapped FFT */
    int i;
    static int flag=0;
    static float level[15];
    CM_memaddr_t ctx = CM_allocate_stack_field (1);

    if (flag == 0) {
      level[0]= -8.0000 * sigma;
      level[1]= -5.7349 * sigma;
      level[2]= -4.0195 * sigma;
      level[3]= -2.7204 * sigma;
      level[4]= -1.7366 * sigma;
      level[5]= -0.9915 * sigma;
      level[6]= -0.4273 * sigma;
      level[7]= -0.0000 * sigma;
      level[8]=  0.4273 * sigma;
      level[9]=  0.9915 * sigma;
      level[10]= 1.7366 * sigma;
      level[11]= 2.7204 * sigma;
      level[12]= 4.0195 * sigma;
      level[13]= 5.7340 * sigma;
      level[14]= 8.0000 * sigma;
      flag=1;
    }
   CM_store_context (ctx);
   CM_set_context();
   CM_u_move_zero_1L (out,8);

   for (i=0; i< 15; i++ ) {
     CM_f_ge_constant_1L(in, level[i], 23, 8);
     CM_logand_context_with_test();
     CM_u_add_constant_2_1L (out,14,8);
   }
  CM_load_context(ctx);
  CM_deallocate_stack_through (ctx);
  }
