/******************************************************************************
*                                                                             *
*  HAMM2 -  Calculates the Hamming window  for the Hamming window required    *
*           for a FFT of size n and stores it in a slice of CM memory. In     *
*           the case when multiple FFT are calculated simultaneously across   *
*           the CM multiple copies of the window are stored in the slice.     *
*                                                                             *
*      window - Pointer to the 64 bit slice of PE memory for the window       *
*      n      - Number of points in the window                                *
*                                                                             *
******************************************************************************/

#include <cm/paris.h>
#include <math.h>

void HAMM2 (window, n)
unsigned window,n ;
{
   unsigned name,send_name ;
   float theta ;
   unsigned j ;
   unsigned p;

   p=CM_geometry_send_address_length(CM_vp_set_geometry(CM_current_vp_set));
   name = CM_allocate_stack_field (p) ;
   send_name = CM_allocate_stack_field (p) ;
   CM_u_move_zero_1L (send_name,p);
   CM_u_move_zero_1L (name,p);
   CM_my_send_address_1L (send_name) ;
   CM_extract_news_coordinate_1L (CM_vp_set_geometry(CM_current_vp_set),
           name,0,send_name,p) ;
   CM_f_u_float_2_2L ( window, name, p, 23, 8) ;
   theta =2.* 2.*M_PI/n ;
   CM_f_multiply_constant_2_1L ( window, theta, 23, 8) ;
   theta = 2.*M_PI/n ;
   CM_u_move_1L (window+32,window,32);
   CM_f_add_constant_2_1L ( window+32, theta, 23, 8) ;
   CM_f_cos_1_1L ( window, 23, 8) ;
   CM_f_mult_const_add_const_1L ( window, window , -.46, .54, 23, 8);
   CM_f_cos_1_1L ( window+32, 23, 8) ;
   CM_f_mult_const_add_const_1L ( window+32, window+32 , -.46, .54, 23, 8);
   CM_deallocate_stack_through ( name);
}
