/******************************************************************************
*                                                                             *
*  LOAD_DECOMP_TABLE - Loads into a slice in CM memory as a shared array      *
*                      the decompansion table.  The data is loaded in such    *
*                      a way that each set of 32 PEs contains all table       *
*                      values.                                                *
*                                                                             *
*       te   - Front end array containing decompansion table                  *
*       cmte - Pointer to slice of CM memory to contain table                 *
*                                                                             *
******************************************************************************/


#include <stdio.h>
#include <cm/paris.h>
#include <math.h>

int LOAD_DECOMP_TABLE(te, cmte)
     float te[];
     CM_memaddr_t cmte;
{
  int i;
  CM_memaddr_t x, j;

  x = CM_allocate_stack_field(32);
  j = CM_allocate_stack_field(32);

  for(i = 0; i < 128; i++){
    CM_f_move_constant_1L(x,te[i], 23, 8);
    CM_s_move_constant_1L(j, i, 32);
    CM_aset32_shared_2L(x, cmte, j, 32, 32, 128);
  }
  CM_deallocate_stack_through(x);
} 
