/******************************************************************************
*                                                                             *
*    SCOT - Calculates the SCOT weighting of the complex vector x stored      *
*           in CM meory, saves the weighting vector in CM memory, and then    *
*           normalizes the vector x by the SCOT weighting storing the result  *
*           in x. The SCOT weighting formula is given by                      *
*                                                                             *
*       w(k)= | x(k)|^2      for k=0                                          *
*       w(k+1)= alpha*w(k) + (1-alpha)*|x(k+1)|^2  for k> 0                   *
*                                                                             *
*   and the normalization is given by                                         *
*                                                                             *
*       x(k)=x(k)/sqrt(w(k))    for all k                                     *
*                                                                             *
*       w      - CM location of SCOT weights                                  *
*       x      - CM vector of complex inputs                                  *
*       number - Number of complex vectors on which to perform SCOT weighting *
*       flag   - Flag to indicate whether or not this is the initial          *
*                calculation of the SCOT weights                              *
*                                                                             *
******************************************************************************/

#include <cm/paris.h>


  void SCOT(w, x,number,flag)
  unsigned w,x;
  int number,flag;

{
  unsigned  j ;
  float alpha=.5;
  unsigned temp;

  temp=CM_allocate_stack_field(32);
  if (flag ==0 ) alpha=0. ;

  for (j=0; j< number; j++) {
    TAKE_AMPLITUDE_SQ (temp,x+64*j);
    CM_f_multiply_constant_2_1L (temp, 1.-alpha,23,8);
    CM_f_mult_const_add_1L (w+32*j,w+32*j,alpha,temp,23,8); 
    CM_f_sqrt_2_1L (temp,w+32*j,23,8);
    CM_f_divide_2_1L ( x+64*j,temp,23,8);
    CM_f_divide_2_1L ( x+64*j+32,temp,23,8);
   }
 CM_deallocate_stack_through(temp);

}
