/******************************************************************************
*                                                                             *
*  sertopar.c - Remultiplex a serial DataVault file aad write it out to       *
*               a parallel DataVault file. Running sequence is                *
*                                                                             *
*    sertopar in out -p n1 -b n2 -f -c n3 -v n4 -m n5 -b n6 -r n7 n8 -z n9    *
*                                                                             *
*       where all - switches are optional and order independent               *
*                                                                             *
*         in       -  Input DataVault file                                    *
*         out      -  Output DataVault file                                   *
*         -p n1    -  Log base 2 of the number of virtual processors          *
*                     for the geometry of the outputDataVault file            *
*         -b n2    -  Number of bytes comprising a time-series sample         *
*         -f       -  Flag indicating that the samples are floating point     *
*         -c n3    -  Number of copies of output data blocks to store across  *
*                     Connection Mahine processing elements                   *
*         -v n4    -  Number of copies of output data blocks to store within  *
*                     each processing element                                 *
*         -m n5    -  Number of channels of data                              *
*         -b n6    -  Number of bytes per sample                              *
*         -r n7 n8 -  Factor to increase block size ; Number of bits per      *
*                     channel in each processing element                      *
*         -z n9    -  Number of bytes in serial input file                    *
*                                                                             *
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <cm/paris.h>
#include <cm/cm_file.h>
#include <cm/cmfs.h>
#include <cm/cm_stat.h>

void main(argc, argv)
int argc;
char **argv;
{

  char dv_in_file[128],dv_out_file[128];
  unsigned in_data, out_data;
  unsigned sa,y,n, py;
  int bits_in,bits_out,ptr;
  int i,j,k,kk,move,p=13,bytes=1, copy=1, vc=1;
  int lines=256,channels=8,pts;
  int number_bytes =0, swap=0, flt=0;
  int steps,buffer,inner,offset;
  int dvfile_in,dvfile_out;
  int saved_channels ;
  int remux=1,depth=8;
  CM_geometry_id_t new_geometry;
  CM_geometry_id_t default_geometry;
  unsigned new_vp_set;
  unsigned dimension_array[1];
  struct CM_axis_descriptor axe_desc[32];
  CM_axis_descriptor_t axe_desc_t[32];
  long transpose_size=16384/2/4;
  pts=1<<p;

  if (argc < 3)
    {printf("useage: reformat in_filename out_filename.\n"); exit(0);}
  strcpy(dv_in_file, argv[1]);
  strcpy(dv_out_file, argv[2]);
  for (ptr=3; ptr<argc; ptr++) {
    char *arg = argv[ptr];
    if (arg[0] == '-') {
      switch (arg[1]) {
      case 'm':                          
        if (++ptr >= argc) exit(1);
        channels = atoi(argv[ptr]);
        continue;
      case 'b':                          
        if (++ptr >= argc) exit(1);
        bytes = atoi(argv[ptr]);
        continue;
      case 'p':                          
        if (++ptr >= argc) exit(1);
        p = atoi(argv[ptr]);
        pts=1<<p;
        continue;
      case 'c':                          
        if (++ptr >= argc) exit(1);
        copy = atoi(argv[ptr]);
        continue;
      case 'v':                          
        if (++ptr >= argc) exit(1);
        vc = atoi(argv[ptr]);
        continue;
      case 'z':                          
        if (++ptr >= argc) exit(1);
        number_bytes = atoi(argv[ptr]);
        continue;
      case 'r':                          
        if (++ptr >= argc) exit(1);
        remux = atoi(argv[ptr]);
        if (++ptr >= argc) exit(1);
        depth = atoi(argv[ptr]);
        continue;
      case 's':
        swap=1;
        continue;
      case 'f':
        flt=1;
        continue;
      default: exit(1);
      }
    }
    else exit(1);
  }

  CM_init();
  CM_set_safety_mode (0);


   for (i = 0; i < 2; i++) {
        axe_desc[i].weight = (1-i)*10;
        axe_desc[i].ordering = CM_send_order;
        axe_desc[i].off_chip_bits = 0;
        axe_desc[i].off_chip_pos = 0;
        axe_desc[i].on_chip_bits = 0;
        axe_desc[i].on_chip_pos = 0;
        axe_desc_t[i] = &(axe_desc[i]);
      }
 default_geometry=CM_vp_set_geometry(CM_current_vp_set);
  axe_desc[1].length = copy;
  axe_desc[0].length = pts/(copy);
  new_geometry = CM_create_detailed_geometry((unsigned *)(&axe_desc_t[0]), 2);

  new_vp_set = CM_allocate_vp_set(new_geometry);
  CM_set_vp_set(new_vp_set);
  CM_set_context();

  in_data=CM_allocate_stack_field(transpose_size/copy);
  out_data=CM_allocate_stack_field(transpose_size*vc);
  sa=CM_allocate_stack_field(p);
  y=CM_allocate_stack_field(p);
  CM_my_send_address_1L (sa);
  CM_u_move_zero_1L(y,p);
  CM_extract_news_coordinate_1L(new_geometry,y,1,sa,p);

  printf("opening in file...\n");
  if ((dvfile_in = CMFS_open(dv_in_file, CMFS_O_RDONLY, 0644))  == -1){
    printf("error on dv file open: %s\n", dv_in_file);
    exit(1);  }
  printf("opening out file...\n");
  if ((dvfile_out = CMFS_open(dv_out_file, 
     CMFS_O_WRONLY | CMFS_O_CREAT | CMFS_O_APPEND, 0644))  == -1){
    printf("error on out file open: %s\n", dv_out_file);
    exit(1);  }
  bits_in=transpose_size/copy;
  while (bits_in == transpose_size/copy) {
    bits_in = CMFS_read_file(dvfile_in, in_data, transpose_size/copy);
    if (bits_in == -1){CMFS_perror("Couldn't read some data"); exit(-1);}
    else printf("read %d bits of data  per vp.\n",
	      bits_in);
     bits_in=(bits_in/channels/remux/depth);
     bits_in=(bits_in*channels*remux*depth);
     REFORM (in_data,bits_in,channels,remux,depth);
     for (inner=0; inner< bits_in; inner+= 8*bytes) {
        CMFS_transpose_always(in_data+inner,8*bytes,
             CMFS_IO_READ_FROM_SERIAL_DATA,CMFS_read_from_column_major); 
      if (swap != 0) {
         for (i=0; i< bytes/2; i++) {
          CM_swap_2_1L ( in_data+inner+16*i+8, in_data+inner+16*i,8);
	 } 
      }
      if (flt != 0) {
         for (i=0; i< bytes/2; i++) {
          CM_swap_2_1L ( in_data+inner+8*(bytes-1-i), in_data+inner+8*i,8);
	 } 
      }
       }
    if (copy == 1) {
      offset =0;
      for (j=0;j<bits_in; j+= 8* bytes * channels){
        for (i=0;i<vc;i++){
         CM_u_move_1L (out_data+offset,in_data+j,8*bytes*channels);
         offset += 8*bytes*channels;
        }
      }
      bits_out = CMFS_write_file(dvfile_out, out_data,bits_in*vc);
     }
    else {
    offset=0;
    k=0;kk=0;
    for (j=0; j< bits_in; j+= 8*bytes*channels/copy ) {
    for (i=0;i< channels;i++) {
       CM_spread_with_copy_1L(out_data+offset, in_data+kk*8*bytes,1,
                8*bytes,k);
       k += 1; if (k == copy) {k=0; kk +=1;}
       offset += 8*bytes;
      }
     offset -= 8*bytes*channels;
     for (i=1;i<vc;i++) {
      CM_u_move_1L(out_data+offset+8*bytes*channels,
                    out_data+offset,8*bytes*channels);
      offset += 8*bytes*channels;
      }
      offset += 8*bytes*channels;
    }
    bits_out = CMFS_write_file(dvfile_out,out_data,bits_in*copy*vc);
    if (bits_out == -1)
      {CMFS_perror("Couldn't write some data"); exit(-1);}
     }

  if (number_bytes != 0) {
    number_bytes -= (transpose_size/8/copy)*pts;
    if (number_bytes <= 0) exit(0);
   }
  }
}
