/******************************************************************************
*                                                                             *
*  SHIFT - Performs a circular shift on an array using power-of-two moves.    *
*          This may or may not be faster than a single send depending on the  *
*          number of power-of-two moves required. The axis must be in news    *
*          order for this algorithm to work efficiently                       *
*                                                                             *
*    dest - Destination field                                                 *
*    src  - Source field                                                      *
*    axis - Axis along which to shift                                         *
*    dist - Distance to shift (positive represents a left shift)              *
*    len  - Length of the source and destination fields                       *
*                                                                             *
******************************************************************************/

#include <cm/paris.h>
#include <math.h>
#include <stdio.h>

SHIFT(dest, src, axis, dist, len)
unsigned dest, src, axis, len;
int dist;
  {
    CM_u_move_1L(dest, src, len);
    while (dist)
      {
	CM_get_from_power_two_always_1L(dest, dest, axis, first(dist), 
			  dist < 0 ? CM_downward : CM_upward, len);
	if (dist > 0)
	  dist -= 1 << first(dist);
	else dist += 1 << first(dist);
      }
  }


first(dist)
int dist;
  {
    int i, j;

    if (dist < 0)
      dist = -dist;
    for (i = 1, j = 0; i < dist; i <<= 1, j++);
    if (i - dist < dist - (i >> 1))
      return j;
    else return j - 1;
  }
