/******************************************************************************
*                                                                             *
*  test.c - Tests svd                                                         *
*                                                                             *
******************************************************************************/

#define TRUE          1
#define FALSE         0
#define DVFILE int
#define print(x) fprintf(stdout,"\n"); fprintf(stdout,x);

#include <sys/types.h>
#include <cm/cm.h>
#include <cm/cm_stat.h>
#include <cm/cm_file.h>
#include <cm/cmfs.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <cm/paris.h>
#include <cm/cmssl-paris.h>

void main(argc, argv)
     int argc;
     char **argv;
{
/******************************************************************************
*                                                                             *
*   Set-up variables                                                          *
*                                                                             *
******************************************************************************/
  char dv_in_file_name[128], dv_out_file_name[128],beam_file_name[128];
  char param_file_name[128], dv_cal_file_name[128];
  DVFILE dv_infile, dv_outfile, dv_crossfile;
  u_char *grid;
  int oddsteps,number;
  int aver,save_aver;
  int pts, steps, ptr, step, outfile_flag = 0;
  int copy,np,nproc;
  float cnst,dstep;
  int show_fb = FALSE, take_hann = FALSE,
  decomp=FALSE, int_data=FALSE,
  do_overlap = FALSE, save_grid = TRUE, ser_data=FALSE;
  int offset=0,update;
  int noff,ndisp=8;
  int sample=32;
  int field_idno=0;
  int sweep_count=0;
  int debu, dbu;
  int ncut,out_channels;
  int n_beams,sweeps_in,sweeps_out,cut,clusters;
  int p,channels_per_cluster_1,channels_per_cluster_2,channels_per_cluster;
  int channels,overlap,saved_channels;

  int time=FALSE;
  int time_all=0,time_in=1,time_bitrev=2,time_reformat=3,time_window=4,
      time_fft=5,time_unsfft=6,time_beam=7,time_amp=8,time_nse=9,
      time_clip=10,time_frame=11,time_trans=12,time_out=13,time_cross=14,
      time_abf=15,time_axis=16;

  FILE *fopen(), *fp;
  struct cm_stat instat;

  unsigned  pure_data, old_data, new_data, beam_table[128], cmte, window,
            fft_aver, fft_temp, amplitude, signed_amplitude, fft_amp,
            amp_out, ali, beam_real, abf_table[128], cross_spect, temp;
  
  CM_geometry_id_t send_geometry,news_geometry;
  unsigned dimension_array[2], new_vp_set;
  struct CM_axis_descriptor axe_desc[32];
  CM_axis_descriptor_t axe_desc_t[32];
  int channel, bits, odd, beam, table_index, i;
  int mv_offset;
  unsigned sa,my_address;

/******************************************************************************
*                                                                             *
*   Initialize CM and file names                                              *
*                                                                             *
******************************************************************************/

  CM_init ();
  CMSSL_init ();
  CM_set_safety_mode (1); 
  CM_reset_timer(0);

  dv_cal_file_name[0] = '\0';
  dv_cal_file_name[0] = '\0';
  dv_out_file_name[0] = '\0';
  param_file_name[0] = '\0';
  beam_file_name[0] = '\0';
  dv_cal_file_name[0] = '\0';
  if (argc > 1) strcpy(dv_in_file_name, argv[1]);
  if (argc > 2) strcpy(dv_out_file_name, argv[2]);
  if (argc > 3) strcpy(param_file_name, argv[3]);
  if (argc > 4) strcpy(beam_file_name, argv[4]);
  if (argc > 5) strcpy(dv_cal_file_name, argv[5]);

/******************************************************************************
*                                                                             *
*   Get parameters from front end file                                        *
*                                                                             *
******************************************************************************/

  fp=fopen (param_file_name,"r");
  if (fp == NULL) { fprintf (stderr,"Couldn't open file \n"); exit(0);}
    fscanf (fp,"%d %*S \n",&p);
      pts=1<<p;
      printf ("%d \n",pts);
      update=pts/2;
    fscanf (fp,"%d %*S \n",&clusters);
      printf ("%d \n",clusters);
    fscanf (fp,"%d %*S \n",&channels_per_cluster_1);
      printf ("%d \n",channels_per_cluster_1);
    fscanf (fp,"%d %*S \n",&channels_per_cluster_2);
      printf ("%d \n",channels_per_cluster_2);
      channels_per_cluster=channels_per_cluster_1+channels_per_cluster_2;
      channels=channels_per_cluster*clusters;
    fscanf (fp,"%d %*S \n",&n_beams);
      printf ("%d \n",n_beams);
      saved_channels=n_beams*clusters;
    fscanf (fp,"%d %*S \n",&sweeps_in);
      printf ("%d \n",sweeps_in);
    fscanf (fp,"%d %*S \n",&sweeps_out);
      printf ("%d \n",sweeps_out);
    fscanf (fp,"%d %*S \n",&cut);
      printf ("%d \n",cut);
      ncut=1<<cut;
    fscanf (fp,"%d %*S \n",&steps);
      printf ("%d \n",steps);
    fscanf (fp,"%d %*S \n",&aver);
      printf ("%d \n",aver);
      save_aver = aver;
    fscanf (fp,"%d %*S \n",&int_data);
      printf ("%d \n",int_data);
    fscanf (fp,"%d %*S \n",&decomp);
      printf ("%d \n",decomp);
    fscanf (fp,"%d %*S \n",&sample);
      printf ("%d \n",sample);
    fscanf (fp,"%d %*S \n",&ser_data);
      printf ("%d \n",ser_data);
    fscanf (fp,"%d %*S \n",&take_hann);
      printf ("%d \n",take_hann);
    fscanf (fp,"%d %*S \n",&overlap);
      printf ("%d \n",overlap);
      update=update/overlap;
    fscanf (fp,"%d %*S \n",&time);
      printf ("%d \n",time);
    fscanf (fp,"%d %*S \n",&nproc);
      printf ("%d \n",nproc);

/******************************************************************************
*                                                                             *
*   set up geometries on CM                                                   *
*                                                                             *
******************************************************************************/

  for (i = 0; i < 2; i++) {
        axe_desc[i].weight = (1-i)*10;
        axe_desc[i].ordering = CM_send_order;
        axe_desc[i].off_chip_bits = 0;
        axe_desc[i].off_chip_pos = 0; 
        axe_desc[i].on_chip_bits = 0;
        axe_desc[i].on_chip_pos = 0;
        axe_desc_t[i] = &(axe_desc[i]);
      }
  copy=nproc/(pts/2);
  axe_desc[1].length = copy;
  axe_desc[0].length = pts/2;
  send_geometry = CM_create_detailed_geometry((unsigned *)(&axe_desc_t[0]),2);
  axe_desc[0].ordering = CM_news_order;
  axe_desc[1].ordering = CM_news_order;
  axe_desc[1].length = copy*ncut;
  axe_desc[0].length = pts/2/ncut;
  news_geometry = CM_create_detailed_geometry((unsigned *)(&axe_desc_t[0]),2);
  new_vp_set = CM_allocate_vp_set(send_geometry);
  CM_set_vp_set(new_vp_set);
  CM_set_context();
  out_channels=(saved_channels/copy+ncut-1)/ncut;

/******************************************************************************
*                                                                             *
*  Allocate FCM memory                                                        *
*                                                                             *
******************************************************************************/

  for (i=0; i<n_beams;i++) {
    abf_table[i]=CM_allocate_stack_field(64*channels_per_cluster 
                * clusters/copy);
    beam_table[i]=CM_allocate_stack_field(64*channels_per_cluster 
                * clusters/copy);
   }
  cross_spect=CM_allocate_stack_field(
        64*channels_per_cluster_2*(channels_per_cluster_2+1)/2);

/******************************************************************************
*                                                                             *
* Open DataVault files for input                                              *
*                                                                             *
******************************************************************************/

  if ((dv_infile = CMFS_open("cross.out", CMFS_O_RDONLY, 0644,
      CM_physical_processors_limit,CM_user_number_of_processors_limit)) == -1){
            printf("error on dv file open: cross.out\n");
            exit(1);
      }
  
/******************************************************************************
*                                                                             *
*  Initialize tables, CM memory, and input buffers                            *
*                                                                             *
******************************************************************************/
    for (i=0;i<n_beams;i++) {
      BEAMTABLEAPT(beam_file_name,beam_table[i],channels_per_cluster_1,
        channels_per_cluster_2,clusters/copy,i,n_beams,&cnst,dv_cal_file_name);
      CM_u_move_1L(abf_table[i],beam_table[i],64*channels_per_cluster);
     }
    if (time) {for (i=0;i<64;i++) CM_timer_clear(i);}
   INITDATA(cross_spect,64*channels_per_cluster_2*(channels_per_cluster_2+1)/2,
              sample/4,0,dv_infile);


/******************************************************************************
*                                                                             *
*  Calculate new adaptive beam weights                                        *
*                                                                             *
******************************************************************************/

      if (time) CM_timer_start(time_abf);
    my_address=CM_allocate_stack_field(16);
    CM_u_move_zero_1L(my_address,16);
    sa=CM_allocate_stack_field(16);
    CM_u_move_zero_1L(sa,16);
    CM_my_send_address_1L (sa,16);
    CM_extract_news_coordinate_1L(send_geometry,my_address,0,sa,16);
    CM_u_gt_constant_1L (my_address,100,16);
    CM_logand_context_with_test();
    CM_u_lt_constant_1L (my_address,1500,16);
    CM_logand_context_with_test();
      channel=0;
       ABFWEIGHTS (cross_spect,beam_table[channel]+64*channels_per_cluster_1,
         channel,channels_per_cluster_2,
         abf_table[channel]+64*channels_per_cluster_1);
        for (i=channels_per_cluster_1;i<channels_per_cluster;i++){
         CMULT2(abf_table[channel]+64*i,beam_table[channel]+64*i);
         }
     if (time) CM_timer_stop(time_abf);

  for (i=channels_per_cluster_1;i<channels_per_cluster;i++) 
    wrc (abf_table[0]+i*64,510,512,23,8);
  print ("CM time to calculate abf beamtables ");
  CM_timer_print(time_abf);
    fflush(stdout);	      

}
