/******************************************************************************
*                                                                             *
*  TRUNCFREQ - Move Spectrum data from input slice to output slice in CM      *
*              truncating spectrum if requested. Geometries for input and     *
*              output slices must be 2D                                       *
*                                                                             *
*      out    -   Slice containing output data                                *
*      in     -   Slice containing input data                                 *
*      size    -  Number of bits in a spectrum                                *
*      p       -  Truncation factor                                           *
*      in_geo  -  CM geometry of input slice                                  *
*      out_geo -  CM geometry of output slice                                 *
*                                                                             *
******************************************************************************/

#include <cm/paris.h>
#include <math.h>

void TRUNCFREQBB (out,in,count,size,p,in_geo, out_geo)
unsigned in,out;
int count,size,p;
CM_geometry_id_t in_geo,out_geo;

{

  unsigned i, n, pcal, name, save_context_flag;
  unsigned temp;
  unsigned x,y;
  unsigned dec= 1 << p;
  int ioff,imod;
  int x_in,x_out,y_in,y_out;

  x_in=CM_geometry_coordinate_length(in_geo, 0);
  x_out=CM_geometry_coordinate_length(out_geo, 0);
  y_in=CM_geometry_coordinate_length(in_geo, 1);
  y_out=CM_geometry_coordinate_length(out_geo, 1);
  n=1<<x_in;
  save_context_flag = CM_allocate_stack_field(1);
  CM_store_context(save_context_flag);
  name = CM_allocate_stack_field(x_in+y_in);
  x=CM_allocate_stack_field(x_in);
  y=CM_allocate_stack_field(y_out);
  CM_my_news_coordinate_1L(x, 0, x_in);
  CM_my_news_coordinate_1L(y, 1, y_in);
  CM_u_move_zero_1L (name,x_in+y_in);

  if ( p != 0) {
/******************************************************************************
*                                                                             *
*  Move spectrum truncating length to 1/dec of original                       *
*                                                                             *
******************************************************************************/
   temp=CM_allocate_stack_field(size);
   CM_make_news_coordinate_1L (out_geo,name,0,x,x_out);
   CM_u_move_constant_1L (y+y_in,count,p);
   CM_deposit_news_coordinate_1L (out_geo,name,1,y,y_out);
   CM_u_lt_constant_1L (x,n/dec,x_in);
   CM_logand_context_with_test();
   CM_u_move_1L (temp,in,size);
   CM_send_1L ( out+ioff*size,name,temp,size,CM_no_field);
   CM_load_context(save_context_flag);
  } else  {
/******************************************************************************
*                                                                             *
*  Move spectrum without truncation                                           *
*                                                                             *
******************************************************************************/
      temp=CM_allocate_stack_field(size);
      CM_make_news_coordinate_1L (out_geo,name,0,x,x_out);
      CM_deposit_news_coordinate_1L (out_geo,name,1,y,y_out);
      CM_u_move_1L (temp,in,size);
      CM_send_1L ( out,name,temp,size,CM_no_field);
  }

  CM_deallocate_stack_through(save_context_flag);
}


