/******************************************************************************
*                                                                             *
*     TSHADE -  Calculate the Taylor shading coefficieents for an array       *
*               Based in T. T. Taylor, Design of Line-Source Antennas for     *
*               Narrow Beamwidth and Low Side Lobes," IRE Transactions on     *
*               Antennas and Propagation, Jan 1955, pp16-28                   *
*                                                                             *
*       x      -  Locations of phones in array                                *
*       w      -  Taylor shading weights                                      *
*       n      -  Number of phones in the array                               *
*       nbar   -  Taylor Sidelobe parameter                                   *
*       deltar -  Mainbeam/Sidelobe ratio (db)                                *
*                                                                             *
******************************************************************************/
#include <math.h>

void TSHADE(x,w,n,nbar,deltar)
float x[64],w[64],deltar;
int n,nbar;

{
  float f[64],r,a,xlen;
  float temp,xold1,xold2,tbar;
  float sigma,sigsq,fmax,ccc;
  float con;
  int c1,c2;
  int i,j;

  tbar=nbar;
  r=pow(10.,deltar/20.);
  a = (log (r+sqrt(r*r-1)))/3.1415926; 
  for (i=n-1; i>=0; i -=1) x[i] -= x[0];
  xlen = 1.5*(x[n-1]-x[0])+0.5*(x[1]-x[n-2]);
  xold1 = x[0];
  xold2 = x[1];
  x[0]= 0.5*(x[1]-x[0]-xlen);
  for (i=2; i< n; i++) {
    x[i-1]=x[i-2]+xold2-xold1;
    xold1=xold2;
    xold2=x[i];
  }
  x[n-1]=x[n-2]+xold2-xold1;

  for (i=0; i<nbar; i++) {
    sigma=nbar/sqrt(a*a+(nbar-.5)*(nbar-.5));
    sigsq=sigma*sigma;
    con=(ngamma(nbar));
    con=con*con;
    f[i]=1.;
    if ( i != 0) {
      c1= nbar+i;
      c2= nbar-i;
      f[i]  = con/(ngamma(c1)*ngamma(c2));
      for (j=1; j<nbar; j++) {
        f[i] *= (1.- (i*i)/(sigsq*(a*a+(j-.5)*(j-.5))));
   }
     }
  }

   fmax=0.;
   for (i=0; i< n; i++) {
     ccc=2*3.1415926*x[i]/xlen; 
     w[i]=f[0];
     for (j=1; j< nbar; j++ ) w[i] += 2*f[j]*cos (j*ccc);
     if ( fmax < w[i] ) fmax=w[i];
    }
   for (i=0; i< n; i++) w[i] = w[i]/fmax;
}

ngamma(n)
   int n;

{
  int i;
  float p;
  p=1.;
  for (i=1;i<n;i++) p *= i;
  return(p);
}

