/******************************************************************************
*                                                                             *
*  visbbsub.c - Generates standard Lofargram data and broadband correlations  *
*               for a set of time series data. The order of processing is     *
*                                                                             *
*           1. Read raw time-series data from DataVault                       *
*           2. Retrieve and Bitreverse sweep of data to be processed          *
*           3. Convert data to floating point                                 *
*           4. Window the data                                                *
*           5. FFT the data                                                   *
*           6. Beamform the data                                              *
*           7. Compute amplitude                                              *
*           8. Noise Spectrun Equalize outputs                                *
*           9. Clip and discretize output to 8 bits                           *
*          10. SCOT window the spectrums                                      *
*          11. Bandpass filter the spectrums                                  *
*          12. Center the correlations                                        *
*          13. Cross multiple the spectrums                                   *
*          14. Inverse FFT to calculate correlations                          *
*          15. Normalize the correlations                                     *
*          16. Merge the correlations from a line array                       *
*          17. Discretize the correations                                     *
*                                                                             *
******************************************************************************/

#define TRUE          1
#define FALSE         0
#define DVFILE int

#include <sys/types.h>
#include <cm/cm.h>
#include <cm/cm_stat.h>
#include <cm/cm_file.h>
#include <cm/cmfs.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <cm/paris.h>
#include <cm/cmssl-paris.h>

void VISBBSUB(fft_amp,bb_amp,do_vis,do_bb)
unsigned *fft_amp,*bb_amp;
int do_vis,do_bb;
{
/******************************************************************************
*                                                                             *
*   Set-up variables                                                          *
*                                                                             *
******************************************************************************/
  static iflag=TRUE;
  static char dv_in_file_name[128], beam_file_name[128];
  static char param_file_name[128], dv_cal_file_name[128];
  static char dv_gfilt_file_name[128];
  static DVFILE dv_infile, dv_gfiltfile;
  static u_char *grid;
  static int number;
  static int aver,save_aver;
  static int pts, ptr, step, outfile_flag = 0;
  static int copy,np,nproc;
  static float cnst,dstep,sigma;
  static int show_fb = FALSE, take_hann = FALSE,
            decomp=FALSE, int_data=FALSE,
            do_overlap = FALSE, save_grid = TRUE, ser_data=FALSE,
            spoke_supp=FALSE,take_scot= FALSE;
  static int scot_flag=0, take_gfilt=FALSE;
  static int offset=0,update;
  static int noff,ndisp=8;
  static int sample=32;
  static int field_idno=0;
  static int sweep_count=0;
  static int debu, dbu;
  static float te[128]; 
  static int n_beams,sweeps_in,clusters;
  static int p,channels_per_cluster_1,channels_per_cluster_2;
  static int channels_per_cluster;
  static int channels,overlap,fft_channels,bb_channels;
  static float alpha=0.,alpha_in,xpts;
  static int cut,ncut,out_channels;

  static int time=FALSE;

  static FILE *fopen(), *fp;
  static struct cm_stat instat;

  static unsigned  pure_data, old_data, new_data, beam_table[128], cmte,
          window, fft_aver, fft_temp, fft_temp_bb, amplitude, signed_amplitude,
          ali, beam_real, temp;
  static unsigned  gfilt, x, y, xamp, yamp, wx, wy,ft_bbamp,bb_aver,
            spoke_temp,xfl, yfl, xyfl, out_amp, bb_out, gtemp;

  static CM_geometry_id_t send_geometry,send_geometry_bb,news_geometry;
  static  unsigned dimension_array[2], new_vp_set;
  static  CMSSL_fft_setup_t id,idinv;
  static  CMSSL_op_t ops[2];
  static CMSSL_bit_order_t sbo[2],dbo[2];
  static CMSSL_scale_t sca[2];
  static CMSSL_op_t opscor[2];
  static CMSSL_bit_order_t sbocor[2],dbocor[2];
  static CMSSL_scale_t scacor[2];
  static struct CM_axis_descriptor axe_desc[32];
  static CM_axis_descriptor_t axe_desc_t[32];
  static int channel, bits, odd, beam, table_index, i;
  static int mv_offset;
  static int start[2],stop[2];
  static int out_bb_channels,ncut_bb,cut_bb;

  start[0]=512;start[1]=0;stop[0]=516;stop[1]=0;

/******************************************************************************
*                                                                             *
*   Initialize CM and file names                                              *
*                                                                             *
******************************************************************************/
  if ( iflag ==1) {
    iflag=0;
    dv_cal_file_name[0] = '\0';
    strcpy (param_file_name,"/s4/wanderson/APT/NEATS/visbbnew_param");
    strcpy (beam_file_name,"/s4/wanderson/APT/NEATS/param1");
    strcpy (dv_in_file_name,"tape95.1");

/******************************************************************************
*                                                                             *
*   Get parameters from front end file                                        *
*                                                                             *
******************************************************************************/

    fp=fopen (param_file_name,"r");
    if (fp == NULL) { fprintf (stderr,"Couldn't open file \n"); exit(0);}
      fscanf (fp,"%d %*S \n",&p);
        pts=1<<p;
        printf ("FFT size = %d \n",pts);
        update=pts/2;
      fscanf (fp,"%d %*S \n",&clusters);
        printf ("Number of clusters = %d \n",clusters);
      fscanf (fp,"%d %*S \n",&channels_per_cluster_1);
        printf ("Number of phones in 1st sub-array = %d \n",
           channels_per_cluster_1);
      fscanf (fp,"%d %*S \n",&channels_per_cluster_2);
        printf ("Number of phines in 2nd sub-array = %d \n",
           channels_per_cluster_2);
        channels_per_cluster=channels_per_cluster_1+channels_per_cluster_2;
      fscanf (fp,"%d %*S \n",&n_beams);
        printf ("Number of beams = %d \n",n_beams);
      fscanf (fp,"%d %*S \n",&sweeps_in);
        printf ("Number of sweeps to read = %d \n",sweeps_in);
      fscanf (fp,"%d %*S \n",&aver);
        save_aver=aver;
        printf ("Number of output sweeps to average = %d \n",aver);
      fscanf (fp,"%d %*S \n",&int_data);
        printf ("if 1 data is integer %d \n",int_data);
      fscanf (fp,"%d %*S \n",&decomp);
        printf ("if 1 data is companded %d \n",decomp);
      fscanf (fp,"%d %*S \n",&sample);
        printf ("Number of bits in input sample = %d \n",sample);
      fscanf (fp,"%d %*S \n",&ser_data);
        printf ("if 1 data is serial on disk %d \n",ser_data);
      fscanf (fp,"%d %*S \n",&take_hann);
        printf ("if 1 Hanning window data%d \n",take_hann);
      fscanf (fp,"%d %*S \n",&overlap);
        printf ("Overlap factor = %d \n",overlap);
        update=update/overlap;
      fscanf (fp,"%d %*S \n",&cut);
        ncut=1<<cut;
        printf ("Factor to truncate spectrum = %d \n",ncut);
      fscanf (fp,"%d %*S \n",&cut_bb);
        ncut_bb=1<<cut_bb;
        printf ("Factor to truncate broadband correlation = %d \n",ncut_bb);
      fscanf (fp,"%S %*S \n", &dv_gfilt_file_name[0]);
        take_gfilt=TRUE;
      fscanf (fp,"%d %*S \n",&take_scot);
        printf ("if 1 SCOT filter spectrum %d \n",take_scot);
      fscanf (fp,"%d %f %*S %*S \n",&spoke_supp, &alpha_in);
        printf ("if 1 suppress spokes by %f %d \n ",alpha_in, spoke_supp);
      fscanf (fp,"%f %*S \n",&sigma);
        printf ("BB background sd = %f \n",sigma);
      fscanf (fp,"%d %*S \n",&time);
        printf ("if 1 time routine %d \n",time);
      fscanf (fp,"%d %*S \n",&nproc);
        printf ("Number of virtual processors = %d \n",nproc);

/******************************************************************************
*                                                                             *
*   set up FFT configuration and geometries on CM                             *
*                                                                             *
******************************************************************************/

    ops[0]=CMSSL_f_xform;
    sbo[0]=CMSSL_bit_reversed;
    dbo[0]=CMSSL_normal;
    sca[0]=CMSSL_noscale;
    ops[1]=CMSSL_nop;
    sbo[1]=CMSSL_normal;
    dbo[1]=CMSSL_normal;
    sca[1]=CMSSL_noscale;
    opscor[0]=CMSSL_i_xform;
    sbocor[0]=CMSSL_normal;
    dbocor[0]=CMSSL_normal;
    scacor[0]=CMSSL_scale_n;
    opscor[1]=CMSSL_nop;
    sbocor[1]=CMSSL_normal;
    dbocor[1]=CMSSL_normal;
    scacor[1]=CMSSL_noscale;
    for (i = 0; i < 2; i++) {
          axe_desc[i].weight = (1-i)*10;
          axe_desc[i].ordering = CM_send_order;
          axe_desc[i].off_chip_bits = 0;
          axe_desc[i].off_chip_pos = 0; 
          axe_desc[i].on_chip_bits = 0;
          axe_desc[i].on_chip_pos = 0;
          axe_desc_t[i] = &(axe_desc[i]);
        }
    copy=nproc/(pts/2);
    fft_channels=n_beams*clusters/copy;
    out_channels=(fft_channels/copy+ncut-1)/ncut;
    out_bb_channels=(fft_channels/copy+ncut_bb-1)/ncut_bb;
    bb_channels=clusters/copy;
    channels=channels_per_cluster*clusters/copy;
    axe_desc[1].length = copy;
    axe_desc[0].length = pts/2;
    send_geometry =CM_create_detailed_geometry((unsigned *)(&axe_desc_t[0]),2);
    axe_desc[1].length = copy*ncut_bb;
    axe_desc[0].length = pts/2/ncut_bb;
    send_geometry_bb =CM_create_detailed_geometry((unsigned *)(&axe_desc_t[0]),2);
    axe_desc[0].ordering = CM_news_order;
    axe_desc[1].ordering = CM_news_order;
    axe_desc[1].length = copy*ncut;
    axe_desc[0].length = pts/2/ncut;
    news_geometry =CM_create_detailed_geometry((unsigned *)(&axe_desc_t[0]),2);
    new_vp_set = CM_allocate_vp_set(send_geometry);
    CM_set_vp_set(new_vp_set);
    CM_set_context();
/******************************************************************************
*                                                                             *
*  Get bandpass filter from front-end file                                    *
*                                                                             *
******************************************************************************/

  gfilt=CM_allocate_stack_field(32);
  gtemp=CM_allocate_stack_field(ncut_bb*32);
  if (take_gfilt) READCMFILE ( dv_gfilt_file_name,gfilt,32);
  for (i=0;i<ncut_bb;i++) CM_u_move_1L(gtemp+i*32,gfilt,32);
  TRUNCFREQ (gfilt,gtemp,ncut_bb,32,cut_bb,
                   send_geometry,send_geometry_bb);

/******************************************************************************
*                                                                             *
*  Allocate FCM memory                                                        *
*                                                                             *
******************************************************************************/

    *fft_amp=CM_allocate_stack_field(8*out_channels);
    *bb_amp=CM_allocate_stack_field(8*bb_channels);
    pure_data=CM_allocate_stack_field(2*sample*channels);
    old_data=CM_allocate_stack_field(2*sample*channels*(sweeps_in+1));
    cmte=CM_allocate_stack_field(4*32);
    window=CM_allocate_stack_field(32*2*2);
    fft_aver=CM_allocate_stack_field(64*fft_channels);    
    fft_temp=CM_allocate_stack_field(64*channels);    
    CM_set_vp_set_geometry(CM_current_vp_set,send_geometry_bb);
    fft_temp_bb=CM_allocate_stack_field(64*fft_channels);    
    CM_set_vp_set_geometry(CM_current_vp_set,send_geometry);
    amplitude=CM_allocate_stack_field(32);   
    signed_amplitude=CM_allocate_stack_field(8);   
    for (i=0; i<n_beams;i++)
      beam_table[i]=CM_allocate_stack_field(64*channels);
    beam_real=CM_allocate_stack_field(64*n_beams* clusters*2/copy);
    temp=CM_allocate_stack_field(32);
    bb_out=CM_allocate_stack_field(64*fft_channels);
    x=CM_allocate_stack_field(64*fft_channels);
    y=CM_allocate_stack_field(64*fft_channels);
    wx=CM_allocate_stack_field(32*fft_channels);
    wy=CM_allocate_stack_field(32*fft_channels);
    xamp=CM_allocate_stack_field(32*fft_channels);
    yamp=CM_allocate_stack_field(32*fft_channels);
    bb_aver=CM_allocate_stack_field(64*fft_channels);
    spoke_temp=CM_allocate_stack_field(32*fft_channels);
    xfl=CM_allocate_stack_field(32*fft_channels);
    yfl=CM_allocate_stack_field(32*fft_channels);
    xyfl=CM_allocate_stack_field(32*fft_channels);
    out_amp=CM_allocate_stack_field(bb_channels);

    id= CMSSL_c_c_fft_setup_1L (fft_temp,23,8);
    CM_set_vp_set_geometry(CM_current_vp_set,send_geometry_bb);
    idinv= CMSSL_c_c_fft_setup_1L (fft_temp_bb,23,8);
    CM_set_vp_set_geometry(CM_current_vp_set,send_geometry);


/******************************************************************************
*                                                                             *
* Open DataVault files for input                                              *
*                                                                             *
******************************************************************************/

  if ((dv_infile = CMFS_open(dv_in_file_name, CMFS_O_RDONLY, 0644,
      CM_physical_processors_limit,CM_user_number_of_processors_limit)) == -1){
            printf("error on dv file open: %s\n", dv_in_file_name);
            exit(1);
      }
  
/******************************************************************************
*                                                                             *
*  Initialize tables, CM memory, and input buffers                            *
*                                                                             *
******************************************************************************/

      if (decomp) {
        for (debu = 0;  debu < 128; debu++) te[debu] = 0;
          COMPUTE_DECOMP_TABLE(te);
          }
      CM_u_move_zero_1L(amplitude,32);
      if (decomp) LOAD_DECOMP_TABLE(te, cmte);
      CM_u_move_zero_1L(amplitude,32);
      CM_u_move_zero_1L(fft_aver,32*fft_channels);
      CM_u_move_zero_1L(bb_aver,32*bb_channels);
      CM_u_move_zero_1L(wx,32*fft_channels);
      CM_u_move_zero_1L(wy,32*fft_channels);
      HANN2(window, pts);
      HAMM2(window+64, pts);
      BITREV2D (window, p-1, 4*32, 0);
      for (i=0;i<n_beams;i++) 
       BEAMTABLE(beam_file_name,beam_table[i],channels_per_cluster_1,
        channels_per_cluster_2,clusters/copy,i,n_beams,&cnst,dv_cal_file_name);
      INITDATA (old_data,2*channels*sample*(sweeps_in+1),
                sample/4,ser_data,dv_infile);
      CM_u_move_1L (pure_data, old_data,2*channels * sample);

  } 
  else {
/******************************************************************************
*                                                                             *
*  Retrieve next sweep of input data                                          *
*                                                                             *
******************************************************************************/
    LOADDATA (pure_data,old_data,2*channels*sample,dv_infile,
               offset,pts/2,sweeps_in,TRUE,sample/4,ser_data);
    if ( offset >= sweeps_in*pts/2) offset += -sweeps_in*pts/2;
    offset += update;

/******************************************************************************
*                                                                             *
*  Float input data and window if required                                    *
*                                                                             *
******************************************************************************/

    if (decomp) {
      for (channel =0; channel < 2*channels; channel++){
	DECOMPAND(fft_temp+32*channel, pure_data+sample*channel, cmte);
       }
      for (channel = 0; channel <2*channels; channel++){
	 CM_f_s_float_2_2L (fft_temp+32*channel,
                     pure_data+sample*channel, sample,23,8);
       }
    }
    else if (int_data){
      for (channel = 0; channel <2*channels; channel++){
	 CM_f_s_float_2_2L (fft_temp+32*channel,
                     pure_data+sample*channel, sample,23,8);
       }
     }

      else {
      for (channel = 0; channel < 2*channels; channel++){
	CM_u_move_1L (fft_temp+32*channel, pure_data+sample*channel, sample);
       }
      }

    if (take_hann != 0) {
      for (channel = 0; channel < 2*channels; channel++){
	  CM_f_multiply_2_1L(fft_temp+32*channel,window,23,8);
          channel ++;
	  CM_f_multiply_2_1L(fft_temp+32*channel,window+32,23,8);
        }
      }

/******************************************************************************
*                                                                             *
*  Peform real-to-complex FFT on time-series data                             *
*                                                                             *
******************************************************************************/

    for (number=0; number< clusters/copy; number++) {
     for (channel = 0; channel < channels_per_cluster; 
                     channel += 1){
        CMSSL_c_c_fft_detailed_a_1L (
                    fft_temp+64*(channel+number*channels_per_cluster),
                    id,ops,sbo,dbo,sca,23,8);
     }
   }

    for (number=0;number< clusters/copy; number++) {
      UNS2REAL2D (
         fft_temp+64*(number*channels_per_cluster), 
           p, channels_per_cluster,0,FALSE); 
    }

/******************************************************************************
*                                                                             *
*  Beamform the clusters                                                      *
*                                                                             *
******************************************************************************/

  if (do_bb) {
    channel = 0;
     for ( number =0; number < clusters/copy ; number ++) {
       for(beam = 0, table_index = 0;beam < n_beams;beam++){
         BEAM (beam_real+64*(beam+number*n_beams),
              fft_temp+64*(number*(channels_per_cluster)),
              beam_table[beam]+64*(table_index),
              channels_per_cluster_1);
        }
     }
  }

    for (number=0; number<clusters/copy;number++) {
      table_index=channels_per_cluster_1+number*channels_per_cluster;      
      for(beam = 0;beam < n_beams;beam++){
           BEAM (beam_real+64*(beam+number*n_beams+fft_channels),
            fft_temp+64*(channels_per_cluster_1+number*(channels_per_cluster)),
            beam_table[beam]+64*table_index,
            channels_per_cluster_2);
       }
    }

/******************************************************************************
*                                                                             *
*  Calculate amplitude of the spectrum                                        *
*                                                                             *
******************************************************************************/
  if ( do_vis)  {
    for( channel = 0 ; channel < fft_channels; channel++){
       TAKE_AMPLITUDE(temp, beam_real+64*(channel+fft_channels));
       CM_f_add_2_1L (fft_aver+32*channel,temp,23,8);
     }

/******************************************************************************
*                                                                             *
*  Noise Sectrum Equalize (NSE) the amplitudes ( reshuffle data among CM      *
*  processing elements to improve efficiency of NSE algorithm, clip and       *
*  discretize to 8-bits                                                       *
*                                                                             *
******************************************************************************/

      if ( aver == 1) {
        TRUNCFREQ (fft_aver,fft_aver,fft_channels,32,cut,
                   send_geometry,news_geometry);
        CM_set_vp_set_geometry(CM_current_vp_set,news_geometry);

        NSE_SAXA(fft_aver, p-1-cut, out_channels, 3, 67);
   
        for( channel = 0 ; channel < out_channels; channel++){
         CLIP(fft_aver+32*channel, 5., 51.);
         CM_s_f_floor_2_2L (signed_amplitude, fft_aver+32*channel, 8, 23, 8);
         CM_u_move_1L (*fft_amp+8*channel,signed_amplitude,8);
         }
        CM_set_vp_set_geometry(CM_current_vp_set,send_geometry);
      }
    }
/******************************************************************************
*                                                                             *
*  SCOT window, bandpass filter, and center subarray spectrums                *
*                                                                             *
******************************************************************************/
   
  if (do_bb) { 
   TRUNCFREQ (beam_real,beam_real,fft_channels,64,cut_bb,
                   send_geometry,send_geometry_bb);
   TRUNCFREQ (beam_real+64*fft_channels,beam_real+64*fft_channels,fft_channels,
               64,cut_bb,send_geometry,send_geometry_bb);
   for ( channel=0; channel< out_bb_channels; channel ++) {
    CM_u_move_1L (x+64*channel,beam_real+64*channel,64);
    CM_u_move_1L (y+64*channel,beam_real+64*channel+64*fft_channels,64);
       }

    if (take_scot ) {
       SCOT (wx,x,out_bb_channels,scot_flag);
       SCOT (wy,y,out_bb_channels,scot_flag);
       scot_flag=1;
     }

     if (take_gfilt) {
       for ( channel=0; channel < 2*out_bb_channels; channel++ ) {
        CM_f_multiply_3_1L (x+32*channel,x+32*channel,gfilt,23,8);
        CM_f_multiply_3_1L (y+32*channel,y+32*channel,gfilt,23,8);
       }
    }

      CM_set_vp_set_geometry(CM_current_vp_set,send_geometry_bb);
      NEGODDC (y,out_bb_channels,0);

/******************************************************************************
*                                                                             *
*  Form cross-spectrum of subarrays                                           *
*                                                                             *
******************************************************************************/

      CONJMULT (fft_temp_bb,x,y,out_bb_channels);

/******************************************************************************
*                                                                             *
*  Calculate broadband correlation using inverse FFT algorithm and normalize  *
*  output to between -1. and 1.                                               *
*                                                                             *
******************************************************************************/

     SCR2REAL2D(fft_temp_bb,  p-cut_bb, out_bb_channels,1,FALSE);
     for (channel = 0; channel < out_bb_channels; channel ++){
        CMSSL_c_c_fft_detailed_a_1L (fft_temp_bb+64*channel,idinv,opscor,sbocor
                    ,dbocor,scacor,23,8);
      }

     for (channel = 0; channel < out_bb_channels; channel ++){
          TAKE_AMPLITUDE_SQ (xamp+32*channel,x+64*channel);
          TAKE_AMPLITUDE_SQ (yamp+32*channel,y+64*channel);
      }
     for (channel = 0; channel < out_bb_channels; channel ++){
         CM_spread_with_f_add_1L ( xfl,xamp+32*channel,0,23,8);
         CM_spread_with_f_add_1L ( yfl,yamp+32*channel,0,23,8);
         CM_f_multiply_3_1L (xyfl+32*channel,xfl,yfl,23,8);
         CM_f_sqrt_1_1L(xyfl+32*channel,23,8);
         xpts=pts/2./ncut_bb;
         CM_f_divide_constant_2_1L (xyfl+32*channel,xpts,23,8);
      }

     for (channel = 0; channel < out_bb_channels; channel ++){
       CM_f_divide_2_1L ( fft_temp_bb+64*channel,xyfl+32*channel,23,8);
       CM_f_divide_2_1L ( fft_temp_bb+32+64*channel,xyfl+32*channel,23,8);
      }

/******************************************************************************
*                                                                             *
*  Merge correlations from beams into single broadband correlogram            *
*                                                                             *
******************************************************************************/

    if (n_beams > 0) {
     for (number=0;number<bb_channels; number ++) {
      BMMACHELL (bb_out+64*(number),
            fft_temp_bb+64*number*n_beams,n_beams,64,cnst,ncut_bb);
        CM_set_context();
      }
       FORMBB(bb_out,bb_channels,p-1-cut_bb);
   }
    CM_set_vp_set_geometry(CM_current_vp_set,send_geometry);

/******************************************************************************
*                                                                             *
*  Suppress spokes in broadband correlation                                   *
*                                                                             *
******************************************************************************/

      for (number=0;number<bb_channels;number ++) 
         CM_f_add_2_1L (bb_aver+32*number,bb_out+32*number,23,8);
      if (aver == 1) {
        for (number=0;number<bb_channels;number ++) 
         CM_f_divide_constant_2_1L (bb_aver+32*number,1.*save_aver,23,8);
        if (spoke_supp) {
        for (number=0;number<bb_channels;number ++) 
             SUPPRESS (bb_aver, spoke_temp,alpha);
          alpha=alpha_in;
        }
       GSMACHELL (*bb_amp,bb_aver,sigma);
       }
      }
/******************************************************************************
*                                                                             *
*  Prepare data for out to DataVault and when output buffer full dump to      *
*  DataVault                                                                  *
*                                                                             *
******************************************************************************/
    aver=aver-1;
    if (aver == 0) {
      aver=save_aver;
      CM_set_vp_set_geometry(CM_current_vp_set,news_geometry);
      NEWSTOSEND (*fft_amp,8*out_channels);
      CM_set_vp_set_geometry(CM_current_vp_set,send_geometry);
      aver = save_aver;
      CM_u_move_zero_1L(bb_aver,32*bb_channels);
      CM_u_move_zero_1L(fft_aver,64*fft_channels);
      }
    }    
}
