/******************************************************************************
*                                                                             *
*  vissub.c - Generates standard Lofargram data for a set of time series data *
*          The order of processing is                                         *
*                                                                             *
*           1. Read raw time-series data from DataVault                       *
*           2. Retrieve and Bitreverse sweep of data to be processed          *
*           3. Convert data to floating point                                 *
*           4. Window the data                                                *
*           5. FFT the data                                                   *
*           6. Beamform the data                                              *
*           7. Compute amplitude                                              *
*           8. Noise Spectrun Equalize outputs                                *
*           9. Clip and discretize output to 8 bits                           *
*                                                                             *
******************************************************************************/

#define TRUE          1
#define FALSE         0
#define DVFILE int

#include <sys/types.h>
#include <cm/cm.h>
#include <cm/cm_stat.h>
#include <cm/cm_file.h>
#include <cm/cmfs.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <cm/paris.h>
#include <cm/cmssl-paris.h>

void VISSUB(fft_amp)
unsigned *fft_amp;
{
/******************************************************************************
*                                                                             *
*   Set-up variables                                                          *
*                                                                             *
******************************************************************************/
  static iflag=TRUE;
  static char dv_in_file_name[128], dv_out_file_name[128],beam_file_name[128];
  static char param_file_name[128], dv_cal_file_name[128];
  static DVFILE dv_infile, dv_outfile;
  static u_char *grid;
  static int oddsteps,number;
  static int aver,save_aver;
  static int pts, steps, ptr, step, outfile_flag = 0;
  static int copy,np,nproc;
  static float cnst,dstep;
  static int show_fb = FALSE, take_hann = FALSE,
            decomp=FALSE, int_data=FALSE,
            do_overlap = FALSE, save_grid = TRUE, ser_data=FALSE;
  static int offset=0,update;
  static int noff,ndisp=8;
  static int sample=32;
  static int field_idno=0;
  static int sweep_count=0;
  static int debu, dbu;
  static int ncut,out_channels;
  static float te[128]; 
  static int n_beams,sweeps_in,sweeps_out,cut,clusters;
  static int p,channels_per_cluster_1,channels_per_cluster_2;
  static int channels_per_cluster;
  static int channels,overlap,saved_channels;

  static int time=FALSE;

  static FILE *fopen(), *fp;
  static struct cm_stat instat;

  static unsigned  pure_data, old_data, new_data, beam_table[128], cmte,
            window, fft_aver, fft_temp, amplitude, signed_amplitude,
            ali, beam_real, temp;

  static CM_geometry_id_t send_geometry,news_geometry;
  static  unsigned dimension_array[2], new_vp_set;
  static  CMSSL_fft_setup_t id;
  static  CMSSL_op_t ops[2];
  static CMSSL_bit_order_t sbo[2],dbo[2];
  static CMSSL_scale_t sca[2];
  static CMSSL_op_t opscor[2];
  static CMSSL_bit_order_t sbocor[2],dbocor[2];
  static CMSSL_scale_t scacor[2];
  static struct CM_axis_descriptor axe_desc[32];
  static CM_axis_descriptor_t axe_desc_t[32];
  static int channel, bits, odd, beam, table_index, i;
  static int mv_offset;

/******************************************************************************
*                                                                             *
*   Initialize CM and file names                                              *
*                                                                             *
******************************************************************************/
  if ( iflag ==1) {
    iflag=0;
    dv_cal_file_name[0] = '\0';
    strcpy (param_file_name,"/s4/wanderson/APT/F059/vis_param");
    strcpy (beam_file_name,"/s4/wanderson/APT/F059/param");
    strcpy (dv_in_file_name,"time");

/******************************************************************************
*                                                                             *
*   Get parameters from front end file                                        *
*                                                                             *
******************************************************************************/

    fp=fopen (param_file_name,"r");
    if (fp == NULL) { fprintf (stderr,"Couldn't open file \n"); exit(0);}
      fscanf (fp,"%d %*S \n",&p);
        pts=1<<p;
        printf ("%d \n",pts);
        update=pts/2;
      fscanf (fp,"%d %*S \n",&clusters);
        printf ("%d \n",clusters);
      fscanf (fp,"%d %*S \n",&channels_per_cluster_1);
        printf ("%d \n",channels_per_cluster_1);
      fscanf (fp,"%d %*S \n",&channels_per_cluster_2);
        printf ("%d \n",channels_per_cluster_2);
        channels_per_cluster=channels_per_cluster_1+channels_per_cluster_2;
        channels=channels_per_cluster*clusters;
      fscanf (fp,"%d %*S \n",&n_beams);
        printf ("%d \n",n_beams);
        saved_channels=n_beams*clusters;
      fscanf (fp,"%d %*S \n",&sweeps_in);
        printf ("%d \n",sweeps_in);
      fscanf (fp,"%d %*S \n",&sweeps_out);
        printf ("%d \n",sweeps_out);
      fscanf (fp,"%d %*S \n",&cut);
        printf ("%d \n",cut);
        ncut=1<<cut;
      fscanf (fp,"%d %*S \n",&steps);
        printf ("%d \n",steps);
      fscanf (fp,"%d %*S \n",&aver);
        printf ("%d \n",aver);
        save_aver = aver;
      fscanf (fp,"%d %*S \n",&int_data);
        printf ("%d \n",int_data);
      fscanf (fp,"%d %*S \n",&decomp);
        printf ("%d \n",decomp);
      fscanf (fp,"%d %*S \n",&sample);
        printf ("%d \n",sample);
      fscanf (fp,"%d %*S \n",&ser_data);
        printf ("%d \n",ser_data);
      fscanf (fp,"%d %*S \n",&take_hann);
        printf ("%d \n",take_hann);
      fscanf (fp,"%d %*S \n",&overlap);
        printf ("%d \n",overlap);
        update=update/overlap;
      fscanf (fp,"%d %*S \n",&time);
        printf ("%d \n",time);
      fscanf (fp,"%d %*S \n",&nproc);
        printf ("%d \n",nproc);
/******************************************************************************
*                                                                             *
*   set up FFT configuration and geometries on CM                             *
*                                                                             *
******************************************************************************/

    ops[0]=CMSSL_f_xform;
    sbo[0]=CMSSL_bit_reversed;
    dbo[0]=CMSSL_normal;
    sca[0]=CMSSL_noscale;
    ops[1]=CMSSL_nop;
    sbo[1]=CMSSL_normal;
    dbo[1]=CMSSL_normal;
    sca[1]=CMSSL_noscale;
    for (i = 0; i < 2; i++) {
          axe_desc[i].weight = (1-i)*10;
          axe_desc[i].ordering = CM_send_order;
          axe_desc[i].off_chip_bits = 0;
          axe_desc[i].off_chip_pos = 0; 
          axe_desc[i].on_chip_bits = 0;
          axe_desc[i].on_chip_pos = 0;
          axe_desc_t[i] = &(axe_desc[i]);
        }
    copy=nproc/(pts/2);
    axe_desc[1].length = copy;
    axe_desc[0].length = pts/2;
    send_geometry =CM_create_detailed_geometry((unsigned *)(&axe_desc_t[0]),2);
    axe_desc[0].ordering = CM_news_order;
    axe_desc[1].ordering = CM_news_order;
    axe_desc[1].length = copy*ncut;
    axe_desc[0].length = pts/2/ncut;
    news_geometry =CM_create_detailed_geometry((unsigned *)(&axe_desc_t[0]),2);
    new_vp_set = CM_allocate_vp_set(send_geometry);
    CM_set_vp_set(new_vp_set);
    CM_set_context();
    out_channels=(saved_channels/copy+ncut-1)/ncut;

/******************************************************************************
*                                                                             *
*  Allocate FCM memory                                                        *
*                                                                             *
******************************************************************************/

 printf ("%d \n",*fft_amp);
    *fft_amp=CM_allocate_stack_field(8*out_channels);
 printf ("%d \n",*fft_amp);
    pure_data=CM_allocate_stack_field(2*sample*channels/copy);
    old_data=CM_allocate_stack_field(2*sample*channels*(sweeps_in+1)/copy);
    cmte=CM_allocate_stack_field(4*32);
    window=CM_allocate_stack_field(32*2*2);
    fft_aver=CM_allocate_stack_field(32*saved_channels/copy);    
    fft_temp=CM_allocate_stack_field(64*channels/copy);    
    amplitude=CM_allocate_stack_field(32);   
    signed_amplitude=CM_allocate_stack_field(8);   
    ali=CM_allocate_stack_field(32*saved_channels/copy);
    for (i=0; i<n_beams;i++)
      beam_table[i]=CM_allocate_stack_field(64*channels_per_cluster 
                  * clusters/copy);
    beam_real=CM_allocate_stack_field(64*n_beams* clusters/copy);
    temp=CM_allocate_stack_field(8);

    id= CMSSL_c_c_fft_setup_1L (fft_temp,23,8);


/******************************************************************************
*                                                                             *
* Open DataVault files for input                                              *
*                                                                             *
******************************************************************************/

  if ((dv_infile = CMFS_open(dv_in_file_name, CMFS_O_RDONLY, 0644,
      CM_physical_processors_limit,CM_user_number_of_processors_limit)) == -1){
            printf("error on dv file open: %s\n", dv_in_file_name);
            exit(1);
      }
  
/******************************************************************************
*                                                                             *
*  Initialize tables, CM memory, and input buffers                            *
*                                                                             *
******************************************************************************/

      if (decomp) {
        for (debu = 0;  debu < 128; debu++) te[debu] = 0;
          COMPUTE_DECOMP_TABLE(te);
          }
      CM_u_move_zero_1L(amplitude,32);
      if (decomp) LOAD_DECOMP_TABLE(te, cmte);
      CM_u_move_zero_1L(fft_aver,32*saved_channels/copy);
      CM_u_move_zero_1L(ali,32*saved_channels/copy);
      HANN2(window, pts);
      HAMM2(window+64, pts);
      BITREV2D (window, p-1, 4*32, 0);
      for (i=0;i<n_beams;i++) 
       BEAMTABLE(beam_file_name,beam_table[i],channels_per_cluster_1,
        channels_per_cluster_2,clusters/copy,i,n_beams,&cnst,dv_cal_file_name);
        printf ("%d \n",2*channels*sample*(sweeps_in+1)/copy);
      INITDATA (old_data,2*channels*sample*(sweeps_in+1)/copy,
                sample/4,ser_data,dv_infile);
      CM_u_move_1L (pure_data, old_data,2*channels * sample/copy );
      oddsteps=0;
  } 
  else {
/******************************************************************************
*                                                                             *
*  Retrieve next sweep of input data                                          *
*                                                                             *
******************************************************************************/
    LOADDATA (pure_data,old_data,2*channels*sample/copy,dv_infile,
               offset,pts/2,sweeps_in,TRUE,sample/4,ser_data);
    if ( offset >= sweeps_in*pts/2) offset += -sweeps_in*pts/2;
    offset += update;

/******************************************************************************
*                                                                             *
*  Float input data and window if required                                    *
*                                                                             *
******************************************************************************/

    if (decomp) {
      for (channel =0; channel < 2*channels/copy; channel++){
	DECOMPAND(fft_temp+32*channel, pure_data+sample*channel, cmte);
       }
      for (channel = 0; channel <2*channels/copy; channel++){
	 CM_f_s_float_2_2L (fft_temp+32*channel,
                     pure_data+sample*channel, sample,23,8);
       }
    }
    else if (int_data){
      for (channel = 0; channel <2*channels/copy; channel++){
	 CM_f_s_float_2_2L (fft_temp+32*channel,
                     pure_data+sample*channel, sample,23,8);
       }
     }

      else {
      for (channel = 0; channel < 2*channels/copy; channel++){
	CM_u_move_1L (fft_temp+32*channel, pure_data+sample*channel, sample);
       }
      }

    if (take_hann != 0) {
      for (channel = 0; channel < 2*channels/copy; channel++){
	  CM_f_multiply_2_1L(fft_temp+32*channel,window,23,8);
          channel ++;
	  CM_f_multiply_2_1L(fft_temp+32*channel,window+32,23,8);
        }
      }

/******************************************************************************
*                                                                             *
*  Peform real-to-complex FFT on time-series data                             *
*                                                                             *
******************************************************************************/

    for (number=0; number< clusters/copy; number++) {
     for (channel = channels_per_cluster_1; channel < channels_per_cluster; 
                     channel += 1){
        CMSSL_c_c_fft_detailed_a_1L (
                    fft_temp+64*(channel+number*channels_per_cluster),
                    id,ops,sbo,dbo,sca,23,8);
     }
   }

    for (number=0;number< clusters/copy; number++) {
      UNS2REAL2D (
         fft_temp+64*(channels_per_cluster_1+number*channels_per_cluster), 
           p, channels_per_cluster_2,0,FALSE); 
    }

/******************************************************************************
*                                                                             *
*  Beamform the clusters                                                      *
*                                                                             *
******************************************************************************/

    channel = 0;
    for (number=0; number<clusters/copy;number++) {
      table_index=channels_per_cluster_1+number*channels_per_cluster;      
      for(beam = 0;beam < n_beams;beam++){
           BEAM (beam_real+64*(beam+number*n_beams),
            fft_temp+64*(channels_per_cluster_1+number*(channels_per_cluster)),
            beam_table[beam]+64*table_index,
            channels_per_cluster_2);
       }
    }

/******************************************************************************
*                                                                             *
*  Calculate amplitude of the spectrum                                        *
*                                                                             *
******************************************************************************/

    for( channel = 0 ; channel < saved_channels/copy; channel++){
       TAKE_AMPLITUDE(beam_real+64*channel, beam_real+64*channel);
       CM_f_add_2_1L (fft_aver+32*channel,beam_real+64*channel,23,8);
     }

/******************************************************************************
*                                                                             *
*  Noise Sectrum Equalize (NSE) the amplitudes ( reshuffle data among CM      *
*  processing elements to improve efficiency of NSE algorithm, clip and       *
*  discretize to 8-bits                                                       *
*                                                                             *
******************************************************************************/

      aver=aver - 1 ;
      if ( aver == 0) {
        TRUNCFREQ (fft_aver,fft_aver,saved_channels/copy,32,cut,
                   send_geometry,news_geometry);
        CM_set_vp_set_geometry(CM_current_vp_set,news_geometry);

        NSE_SAXA(fft_aver, p-1-cut, out_channels, 3, 67);
   
        for( channel = 0 ; channel < out_channels; channel++){
         CLIP(fft_aver+32*channel, 5., 51.);
         CM_s_f_floor_2_2L (signed_amplitude, fft_aver+32*channel, 8, 23, 8);
         CM_u_move_1L (*fft_amp+8*channel,signed_amplitude,8);
         CM_u_add_3_3L (ali+32*channel,ali+32*channel,*fft_amp+8*channel,
             32,32,8);
         }

/******************************************************************************
*                                                                             *
*  Prepare data for out to DataVAult and when output buffer full dump to      *
*  DataVault                                                                  *
*                                                                             *
******************************************************************************/

      NEWSTOSEND (*fft_amp,8*out_channels);
      CM_set_vp_set_geometry(CM_current_vp_set,send_geometry);
      aver = save_aver;
      CM_u_move_zero_1L(fft_aver,32*saved_channels/copy);
      }
    }
 }
    
