#include <geodata_private.h>
#include <stdio.h>
#include <ci/library.h>

/* This module demonstrates the use of front end data sets with geodata.
   Front end arrays can be tied to a Geodata data set descriptor and
   are automatically copied to the CM when the data is referenced. */

/*  Global Constants  */
#define LON -77.03
#define LAT 38.86
#define ALT 1.3

#define SIDE_1 2.0
#define SIDE_2 1.0

#define RTD 57.29578
#define DTR 0.017453293

typedef struct {
  float lat1,lon1,alt1;
  float lat2,lon2,alt2;
  float val;
} vector;

generate_box(box)
   vector *box;
{
  int n,k;
  float side;

  for(n=0;n<4;n++) {
    box[n].lat1= n%2 ? LAT-SIDE_1 : LAT+SIDE_1;
    box[n].lon1= n%2 ? LON-SIDE_1 : LON+SIDE_1;
    box[n].lat2= n>1 ? LAT-SIDE_1 : LAT+SIDE_1;
    box[n].lon2= n>1 ? LON+SIDE_1 : LON-SIDE_1;
    box[n].val=1.0;    
    box[n].alt1=box[n].alt2=0.0;    
  }
  for(;n<8;n++) {
    box[n].lat1= n%2 ? LAT-SIDE_2 : LAT+SIDE_2;
    box[n].lon1= n%2 ? LON-SIDE_2 : LON+SIDE_2;
    box[n].lat2= n>5 ? LAT-SIDE_2 : LAT+SIDE_2;
    box[n].lon2= n>5 ? LON+SIDE_2 : LON-SIDE_2;
    box[n].val=1.0;    
    box[n].alt1=box[n].alt2=0.0;    
  }
}


main(argc,argv)
     int argc;
     char **argv;
{
  GB_id scene,display;
  GB_id box_data_id;
  GI_proj_desc *proj;
  
  float lat,lon,alt,zoom;
  int i,box_shape[1];
  vector box[8];
  
  CM_init();
  if (argc == 1)
    GI_init("sample_3.gl");
  else
    GI_init(argv[1]);
  
  /* get the id for the scene */
  scene=GI_get_scene_id("sample_3_scene");

  /* set up the vectors describing the two boxes */
  generate_box(box);
  box_shape[0]=8;

  /* get the data set descriptor for the front_end data */
  box_data_id = GI_get_data_id ("sample_3_data");
  GI_set_fe_data (box_data_id, box, 1, box_shape );
    
  /* get the display specified in the sample_3.gl file  */
  display=GI_get_display_id("sample_3_display");

  /* render the scene, and display it */
  GI_update_scene(scene);
  GI_display_scene(display,scene,0,0);
    
  /********************************************************************/
  /*   Part II    enter a loop                                        */
  /********************************************************************/
    
  lat=0.0;
  lon=0.0;
  alt=5.0;
  zoom=0.85;
  
  for(i=0;i<30;i++) {
    proj=GI_globe_view(lat*DTR, lon*DTR, alt,GD_EARTH_RADII, 0.0, 0.0);
    GI_set_scene_projection(scene, proj);
    
    GI_update_scene(scene);
    GI_display_scene(display,scene,0,0);
    
    lon = LON + (lon - LON) * zoom;
    alt = ALT + (alt-ALT) * zoom;
    lat = LAT + (lat-LAT) * zoom;
  }
}
