/*   File Header   ---------------------------------
     File Name: projections.c
     Author and Date: Eric Hoffman 2/22/90
     Description: front ends to projection functions
     End Header   ---------------------------------- */

#include <geodata.h>
#include <geodata_private.h>

extern void GD_miller();
extern void GD_mollweide();
extern void GD_lambert();
extern void GD_polar_stereo();
extern void GD_gnomonic();
extern void GD_globe_view();
extern void GD_general_perspective();

extern void GD_inverse_miller();
extern void GD_inverse_mollweide();
extern void GD_inverse_lambert();
extern void GD_inverse_polar_stereo();
extern void GD_inverse_gnomonic();
extern void GD_inverse_globe_view();
extern void GD_inverse_general_perspective();

extern void GD_fe_miller();
extern void GD_fe_mollweide();
extern void GD_fe_lambert();
extern void GD_fe_polar_stereo();
extern void GD_fe_gnomonic();
extern void GD_fe_globe_view();
extern void GD_fe_general_perspective();

extern void GD_fe_inverse_miller();
extern void GD_fe_inverse_mollweide();
extern void GD_fe_inverse_lambert();
extern void GD_fe_inverse_polar_stereo();
extern void GD_fe_inverse_gnomonic();
extern void GD_fe_inverse_globe_view();
extern void GD_fe_inverse_general_perspective();

extern void GD_miller_border();
extern void GD_mollweide_border();
extern void GD_polar_stereo_border();
extern void GD_globe_view_border();


void GD_par_useless_stub(data,success,par)
     CM_field_id_t data,success;
     float *par;
{
  CM_u_move_constant_1L (success,1,1);
}

void GD_border_par_useless_stub(data,param)
{
  CM_clear_context();
}

void GD_ser_useless_stub(data,success,par)
     float *data;
     char *success;
     float *par;     
{
  *success=1;
}

typedef void (*void_func)();

static void_func proj[]={
  GD_par_useless_stub,
  GD_miller,
  GD_mollweide,
  GD_lambert,
  GD_polar_stereo,
  GD_gnomonic,
  GD_globe_view,
  GD_general_perspective};

static void_func inverse_proj[]={
  GD_par_useless_stub,
  GD_inverse_miller,
  GD_inverse_mollweide,
  GD_inverse_lambert,
  GD_inverse_polar_stereo,
  GD_inverse_gnomonic,
  GD_inverse_globe_view,
  GD_inverse_general_perspective};

static void_func fe_proj[]={
  GD_ser_useless_stub,
  GD_fe_miller,
  GD_fe_mollweide,
  GD_fe_lambert,
  GD_fe_polar_stereo,
  GD_fe_gnomonic,
  GD_fe_globe_view,
  GD_fe_general_perspective};

static void_func fe_inverse_proj[]={
  GD_ser_useless_stub,
  GD_fe_inverse_miller,
  GD_fe_inverse_mollweide,
  GD_fe_inverse_lambert,
  GD_fe_inverse_polar_stereo,
  GD_fe_inverse_gnomonic,
  GD_fe_inverse_globe_view,
  GD_fe_inverse_general_perspective};

static void_func projection_border[]={
  GD_border_par_useless_stub,
  GD_miller_border,
  GD_mollweide_border,
  GD_border_par_useless_stub,
  GD_polar_stereo_border,
  GD_polar_stereo_border,
  GD_globe_view_border,
  GD_border_par_useless_stub};



static int proj_coord[] = {
  0 ,          /* GS_NO_PROJECTION */
  GD_ANGULAR , /* GS_MILLER      */
  GD_ANGULAR , /* GS_MOLLWEIDE   */
  GD_ANGULAR , /* GS_LAMBERT     */
  GD_ANGULAR , /* GS_POLAR_STEREO*/
  GD_ANGULAR , /* GS_GNOMONIC    */
  GD_ANGULAR , /* GS_GLOBE_VIEW */
  GD_CART_3D   /* GS_GENERAL_PERSPECTIVE*/
  };


void GD_project(projection,data,success,parameters,coord_syst,origin)
     int projection;
     CM_field_id_t data,success;
     float *parameters;
     int coord_syst;
     float *origin;
{
  if( coord_syst == GD_ANGULAR && 
     proj_coord[projection-GS_NO_PROJECTION] == GD_CART_3D )
    GD_map_angular_3d(data, data, origin);
    
  if( coord_syst == GD_CART_3D && 
     proj_coord[projection-GS_NO_PROJECTION] == GD_ANGULAR )
    GD_map_3d_angular(data, data, origin);

  proj[projection-GS_NO_PROJECTION](data,success,parameters);
  
  if (projection != GS_GENERAL_PERSPECTIVE)
    CM_f_move_zero_always_1L (data+64,23,8);
}  


void old_GD_project (projection,data,success,parameters)
     int projection;
     CM_field_id_t data,success;
     float *parameters;
{
  proj[projection-GS_NO_PROJECTION](data,success,parameters);
  
  if (projection != GS_GENERAL_PERSPECTIVE)
    CM_f_move_zero_always_1L (data+64,23,8);
}


void GD_inverse_project (projection,data,success,parameters)
     int projection;
     CM_field_id_t data,success;
     float *parameters;
{
  inverse_proj[projection-GS_NO_PROJECTION](data,success,parameters);

}



void GD_fe_project (projection,data,success,parameters)
     int projection;
     char *success;
     float *data,*parameters;     
{
  fe_proj[projection-GS_NO_PROJECTION](data,success,parameters);

  if (projection != GS_GENERAL_PERSPECTIVE)
    data[2]=0.0;
}



void GD_fe_inverse_project (projection,data,success,parameters)
     int projection;
     char *success;
     float *data,*parameters;
{
  fe_inverse_proj[projection-GS_NO_PROJECTION](data,success,parameters);
}


void GD_projection_border (dest,projection,parameters)
     CM_field_id_t dest;
     unsigned projection;
     float *parameters;
{
  if (projection_border[projection-GS_NO_PROJECTION])
    projection_border[projection-GS_NO_PROJECTION](dest,parameters);
}




