#include <geodata.h>
#include <geodata_private.h>
#include <cm/paris.h>
#include <stdio.h>
#include <math.h>
#include "vector.h"


GD_print_memory_statistics (string)
     char *string;
{
  CM_vp_set_id_t entry_vp_set=CM_current_vp_set;
  CM_set_vp_set(CM_physical_vp_set());
  printf ("%s:%d%%\n",string,(int)(100.0*((float)CM_available_memory()/
					  (float)(CM_physical_memory_limit))));
  CM_set_vp_set(entry_vp_set);
}

/*   Function Header   -----------------------------
     NAME:        GD_transform_coordinates
     DESCRIPTION: take a data field and transform it into a transfer 
                  format field.
     PARAMETERS:  dest    transfer field
                  src     source field
                  desc    a data descriptor
                  proj    projection code
		  parm    projection parameter array
     PRECOND:     meshes have been expanded to points
     POSTCOND:    none 
     RETURN:      void
     LOGIC:       return format:
       POINTS & MESH: lat, lon, alt, value, context
       VECTOR: lat, lon, alt, value, context, lat, lon, alt, value, context
     End Header   ---------------------------------- */

void GD_transform_coordinates (transfer, desc, proj, parm,success)
     CM_field_id_t transfer;
     GD_data_desc *desc;
     int proj;  float *parm;
     CM_field_id_t success;
{
  float xp[3], yp[3], d;


  if (desc->coord_syst == GD_CART_2D) {  /*  This is not right probably */
    d = compute_axes(desc->r, desc->nv, desc->yp, xp, yp, desc->origin);
/*      printf("xp = (%f %f %f), yp = (%f %f %f).\n", xp[0], xp[1], 
	     xp[2], yp[0], yp[1], yp[2]);*/
    if (desc->rank == 3) {
      GD_map_2d_volume_3d(transfer, transfer, xp, yp, desc->nv, desc->r); }
    else {
      GD_map_2d_plane_3d(transfer, transfer, transfer+32, xp, yp, desc->r);
    }
    GD_map_3d_angular(transfer, transfer, desc->origin);
    GD_project(proj,transfer,success,parm,desc->coord_syst,desc->origin);
  }
  
  else if (desc->coord_syst == GD_CART_3D) {
    GD_project(proj,transfer,success,parm,desc->coord_syst,desc->origin);
  }    

  else if (desc->coord_syst == GD_MAP) {
    GD_inverse_project(desc->proj_fn,transfer,success,desc->proj_params);
    CM_logand_context(success);
    GD_project(proj,transfer,success,parm,desc->coord_syst,desc->origin);
  }

  else if (desc->coord_syst == GD_ANGULAR) {
    GD_project(proj,transfer,success,parm,desc->coord_syst,desc->origin);
  }

}   /* end of GD_transform_coordinates */     



/*   Function Header   -----------------------------
     NAME:        GD_make_transfer
     DESCRIPTION: take a data field and transform it into a transfer 
                  format field.
     PARAMETERS:  transfer  transfer field
                  desc      a data descriptor
                  proj      projection code
		  parm      projection parameter array
     PRECOND:     meshes have been expanded to points
     POSTCOND:    transfer is ready for SG
     RETURN:      void
     LOGIC:
       1.  Transform coordinates on first (or only half) of
           data.
       2.  Transform coordinates on second part of vector data.
     End Header   ---------------------------------- */

void GD_make_transfer (transfer, desc, proj, parm)
     CM_field_id_t transfer;
     GD_data_desc *desc;
     int proj;  float *parm;
{
  CM_field_id_t success = CM_allocate_stack_field(1);
  CM_field_id_t line_success = CM_allocate_stack_field(1);
  CM_field_id_t temp_success = CM_allocate_stack_field(1);



  /* this is a mess*/
  if (desc->load_type != GD_BORDER_LOAD)
    GD_expand_type(desc->load_type,
		   desc->transfer,
		   desc->values,desc);
  else GD_projection_border (desc->transfer,proj,parm);


    /* this is a mess*/
  if (desc->transform_t){  
    GD_convert_units_std(desc->transfer,desc->transfer,desc,0);
    if(desc->type == GD_VECTOR) 
      GD_convert_units_std(desc->transfer+128,desc->transfer+128,desc,0);
    
    CM_store_context(success);

/*    if (desc->print_stats){
      printf ("after convert units std:\n");
      GD_transfer_statistics((desc->type == GD_VECTOR)?8:4,desc->transfer);
    }
*/
    if (desc-> type != GD_POINT_TYPE)
      GD_long_line_filter(transfer, proj, parm[0], desc->type, line_success);
    
    
    GD_transform_coordinates(transfer,desc,proj,parm,temp_success);
    CM_logand_2_1L(success,temp_success,1);
    
    if(desc->type == GD_VECTOR) {
      GD_transform_coordinates(transfer+128,desc,proj,parm,temp_success);
      /*    CM_logand_2_1L(success,temp_success,1);  eah*/    
      CM_logior_2_1L(success,temp_success,1);
      CM_store_context (success);
    }

    if (desc-> type != GD_POINT_TYPE)
      CM_logand_2_1L(success,line_success,1); 
    
    CM_logand_context(success);
  }
  CM_deallocate_stack_through(success);
} /* end of GD_make_transfer */
