/*   File Header   ---------------------------------

     File Name:       units.c

     Author and Date: Dan Van Belleghem: conversion routines--7/26/90
                      George Lamb: interface to DM--8/2/90
                      Paul B. Anderson:  added guts to convert_std  10/5/90
     Description:     this file contains code to switch between units
     over all axes of a data desc.

     int   GD_convert_units_std(dest, src, desc)
     int   GD_convert_units_from_std(dest, src, desc)
     int   GD_cm_convert_units(dest, src, t1, t2)
    End Header   ---------------------------------- */


#include <geodata.h>
#include <geodata_private.h>
#include <cm/paris.h>
#include <stdio.h>

struct conversion {int src_units, dst_units; float factor; } 
  table[30] = { {GD_RADIANS,     GD_RADIANS,     1.0},
                {GD_RADIANS,     GD_DEGREES,     57.29578},
                {GD_DEGREES,     GD_RADIANS,     0.017453293},
                {GD_DEGREES,     GD_DEGREES,     1.0},
  
                {GD_METERS,      GD_METERS,      1.0},
                {GD_METERS,      GD_FEET,        3.280833},
                {GD_METERS,      GD_KM,          0.001},
                {GD_METERS,      GD_MILES,       0.0005396107},
                {GD_METERS,      GD_EARTH_RADII, 1.569610e-7},

                {GD_FEET,        GD_METERS,      0.3048006},
                {GD_FEET,        GD_FEET,        1.0},
                {GD_FEET,        GD_KM,          3.048006e-4},
                {GD_FEET,        GD_MILES,       1.644736e-4},
                {GD_FEET,        GD_EARTH_RADII, 4.784181e-8},

                {GD_KM,          GD_METERS,      1000.0},
                {GD_KM,          GD_FEET,        3280.833},
                {GD_KM,          GD_KM,          1.0},
                {GD_KM,          GD_MILES,       0.53961069},
                {GD_KM,          GD_EARTH_RADII, 1.569610e-4},

                {GD_MILES,       GD_METERS,      1853.1876},
                {GD_MILES,       GD_FEET,        6080.0},
                {GD_MILES,       GD_KM,          1.8531876},
                {GD_MILES,       GD_MILES,       1.0},
                {GD_MILES,       GD_EARTH_RADII, 2.908782e-4},

                {GD_EARTH_RADII, GD_METERS,      6371009.0},
                {GD_EARTH_RADII, GD_FEET,        20902217.0},
                {GD_EARTH_RADII, GD_KM,          6371.0090},
                {GD_EARTH_RADII, GD_MILES,       3437.8646},
                {GD_EARTH_RADII, GD_EARTH_RADII, 1.0},

                {NULL,           NULL,           1.0} };

void GD_cm_convert_units(dest, src, t1, t2)
     CM_field_id_t dest, src;
     short t1, t2;
{
  struct conversion *ptr;

  for (ptr = table; ptr->src_units != NULL; ptr++) {
    if (ptr->src_units == t1 && ptr->dst_units == t2) {
      CM_f_multiply_constant_3_1L(dest, src, ptr->factor, 23, 8);
      break;
    }
  }

}

float GD_fe_convert_units(src, t1, t2)
     float src;
     short t1, t2;
{
  struct conversion *ptr;  float result = 0.0;

  for (ptr = table; ptr->src_units != NULL; ptr++) {
    if (ptr->src_units == t1 && ptr->dst_units == t2) {
      result = src*ptr->factor; break;
    }
  }
  
  return(result);

}

/*   Function Header   -----------------------------
     NAME:        GD_convert_units_std
     DESCRIPTION: convert the units of a std field
     PARAMETERS:  dest--where to put the converted units
                  src--which values to convert
		  desc--describes the data to be converted
     PRECOND:     dest and src are allocated and have legit values.
                  desc != null
     POSTCOND:    dest contains values of the correct units.
     RETURN:      sucess or failure
     LOGIC: we're going to std format, ie RADIANS, RADIANS, METERS.
        Assuming that MAP and CART_2D coordinate systems have fields (x, y) to
	convert, and ANGULAR and CART_3D have (x, y, z) = (lat, lon, alt), we
	switch on the coordinate system, calling GD_cm_convert_units with the 
	appropriate 2 or 3 fields and type to convert to.
    End Header---------------------------------- */

int GD_convert_units_std(dest, src, desc, rank)
     CM_field_id_t dest, src;
     GD_data_desc *desc;
     int rank;
{
  if (desc->coord_syst == GD_ANGULAR) {
    GD_cm_convert_units(dest, src, desc->units[0], GD_RADIANS);
    GD_cm_convert_units(dest+32, src+32, desc->units[1], GD_RADIANS);
    GD_cm_convert_units(dest+2*32, src+2*32, desc->units[2], GD_METERS);
  }
  else {
    GD_cm_convert_units(dest, src, desc->units[0], GD_EARTH_RADII);
    GD_cm_convert_units(dest+32, src+32, desc->units[1], GD_EARTH_RADII);
/*    if ((desc->coord_syst == GD_CART_3D) || (rank != 0)){
      GD_cm_convert_units(dest+2*32, src+2*32, desc->units[2], GD_METERS);
    } */
    GD_cm_convert_units(dest+2*32, src+2*32, desc->units[2], GD_EARTH_RADII);
  }
  return(1);
}      /* end of GD_convert_units_std */
