/*   File Header   ---------------------------------
     File Name: geodata.h
     Author and Date:
     Description: include file containing enumerated 
                  constants
     End Header   ---------------------------------- */

/**********************\
 * Data Types         *
\**********************/
#define GD_POINT_TYPE   20
#define GD_VECTOR  21
#define GD_MESH    22


/*****************************\
 * Coordinate Systems        *
\*****************************/
#define GD_MAP      25
#define GD_ANGULAR  26
#define GD_CART_2D  27
#define GD_CART_3D  28


/**********************\
 *  Angular units     *
\**********************/
#define GD_RADIANS       30
#define GD_DEGREES       31


/************************\
 *  Cartesian units     *
\************************/
#define GD_METERS 40
#define GD_FEET   41
#define GD_KM     42
#define GD_MILES  43
#define GD_EARTH_RADII 44


/*******************************/
/*  client interface constants */
/*******************************/
#define GD_CENTER 60
#define GD_MSL    61

#define GD_FE     80
#define GD_CM     81
#define GD_REF    82


/*******************************/
/*  projections                */
/*******************************/
#define GS_NO_PROJECTION 100
#define GS_MILLER        101
#define GS_MOLLWEIDE     102
#define GS_LAMBERT       103
#define GS_POLAR_STEREO  104
#define GS_GNOMONIC      105
#define GS_GLOBE_VIEW    106
#define GS_GENERAL_PERSPECTIVE 107


/*******************************/
/*  projection arguments       */
/*******************************/
#define GS_LAT_0 110  
#define GS_LON_0 111  
#define GS_X_0 	 110  
#define GS_Y_0   111  
#define GS_ALT_0 112
#define GS_LAT_1 113
#define GS_LON_1 114  
#define GS_X_1 	 110  
#define GS_Y_1   111  
#define GS_ALT_1 115

/*******************************/
/*  representations            */
/*******************************/
#define GS_SYMBOL_REP 121
#define GS_LINE 122
#define GS_AREA 123
#define GS_AREA_CONTOUR 124
#define GS_VOLUME 125
#define GS_VOLUME_CONTOUR 126

/*******************************/
/* what varies                 */
/*******************************/
#define GS_NOTHING_VARIES 151
#define GS_COLOR_VARIES 152
#define GS_SCALE_VARIES 153
#define GS_ROTATION_VARIES 154

/*******************************/
/* display types               */
/*******************************/
#define GS_CMFB 161
#define GS_SMOOTH_CMFB 162
#define GS_XWINDOWS 163
#define GS_GLWINDOWS 164
#define GS_CMFS 165
#define GS_XL7700 166

/*******************************/
/* discrete components         */
/*******************************/
#define GS_SYM 171
#define GS_STYLE 172
#define GS_PATTERN 173

/*******************************/
/* light types                 */
/*******************************/
#define GS_AMBIENT 181
#define GS_SUN 182
#define GS_CURRENT_SUN 183
#define GS_POINT_LIGHT 184

/**************************\
 *  Load formats          *
\**************************/
#define GD_BEGINNING_OF_LOAD_TYPES 249
#define GD_2DP            250
#define GD_3DP            251
#define GD_2DP_NO_VALUE   252
#define GD_3DP_NO_VALUE   253
#define GD_2DV            254
#define GD_3DV            255
#define GD_2DV_NO_VALUE   256
#define GD_3DV_NO_VALUE   257
#define GD_WDB2           258
#define GD_3D_VCHAIN      259
#define GD_3D_VCHAIN_NO_VALUE 260
#define GD_MESH_32        261
#define GD_DBDB5          262
#define GD_DBDB5_EDGES    263
#define GD_CLIENT         264
#define GD_CLIENT_VECTOR  265
#define GD_DLG_LOAD       266
#define GD_BORDER_LOAD    267
#define GD_END_OF_LOAD_TYPES 268

/**************************\
 *  DB argument types     *
\**************************/
#define GB_FLOAT 0
#define GB_INT 1
#define GB_STRING 2
#define GB_LIST 3
#define GB_OBJECT 4
#define GB_QUOTED_CHAR_ARRAY 5
#define GB_FUNCTION 6

/******************************\
*   periodic axes             *
\******************************/

#define GD_PERIODIC 270
#define GD_NON_PERIODIC 271

/*****************************\
*   video disk defines        *
\*****************************/

#define GB_RECORDER 301
#define GB_PLAYER 302
#define GB_MONITOR 303
