/*   File Header   ---------------------------------
     File Name: geodata_private.h
     Author and Date:
     Description: internal data structures
     End Header   ---------------------------------- */
#ifndef EXTRACT
#include <geodata.h>
#include <cm/paris.h>
#include <cm/cmfs.h>
#include <X11/Xlib.h>
#include <stdio.h>
#endif

/* constants */
#define TRUE   1
#define FALSE  0
#define DV_FILE int
#define DETAIL_LENGTH 128
#define MAX_GEOMETRY 4
#define LABEL_LENGTH 24
#define NO_PROJECTION_PARAMS 12
#define GD_FILE_NAME_LENGTH 100

/*************************************************************
   GB information
*************************************************************/
typedef int GB_id;

#define GB_ERROR 1
#define GB_EPRINT 2
#define GB_NOTIFY 3
#define GB_NPRINT 4

#define GB_LABEL_LENGTH 24
#define GB_HASH_TABLE_SIZE 509  /* primes about 512 = 503, 509, & 521 */


typedef struct GB_entry 
{
  GB_id id;
  char label[GB_LABEL_LENGTH];
  int type;
  int *instance;
  struct GB_entry *next;
}  GB_entry,*GB_entry_t;

#define END_OF_CONSTANTS 9999

typedef struct GB_constant 
{
 char *name;
 int value;
} GB_constant,*GB_constant_t;


typedef struct GB_structure {
  int number,size,type;
  char *(*accessor)();
  int list_type;
} GB_structure, *GB_structure_t;

typedef struct GB_GL_structure {
  char *name;
  int symbol_type;
  int val_type;
  int number,size,type;
  char *(*accessor)();
  int list_type;
} GB_GL_structure, *GB_GL_structure_t;


typedef struct GB_type {
  void (*scanner)();
  int length;
} GB_type, *GB_type_t;


typedef struct GB_parse_frame
{
  FILE *in,*out,*err;
  int line_count;
  char file_name[GD_FILE_NAME_LENGTH];
} GB_parse_frame,*GB_parse_frame_t;


/*************************************************************
   GD information
*************************************************************/

/* nuke eventually */
typedef struct GD_point 
{  float lat, lon, alt, value;}      GD_point;


typedef struct GD_load_functions
{
  int size;
  int (*expand_func)();
/*  void (*context_func)();*/
}  GD_load_functions;


typedef struct GD_data_desc        /*MANAGED*/
{	
  char          detail[DETAIL_LENGTH];    /*MANAGED*/
  char          filename[GD_FILE_NAME_LENGTH];    /*MANAGED*/
  int           coord_syst;        /*MANAGED*/
  int           units[3];          /*MANAGED*/
  int           type;              /*MANAGED*/
  int           proj_fn;           /*MANAGED*/
  int           at_end;
  int           fortran_data_flag;
  CM_geometry_id_t fortran_geometry;
  int           *fe_data;
  int           cm_fe_flag;
  int           fe_shape[3];
  int           rank;              /*MANAGED*/
  CM_field_id_t context;
  CM_field_id_t values;
  CM_field_id_t transfer;
  float         r[3];              /*MANAGED*/
  float         nv[3];             /*MANAGED*/
  int           user_yp;           /*MANAGED*/
  float         yp[3];             /*MANAGED*/
  float         origin[3];         /*MANAGED*/
  float         save_origin[3];   
  float         surface;           /*MANAGED*/
  float         extent[3];         /*MANAGED*/
  float         proj_params[NO_PROJECTION_PARAMS];   /*MANAGED*/
  int           transform_t;       /*MANAGED*/
  int           long_lines_flag;
  int           periodic_axes[3];  /*MANAGED*/
  int           file;
  int           span[3];           /*MANAGED*/
  CM_field_id_t entry_context;
  CM_vp_set_id_t  *vp_set;
  int            geometry[4];      /*MANAGED*/
  int            load_type;        /*MANAGED*/
  float          exaggeration;     /*MANAGED*/
  int            print_stats;      /*MANAGED*/
}       GD_data_desc;


/*************************************************************
   GS information
*************************************************************/

#define VREF(x,y) (CM_add_offset_to_field_id((x),((y)*32)))

/*#define MAX(x,y) ((x)>(y)?(x):(y))*/

#define MIN(x,y) ((y)>(x)?(x):(y))


#define GS_PI 3.14159265358979323846
#define GS_E 2.7182818284590452354
#define GS_ZOOM_RATIO 0.90
#define GS_EARTH_RADIUS 6371009.0  /* R_1 Mean radius of semiaxes in meters
                                      from WGS-84 */
#define GS_SOLAR_DISTANCE 10000000.0 /* completely meaningless*/
#define GS_SYMBOL_FIELD_SIZE 16384 /* Should be set to 65536 for release */

/* plot types */
#define GS_PLOT_POINT 0
#define GS_PLOT_LINE 1
#define GS_PLOT_TRIANGLE 2
#define GS_PLOT_QUAD 3
#define GS_PLOT_PRINT 4

typedef struct GS_vertex        /*MANAGED*/
{
  float point[3];               /*MANAGED*/
  struct GS_vertex *next;
} GS_vertex, *GS_vertex_t;

typedef struct GS_polygon        /*MANAGED*/
{
  struct GS_vertex *vertices;    /*MANAGED*/
  struct GS_polygon *next;      
} GS_polygon,*GS_polygon_t;

typedef struct GS_symbol         /*MANAGED*/
{
  CM_field_id_t symbol_field;   
  struct GS_polygon *polygons;   /*MANAGED*/
} GS_symbol,*GS_symbol_t;

typedef struct GS_table      /*MANAGED*/
{
  float value;               /*MANAGED*/
  int color[3];              /*MANAGED*/
  GB_id discrete;            /*MANAGED*/
  float scale;               /*MANAGED*/
  float rotation;            /*MANAGED*/
  float opacity;             /*MANAGED*/
  struct GS_table *next;    
} GS_table,*GS_table_t;


typedef struct GS_line_style         /*MANAGED*/
{
  float width;                          /*MANAGED*/
} GS_line_style,*GS_line_style_t;


typedef struct GS_pattern{
  int foo;
} GS_pattern,*GS_pattern_t;


typedef (*GS_void_func)();


typedef struct GS_rep_func {
  GS_void_func open;
  GS_void_func write;
  GS_void_func close;
  int edge_size,rank;
} GS_rep_func,*GS_rep_func_t;


typedef struct GS_representation          /*MANAGED*/
{
  int translucency;                       /*MANAGED*/
  int what_varies;                        /*MANAGED*/
  int rep_code;                           /*MANAGED*/
  struct GS_table *tables;                 /*MANAGED*/
} GS_representation,*GS_representation_t;


typedef struct GS_rep_list                /*MANAGED*/
{
  GB_id rep_id;                           /*MANAGED*/
  GB_id data_id;                          /*MANAGED*/
  int priority;                           /*MANAGED*/
  int rendering_memory;                   /*MANAGED*/
  struct GS_rep_list *next;              
} GS_rep_list,*GS_rep_list_t;


typedef struct GS_display_scene_list       /*MANAGED*/
{
  GB_id scene_id;                          /*MANAGED*/
  int offset_foo[2];                       /*MANAGED*/ 
  struct GS_display_scene_test *next;    
} GS_display_scene_list,*GS_display_scene_list_t;


typedef struct GS_display                 /*MANAGED*/
{
  int display_type;                       /*MANAGED*/
  char name[GD_FILE_NAME_LENGTH];         /*MANAGED*/
  int size[2];                            /*MANAGED*/
  char *display_id;
  struct GS_display_scene_list *scenes;   /*MANAGED*/
} GS_display,*GS_display_t;



typedef struct GS_scene                       /*MANAGED*/
{
  int dimensions[2];                          /*MANAGED*/
  int projection;                             /*MANAGED*/
  float parameters[NO_PROJECTION_PARAMS];     /*MANAGED*/
  float center[3];                            /*MANAGED*/
  float zoom;                                 /*MANAGED*/
  int using_slice;
  float slice[4];                             /*MANAGED*/
  int time[6];                                /*MANAGED*/
  float ambient;                              /*MANAGED*/
  float light_source[3];                      /*MANAGED*/
  int light_type;                             /*MANAGED*/
  struct GS_rep_list *representations;        /*MANAGED*/
  int background[3];                          /*MANAGED*/
  int time_this_scene;                        /*MANAGED*/
  int remote_render_t;                        /*MANAGED*/
  int record_this_scene;                      /*MANAGED*/
  /* this should be PGL_buffer_t*/
  char *buffer;
  int flags;
  int priority_bits;                          /*MANAGED*/
  int depth_bits;                             /*MANAGED*/
} GS_scene,*GS_scene_t;


typedef struct GS_movie                       /*MANAGED*/
{
  char movie_filename[GD_FILE_NAME_LENGTH];   /*MANAGED*/
  float frames_per_second;                    /*MANAGED*/
  int preshuffled;                            /*MANAGED*/
  int frame_offset;                           /*MANAGED*/
  int frames;                                 /*MANAGED*/
}  GS_movie,*GS_movie_t;

typedef struct GS_video                       /*MANAGED*/
{
  int disk;                                 /*MANAGED*/
  int start_frame;                          /*MANAGED*/
  int stop_frame;                           /*MANAGED*/
  int speed;                                /*MANAGED*/
  char description[128];                    /*MANAGED*/
}  GS_video,*GS_video_t;

/*************************************************************
   GI information
*************************************************************/


typedef struct GI_proj_desc {  int   code;
		  float param[NO_PROJECTION_PARAMS];  } GI_proj_desc;




/*************************************************************
   GB functions
*************************************************************/


typedef struct GB_include   /*MANAGED*/
{
  char file[GD_FILE_NAME_LENGTH];         /*MANAGED*/
} GB_include, *GB_include_t;


typedef struct GB_update_scene          /*MANAGED*/
{
  GB_id scene_id;                         /*MANAGED*/
} GB_update_scene, *GB_update_scene_t;


 /* don't reorder this struct!*/

typedef struct GB_display_scene    /*MANAGED*/
{
  GB_id scene_id;                    /*MANAGED*/ 
  GB_id display_id;                  /*MANAGED*/
  int offset[2];                     /*MANAGED*/
} GB_display_scene, *GB_display_scene_t;


typedef struct GB_print    /*MANAGED*/
{
  GB_id id;                 /*MANAGED*/
} GB_print, *GB_print_t;

typedef struct GB_recur_print    /*MANAGED*/
{
  GB_id obj_id;                 /*MANAGED*/
  char out_file[GD_FILE_NAME_LENGTH];         /*MANAGED*/
} GB_recur_print, *GB_recur_print_t;



typedef struct GB_map_data    /*MANAGED*/
{
  GB_id from;                 /*MANAGED*/
  GB_id to;                   /*MANAGED*/
} GB_map_data, *GB_map_data_t;


typedef struct GB_print_objects     /*MANAGED*/
{
  int trash;
} GB_print_objects, *GB_print_objects_t;


typedef struct GB_play_movie     /*MANAGED*/
{
  /* don't reorder this struct!*/
  GB_id movie_id;                /*MANAGED*/
  GB_id display_id;              /*MANAGED*/ 
  int display_offset[2];         /*MANAGED*/
} GB_play_movie, *GB_play_movie_t;

typedef struct GB_open_video_display     /*MANAGED*/
{
  int trash;
} GB_open_video_display, *GB_open_video_display_t;

typedef struct GB_close_video_display     /*MANAGED*/
{
  int trash;
} GB_close_video_display, *GB_close_video_display_t;

typedef struct GB_play_video     /*MANAGED*/
{
  GB_id video_id;                /*MANAGED*/
} GB_play_video, *GB_play_video_t;

typedef struct GB_seek_to_frame     /*MANAGED*/
{
  int seek_pana;                    /*MANAGED*/
  int frame;                    /*MANAGED*/
  int plus;                    /*MANAGED*/
  int minus;                    /*MANAGED*/
} GB_seek_to_frame, *GB_seek_to_frame_t;

typedef struct GB_set_video_switch     /*MANAGED*/
{
  int set_to;                              /*MANAGED*/
} GB_set_video_switch, *GB_set_video_switch_t;

typedef struct GB_play_pana    /*MANAGED*/
{
  int pana_play;                    /*MANAGED*/
  int play_from;                    /*MANAGED*/
  int play_to;                    /*MANAGED*/
  int speed;                    /*MANAGED*/
} GB_play_pana, *GB_play_pana_t;

typedef struct GB_handshake  /*MANAGED*/
{
  int trash;
} GB_handshake, *GB_handshake_t;


typedef struct GB_sleep  /*MANAGED*/
{
  int seconds;           /*MANAGED*/
} GB_sleep, *GB_sleep_t;


typedef struct GB_die  /*MANAGED*/
{
  int trash;
} GB_die, *GB_die_t;


typedef struct GB_detach  /*MANAGED*/
{
  int trash;
} GB_detach, *GB_detach_t;



/*************************************************************
   GX Structures and Globals
*************************************************************/

#define GX_COM_LEN 255
#define GX_NO_SCALES 12
#define GX_DEG2RAD 0.00017453  /* ((GS_PI / 180.0) / 100.0) */
#define GX_DIV100  0.0100000

typedef struct GX_xtool_projection                        /*MANAGED*/
{
  int GX_proj_code;                                       /*MANAGED*/
  int GX_num_sc;                                          /*MANAGED*/
  int GX_scale_map[GX_NO_SCALES];                         /*MANAGED*/
  int GX_precision[NO_PROJECTION_PARAMS];                 /*MANAGED*/
  int GX_decimals[NO_PROJECTION_PARAMS];                  /*MANAGED*/
  float GX_defaults[NO_PROJECTION_PARAMS];                /*MANAGED*/
  float GX_convert[NO_PROJECTION_PARAMS];                 /*MANAGED*/
  float GX_min[NO_PROJECTION_PARAMS];                     /*MANAGED*/
  float GX_max[NO_PROJECTION_PARAMS];                     /*MANAGED*/
  char GX_labels[NO_PROJECTION_PARAMS][GB_LABEL_LENGTH];  /*MANAGED*/
} GX_xtool_projection, *GX_xtool_projection_t;
