#include <sys/types.h>
#include <sgi_vmeio.h>
#include <cm/paris.h>
#include <cm/cmfs.h>
#include <cm/cm_file.h>
#include <cmsvio.h>
#include <cm/cm_ioctl.h>
#include <math.h>

static int fd;
static int data_set_count;
static int light_count;

#define CLIPPING_SIZE 1.0e6

void SGI_rot (x,y,z)
     float x,y,z;
{
  sgi_vec arg;
  
  arg.x=x;
  arg.y=y;
  arg.z=z;
  
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_ROT,sgi_vec),&arg);
}


void SGI_view (from,at,twist)
     sgi_vec from;
     sgi_vec at;
     float twist;
{
  sgi_view_t arg;

  arg.lf.x=from.x;
  arg.lf.y=from.y;
  arg.lf.z=from.z;

  arg.la.x=at.x;
  arg.la.y=at.y;
  arg.la.z=at.z;

  arg.twist = twist;

 CMFS_ioctl(fd,_CMFS_IOW(0,SGI_VIEW,sgi_view_t),&arg);
}

void SGI_add_view (from,at,twist)
     sgi_vec from;
     sgi_vec at;
     float twist;
{
  sgi_view_t arg;

  arg.lf.x=from.x;
  arg.lf.y=from.y;
  arg.lf.z=from.z;

  arg.la.x=at.x;
  arg.la.y=at.y;
  arg.la.z=at.z;

  arg.twist = twist;

  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_ADD_VIEW,sgi_view_t),&arg);
}


void SGI_fov (arg)
     float arg;
{
 CMFS_ioctl(fd,_CMFS_IOW(0,SGI_FOV,float),&arg);
}

void SGI_set_vertex (arg)
     int arg;
{
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_SET_VERTEX,int),&arg);
}

void SGI_send_color (arg)
     int arg;
{
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_SEND_COLOR,int),&arg);
}

void SGI_send_normal (arg)
     int arg;
{
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_SEND_NORMAL,int),&arg);
}

void SGI_color (r,g,b)
     unsigned char r,g,b;
{
  sgi_color_t arg;

  arg.r=r;
  arg.g=g;
  arg.b=b;
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_COLOR,sgi_color_t),&arg);
}

void SGI_delete (arg)
     int arg;
{
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_DELETE,int),&arg);
}

void SGI_delete_all ()
{
  int arg;
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_DELETE_ALL,int),&arg);
}

void SGI_delete_list (number,arg)
     int number;
     int *arg;
{
  CMFS_ioctl(fd,_CMFS_IOWN(0,SGI_DELETE_LIST,number*sizeof(int)),arg);
}


void SGI_activate (arg)
     int arg;
{
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_ACTIVATE,int),&arg);
}


void SGI_activate_all ()
{
  int arg;
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_ACTIVATE_ALL,int),&arg);
}

void SGI_activate_list (number,arg)
     int number;
     int *arg;
{
  CMFS_ioctl(fd,_CMFS_IOWN(0,SGI_ACTIVATE_LIST,number*sizeof(int)),arg);
}


void SGI_deactivate (arg)
     int arg;
{
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_DEACTIVATE,int),&arg);
}


void SGI_deactivate_all ()
{
  int arg;
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_DEACTIVATE_ALL,int),&arg);
}


void SGI_deactivate_list (number,arg)
     int number;
     int* arg;
{
  CMFS_ioctl(fd,_CMFS_IOWN(0,SGI_DEACTIVATE_LIST,number*sizeof(int)),arg);
}


void SGI_transfer_mode (arg)
     int arg;
{
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_TRANSFER_MODE,int),&arg);
}



void SGI_translate (x,y,z)
     float x,y,z;
{
  sgi_vec arg;
  arg.x=x;
  arg.y=y;
  arg.z=z;
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_TRANSLATE,sgi_vec),&arg);
}


void SGI_scale (x,y,z)
     float x,y,z;
{
  sgi_vec arg;
  arg.x=x;
  arg.y=y;
  arg.z=z;
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_SCALE,sgi_vec),&arg);
}



int SGI_add_light (ambient,lcolor,pos)
     sgi_vec *ambient;
     sgi_vec *lcolor;
     sgi_vec *pos;
{
  sgi_light_t arg;
  float norm = sqrt (pos->x*pos->x+
		     pos->y*pos->y+
		     pos->z*pos->z);
  
  arg.ambient.x = ambient->x;
  arg.ambient.y = ambient->y;
  arg.ambient.z = ambient->z;
  
  arg.lcolor.x = lcolor->x;
  arg.lcolor.y = lcolor->y;
  arg.lcolor.z = lcolor->z;
  
  arg.pos.x = pos->x/norm;
  arg.pos.y = pos->y/norm;
  arg.pos.z = pos->z/norm;
  
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_ADD_LIGHT,sgi_light_t),&arg);
  return (light_count++);
}


void SGI_light_on (arg)
     int arg;
{
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_LIGHT_ON,int),&arg);
}


void SGI_light_off (arg)
     int arg;
{
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_LIGHT_OFF,int),&arg);
}


void SGI_record_on ()
 
{
    int arg;
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_RECORD_ON,int),&arg);
}


void SGI_record_off ()
{
  int arg;
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_RECORD_OFF,int),&arg);
}


void SGI_frames (arg)
     int arg;
{
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_FRAMES,int),&arg);
}


void SGI_type_quad_grid (x,y)
     int x,y;
{
  int dim[2];
  dim[0]=x;
  dim[1]=y;
  CMFS_ioctl(fd,_CMFS_IOWN(0,SGI_TYPE_QUAD_GRID,2*sizeof(int)),dim);
}



void SGI_type_default ()
{
  int arg;
  CMFS_ioctl(fd,_CMFS_IOWN(0,SGI_TYPE_DEFAULT,sizeof(int)),&arg);
}


void SGI_frame_block ()
{
  int arg;
  CMFS_ioctl(fd,_CMFS_IOWN(0,SGI_FRAME_BLOCK,sizeof(int)),&arg);
}



void SGI_debug ()
{
  int arg;
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_DEBUG,int),&arg);
}


void SGI_tmc_sucks (arg)
     int arg;
{
  CMFS_ioctl(fd,_CMFS_IOW(0,SGI_TMC_SUCKS,int),&arg);
}


int SGI_open(machine)
     char *machine;
{
  char filename[255];
  
  sprintf(filename,"%s:%s",machine,DEVICE_NAME);
  if ((fd=CMFS_open (filename,CMFS_O_WRONLY))< 0){
    CMFS_perror ("sgi open failed");
    exit(1);
  }
  data_set_count=0;
  light_count=1;
  return(fd);
}

int SGI_write(number,flags,field)
     unsigned number,flags;
     CM_field_id_t field;
{
  CM_field_id_t temp,index;
  int whole_length;
  unsigned length=VERTEX_LENGTH;
  CM_field_id_t fields;
  int i;

  if (flags & NORMALS){
    SGI_send_normal(1);
    length += NORMAL_LENGTH;
  } else  SGI_send_normal(0);
  
  if (flags & COLORS){ 
    SGI_send_color(1);
    length +=COLOR_LENGTH;
  } else SGI_send_color(0);

  whole_length = length*number;
  whole_length += (whole_length%32)?(32-(whole_length%32)):0;
  temp=CM_allocate_stack_field(whole_length);
  CM_u_move_zero_always_1L (temp,length*number);
  CM_u_move_1L (temp,field,length*number);
  
#ifndef DONT_CLIP
  for (i=0;i<number*3;i++){
    CM_f_min_constant_2_1L (temp+i*32,CLIPPING_SIZE,23,8);
    CM_f_max_constant_2_1L (temp+i*32,-CLIPPING_SIZE,23,8);
  }
#endif

  if (flags & PRINT_INPUTS){
    for (i=0;i<number;i++){
      printf ("vertex %d: (%f %f) (%f %f) (%f %f)\n",i,
	      CM_global_f_min_1L (temp+96*i,23,8),
	      CM_global_f_max_1L (temp+96*i,23,8),
	      CM_global_f_min_1L (temp+96*i+32,23,8),
	      CM_global_f_max_1L (temp+96*i+32,23,8),
	      CM_global_f_min_1L (temp+96*i+64,23,8),
	      CM_global_f_max_1L (temp+96*i+64,23,8));

      if (flags & NORMALS)
	printf ("normal %d: (%f %f) (%f %f) (%f %f)\n",i,
		CM_global_f_min_1L (temp+96*i+96*number,23,8),
		CM_global_f_max_1L (temp+96*i+96*number,23,8),
		CM_global_f_min_1L (temp+96*i+96*number+32,23,8),
		CM_global_f_max_1L (temp+96*i+96*number+32,23,8),
		CM_global_f_min_1L (temp+96*i+96*number+64,23,8),
		CM_global_f_max_1L (temp+96*i+96*number+64,23,8));


      if (flags & COLORS)
	printf ("color %d: (%d %d) (%d %d) (%d %d)\n",i,
		CM_global_u_min_1L(temp+24*i+((flags&NORMALS)?96:0+96)*number,
				   8),	
		CM_global_u_max_1L(temp+24*i+((flags&NORMALS)?96:0+96)*number,
				   8),
		CM_global_u_min_1L(temp+24*i+((flags&NORMALS)?96:0+96)*number
				   +8,8),	
		CM_global_u_max_1L(temp+24*i+((flags&NORMALS)?96:0+96)*number
				   +8,8),
		CM_global_u_min_1L(temp+24*i+((flags&NORMALS)?96:0+96)*number
				   +16,8),	
		CM_global_u_max_1L(temp+24*i+((flags&NORMALS)?96:0+96)*number
				   +16,8));
		
		
    }
  }
  
  CMFS_cm_to_standard_byte_order (temp,number*VERTEX_LENGTH);
  
  if (flags & NORMALS) 
    CMFS_cm_to_standard_byte_order (temp+number*VERTEX_LENGTH,
				    number*NORMAL_LENGTH);
  
  SGI_set_vertex (number);
  CMFS_transpose_record_always (temp,whole_length,
				CMFS_IO_WRITE_TO_SERIAL_DATA);

  if (CMFS_write_file_always (fd,temp,whole_length) != whole_length){
    CMFS_perror ("sgi write failed");
    exit(1);
  }

  CM_deallocate_stack_through(temp);
  return(data_set_count++);
}

int SGI_write_news_grid (flags,field,axis0,axis1)
     unsigned flags;
     CM_field_id_t field;
     unsigned axis0,axis1;     
{
  unsigned length = VERTEX_LENGTH;
  CM_field_id_t temp;
  CM_field_id_t x=CM_allocate_stack_field(32);
  CM_field_id_t y=CM_allocate_stack_field(32);
  unsigned xl=CM_geometry_axis_length(CM_vp_set_geometry(CM_current_vp_set),
				      axis0);
  unsigned yl=CM_geometry_axis_length(CM_vp_set_geometry(CM_current_vp_set),
				      axis1);
  int result;

  if (flags & NORMALS) length += NORMAL_LENGTH;
  if (flags & COLORS) length += COLOR_LENGTH;
  temp = CM_allocate_stack_field(length);
  CM_my_news_coordinate_1L (x,axis0,32);
  CM_my_news_coordinate_1L (y,axis1,32);
  CM_u_multiply_constant_2_1L (y,xl,32);
  CM_s_add_2_1L (x,y,32);
  CM_send_1L (temp,x,field,length,CM_no_field);
  SGI_type_quad_grid (yl,xl);
  result = SGI_write (1,flags,temp);
  SGI_type_default();
  CM_deallocate_stack_through(x);
  return(result);
}


void SGI_close()
{
  CMFS_close(fd);
}
