#include "pgl_internal.h"
#include <stdio.h>

static char axis_text[]={'x','y','z','r','g','b','C','X','Y','Z','R',
			   'O','U','V','W','P','v','s','d','E','T',0};

static char type_text[]={'f','s','u',0};

void PGL_delete_format (format,name)
     PGL_format_t *format;
     unsigned name;
{ 
  PGL_format_t *i,tmp;
  
  for (i=format;(*i)!=(PGL_format_t)0;i=(&((*i)->next)))
    if (((*i)->name) == name){
      tmp=(*i);
      *i=(*i)->next;
      free(tmp);
      return;
    }
}


void PGL_push_format (format,name,type,length,constant_t,constant_value)
     PGL_format_t *format;
     unsigned name,type,length,constant_t;
     char *constant_value;       
{
  PGL_format_t new;

  PGL_delete_format(format,name);
  new=(PGL_format_t)malloc(sizeof(PGL_format));
  new->name =name;
  new->type =type;
  new->length = length;
  new->next = (*format);
  if (new->constant_t = constant_t)
    bcopy (constant_value,new->constant_value,CONSTANT_LENGTH);    
  *format = new;
}


void PGL_append_format (format,name,type,length,constant_t,constant_value)
     PGL_format_t *format;
     unsigned name,type,length,constant_t;
     char *constant_value;       
{
  PGL_format_t *new;
  
  PGL_delete_format(format,name);
  for (new=format;(*new)!=(PGL_format_t)0;new=(&((*new)->next)));
  (*new)=(PGL_format_t)malloc(sizeof(PGL_format));
  (*new)->name =name;
  (*new)->type =type;
  (*new)->length = length;
  (*new)->next = (PGL_format_t) 0;
  if ((*new)->constant_t = constant_t)
    bcopy (constant_value,(*new)->constant_value,CONSTANT_LENGTH);    
}


PGL_format_t PGL_translate_format(in)
     char *in;
{
  int i,j,o;
  PGL_format_t result=(PGL_format_t)0;
  
  if (in[0] != 0) {
    for (i=0;axis_text[i]!=in[0];i++)
      if (axis_text[i] == 0){
	fprintf (stderr,"PGL:axis '%c' is not a PGL axis.\n",in[0]);
	exit(1);
      }
    result = (PGL_format_t)malloc(sizeof(PGL_format));
    result->constant_t =0;
    result->name = PGL_x + i;
    for (i=0;type_text[i]!=in[1];i++)
      if (type_text[i] == 0){
	fprintf (stderr,"PGL:type '%c' is not a PGL type.\n",in[1]);
	exit(1);
      }
    result->type = PGL_float + i;
    result->length = 0;
    
    for (i=2;((in[i]<='9') && (in[i]>='0'));i++);
    for (j=i-1,o=1;j>=2;j--,o*=10) result->length += o*(in[j]-'0');
    if ((result->length)==0) result->length =32;
    if (in[i] == '('){
      result->constant_t =1;
      switch (result->type){
      case PGL_float:
	sscanf (in+i+1,"%f)",result->constant_value);
	break;
      case PGL_signed:
	sscanf (in+i+1,"%d)",result->constant_value);
	break;
      case PGL_unsigned:
	sscanf (in+i+1,"%d)",result->constant_value);
	break;
      }
    }
    while (in[i]!=0)
      if (in[i++] == ' '){
	result->next = PGL_translate_format(in+i);
	return(result);
      }
  }
  result-> next = (PGL_format_t)0;
  return (result);
}

PGL_format_t PGL_find_name (format,name)
     PGL_format_t format;
     unsigned name;
{ 
  PGL_format_t i;
  for (i=format;i!=(PGL_format_t)0;i=i->next)
    if ((i->name) == name) return (i);
  return ((PGL_format_t)0);
}


PGL_format_t PGL_copy_format(format)
     PGL_format_t format;
{
  PGL_format_t new=(PGL_format_t)0;
  
  if (format != (PGL_format_t)0){
    new = (PGL_format_t)malloc(sizeof(PGL_format));
    bcopy (format,new,sizeof(PGL_format));
    new->next = PGL_copy_format (format->next);
  }
  return (new);
}


void PGL_free_format(format)
     PGL_format_t format;
{
  if (format!=(PGL_format_t)0) PGL_free_format(format->next);
  free (format);
}


unsigned PGL_point_length (format)
     PGL_format_t format;
{
  PGL_format_t i;
  unsigned result=0;
  for (i=format;i!=(PGL_format_t)0;i=i->next)
    if (i->constant_t == 0) result+=i->length;
  return(result);
}


/* error handling for these routines would be nice*/
void PGL_extract_copy (dest,source,format,axis,d_type,d_length)
     CM_field_id_t dest,source;
     PGL_format_t format;
     unsigned axis,d_type,d_length;
{
  PGL_format_t i;
  
  for (i=format;i!=(PGL_format_t)0;i=i->next)
    if (i->name == axis){
      if (i->constant_t)
	PGL_broadcast (dest,i->constant_value,d_type,i->type,
		       d_length,i->length);
      else 
	PGL_cast (dest,PGL_extract(source,format,i->name),
		  d_type,i->type,d_length,i->length);
      return;
    }
/* we want a union in some cases
  CM_u_move_zero_always_1L (dest,d_length);*/
}


void PGL_change_format (dest,source,dest_format,source_format)
     CM_field_id_t dest,source;
     PGL_format_t dest_format,source_format;
{
  PGL_format_t i,j;
  
  for (i=dest_format;i!=(PGL_format_t)0;i=i->next)
    if (!i->constant_t)
      PGL_extract_copy(PGL_extract(dest,dest_format,i->name),
		       source,source_format,i->name,
		       i->type,i->length);
}


CM_field_id_t PGL_extract (field,format,name)
     CM_field_id_t field;
     PGL_format_t format;
     unsigned name;
{
  PGL_format_t i;
  CM_field_id_t result=field;
  
  for (i=format;i!=(PGL_format_t)0;i=i->next){
    if (i->name == name) return (result);
    if (!i->constant_t)
      result = CM_add_offset_to_field_id(result,i->length);
  }
  fprintf (stderr,"PGL: couldn't find axis %c in format\n",axis_text[name]);
  return (0);
}

void PGL_print_range (field,type,length)
     CM_field_id_t field;
     unsigned type,length;
{
  switch(type){
  case PGL_float:
    printf("[%f %f]",CM_global_f_min_1L(field,S(length),E(length)),
	   CM_global_f_max_1L(field,S(length),E(length)));
    break;
  case PGL_signed:
    printf("[%d %d]",CM_global_s_min_1L(field,length),
	   CM_global_s_max_1L(field,length));
    break;
  case PGL_unsigned:
    printf("[%d %d]",CM_global_u_min_1L(field,length),
	   CM_global_u_max_1L(field,length));
    break;
  }
}

     
/* put constant values in here*/
void PGL_print_field(field,format)
     CM_field_id_t field;
     PGL_format_t format;
{
  PGL_format_t i;
  CM_field_id_t pointer=field;

  for (i=format;i!=(PGL_format_t)0;i=i->next)
    if (i->constant_t == 0){
      printf ("%c:",axis_text[i->name]);  
      PGL_print_range (PGL_extract(field,format,i->name),i->type,i->length);
      printf ("\n");
    }
}


/* this could be made more general purpose by the exclusion of the
   newline and the inclusion of a destination FILE *  */
/* perhaps a string destination would be better*/
/* the format could have a header and a list...accessors would need
   to be provided (such as the for_parallel_fields) for existing 
   code*/
void PGL_print_format(format)
     PGL_format_t format;
{
 PGL_format_t i;

 for (i=format;i!=(PGL_format_t)0;i=i->next){
   printf ("%c%c%d",axis_text[i->name],type_text[i->type],i->length);
   if (i->constant_t)
     switch(i->type){
     case PGL_float:
       printf ("(%f)",*((float *)i->constant_value));
       break;
     case PGL_signed:
       printf ("(%d)",*((int *)i->constant_value));
       break;
     case PGL_unsigned:
       printf ("(%d)",*((unsigned int *)i->constant_value));
       break;
     }
   printf (" ");
 }
 printf ("\n");
}



void PGL_extract_float_range(dest,source,format,axis,length)
     CM_field_id_t dest,source;
     PGL_format_t format;
     unsigned axis;
     unsigned length;
{
  if (PGL_find_name(format,axis)){
    PGL_extract_copy(dest,source,format,axis,PGL_float,length);
    switch (PGL_type(format,axis)){
    case PGL_signed:
      CM_f_divide_constant_2_1L (dest,(float)(1<<(PGL_length(format,axis)-1)),
				 S(length),E(length));
      break;
    case PGL_unsigned:
      CM_f_divide_constant_2_1L (dest,(float)(1<<PGL_length(format,axis)),
				 S(length),E(length));
      break;
    }
  } else CM_f_move_constant_1L (dest,1.0,23,8);
}


void PGL_insert_float_range(dest,source,format,axis,length)
     CM_field_id_t dest,source;
     PGL_format_t format;
     unsigned axis;
     unsigned length;
{
  CM_field_id_t temp=source;
  if (PGL_find_name(format,axis)){
    switch (PGL_type(format,axis)){
    case PGL_signed:
      temp =CM_allocate_stack_field(length);
      CM_f_multiply_constant_3_1L (temp,source,1<<(PGL_length(format,axis)-1),
				   S(length),E(length));
      break;
    case PGL_unsigned:
      temp =CM_allocate_stack_field(length);
      CM_f_multiply_constant_3_1L (temp,source,1<<PGL_length(format,axis),
				   S(length),E(length));
      break;
    }
    PGL_cast (PGL_extract(dest,format,axis),temp,
	      PGL_type(format,axis),PGL_float,
	      PGL_length(format,axis),length);
    if (temp != source) CM_deallocate_stack_through(temp);
  }
}
