#include <cm/paris.h>
#include <cm/cmfb.h>
#include "pgl.h"

#ifdef X_WINDOWS
#include <X11/Xlib.h>
#include <X11/Xos.h>
#define NUMBER_OF_CELLS 100
#endif

#define CONSTANT_LENGTH 8  

typedef struct PGL_format{
  struct PGL_format *next;
  unsigned constant_t,name,type,length;
  char constant_value[CONSTANT_LENGTH];
} PGL_format,*PGL_format_t;


typedef struct PGL_light {
  struct PGL_light *next;
  float position[3];
  unsigned char color[3];
  unsigned light_number;
} PGL_light,*PGL_light_t;


typedef struct PGL_buffer {
  unsigned width,height,depth;
  unsigned rank;
  unsigned preshuffled_t,transform_t;
  float transform[4][4];
  PGL_format_t format;
  CM_field_id_t field;
  CM_geometry_id_t geometry;
  CM_vp_set_id_t vp_set;
  unsigned combiner;
  unsigned stack_size;
  unsigned char ambient[3];
  PGL_light_t lights;
  unsigned light_count;
  int *data_items;
  int data_item_count;
  int *old_data_items;
  int old_data_item_count;
  unsigned deallocate_t;
} PGL_buffer,*PGL_buffer_t;


typedef struct PGL_plot_func {
  void (*plot)();
  unsigned rank;
} PGL_plot_func,*PGL_plot_func_t;

typedef struct PGL_display {
  unsigned type;
  struct CMFB_display_id cmfb;
  unsigned width,height;
  CM_vp_set_id_t vp_set;
  unsigned file;
#ifdef X_WINDOWS
  Display *x_display;
  Screen *x_screen;
  Window x_window;
  Pixmap x_pixmap;
  GC gc;
  unsigned mapped;
  float colors[NUMBER_OF_CELLS][3];
  unsigned translation[NUMBER_OF_CELLS];
  unsigned number_of_colors,valid_colors;
#endif
} PGL_display, *PGL_display_t;


typedef struct PGL_display_function{
  void (*open)();
  void (*write)();
  void (*close)();
  unsigned preshuffled_t;
} PGL_display_function,*PGL_display_function_t;


typedef struct PGL_pixmap{
  struct PGL_pixmap *next;
  unsigned number;
  unsigned dimensions[3];
  CM_field_id_t field;
  CM_vp_set_id_t vp_set;
  CM_geometry_id_t geometry;
  unsigned shared_t,rank,plength;  
  PGL_format_t format;
}PGL_pixmap,*PGL_pixmap_t;

typedef struct PGL_spawn_frame{
  PGL_buffer_t dest;
  void (*fill)();
  unsigned flags;
  int size,length,pointer,count;
  PGL_format_t format;
  CM_geometry_id_t geometry;
  CM_vp_set_id_t vp_set;
  CM_field_id_t xh,xp,seg;
  struct PGL_spawn_frame *next;
} PGL_spawn_frame,*PGL_spawn_frame_t;


#ifdef sparc
#define alloca(x) __builtin_alloca(x)
#define PGL_STACK_FRAME __builtin_va_alist
#endif

#define PGL_length(MACRO_TEMP_1,MACRO_TEMP_2) \
  (((PGL_format_t)PGL_find_name((MACRO_TEMP_1),(MACRO_TEMP_2)))->length)

#define PGL_type(MACRO_TEMP_1,MACRO_TEMP_2) \
  (((PGL_format_t)PGL_find_name((MACRO_TEMP_1),(MACRO_TEMP_2)))->type)

#define PGL_length_and_type(MACRO_FORMAT,MACRO_NAME,MACRO_LEN,MACRO_TYPE)\
{PGL_format_t MACRO_TEMP=(PGL_format_t)\
   PGL_find_name((MACRO_FORMAT),(MACRO_LEN));\
   (MACRO_LEN) = MACRO_TEMP->length;\
   (MACRO_LEN) = MACRO_TEMP->type;\
}


#define S(len) ((len==32)?23:52)
#define E(len) ((len==32)?8:11)

#define MAX(x,y) ((x)>(y)?(x):(y))
#define MIN(x,y) ((y)>(x)?(x):(y))

#define NEXT32(x) (((((x)-1)/32)+1)*32)
PGL_display_t PGL_allocate_display();
PGL_buffer_t PGL_allocate_buffer();
PGL_buffer_t PGL_wrap_buffer_around_field();
PGL_format_t PGL_generate_transfer_format();
PGL_format_t PGL_translate_format();
PGL_spawn_frame_t PGL_open_spawn();
PGL_spawn_frame_t PGL_open_scan_line();

typedef void (*function_pointer)();
