#include "pgl_internal.h"

extern PGL_fill_triangle();
extern PGL_fill_line();

/* overlap would be nice*/
void PGL_news_normals(dest,source,d_format,s_format,a1,a2)
     CM_field_id_t dest,source;
     PGL_format_t d_format,s_format;
     int a1,a2;
{ 
  unsigned xl= PGL_length(s_format,PGL_x);
  unsigned yl= PGL_length(s_format,PGL_y);
  unsigned zl= PGL_length(s_format,PGL_z);
  unsigned Xl= PGL_length(d_format,PGL_X);
  unsigned Yl= PGL_length(d_format,PGL_Y);
  unsigned Zl= PGL_length(d_format,PGL_Z);

  unsigned xt= PGL_type(s_format,PGL_x);
  unsigned yt= PGL_type(s_format,PGL_y);
  unsigned zt= PGL_type(s_format,PGL_z);
  unsigned Xt= PGL_type(d_format,PGL_X);
  unsigned Yt= PGL_type(d_format,PGL_Y);
  unsigned Zt= PGL_type(d_format,PGL_Z);

  CM_field_id_t x = PGL_extract(source,s_format,PGL_x);
  CM_field_id_t y = PGL_extract(source,s_format,PGL_y);
  CM_field_id_t z = PGL_extract(source,s_format,PGL_z);

  CM_field_id_t X = PGL_extract(dest,d_format,PGL_X);
  CM_field_id_t Y = PGL_extract(dest,d_format,PGL_Y);
  CM_field_id_t Z = PGL_extract(dest,d_format,PGL_Z);

  CM_field_id_t gx=CM_allocate_stack_field(xl);
  CM_field_id_t gy=CM_allocate_stack_field(yl);
  CM_field_id_t gz=CM_allocate_stack_field(zl);
  CM_field_id_t tx=CM_allocate_stack_field(xl);
  CM_field_id_t ty=CM_allocate_stack_field(yl);
  CM_field_id_t tz=CM_allocate_stack_field(zl);
  CM_field_id_t gN=CM_allocate_stack_field(MAX(MAX(Xl,Yl),Zl));
    
  CM_get_from_news_always_1L (gx,x,a1,CM_upward,xl);
  CM_get_from_news_always_1L (gy,y,a1,CM_upward,yl);
  CM_get_from_news_always_1L (gz,z,a1,CM_upward,zl);
  
  PGL_subtract_2_1L(gx,x,xt,xl);
  PGL_subtract_2_1L(gy,y,yt,yl);
  PGL_subtract_2_1L(gz,z,zt,zl);

  CM_get_from_news_always_1L (tx,x,a2,CM_upward,xl);
  CM_get_from_news_always_1L (ty,y,a2,CM_upward,yl);
  CM_get_from_news_always_1L (tz,z,a2,CM_upward,zl);
  
  PGL_subtract_2_1L(tx,x,xt,xl);
  PGL_subtract_2_1L(ty,y,yt,yl);
  PGL_subtract_2_1L(tz,z,zt,zl);

  PGL_multiply_3_3L (X,gy,tz,Xt,yt,zt,Xl,yl,zl);
  PGL_multiply_3_3L (gN,ty,gz,Xt,yt,zt,Xl,yl,zl);
  PGL_subtract_2_1L (X,gN,Xt,Xl);
  
  PGL_multiply_3_3L (Y,tz,gx,Yt,zt,xt,Yl,zl,xl);
  PGL_multiply_3_3L (gN,gz,tx,Yt,zt,xt,Yl,zl,xl);
  PGL_subtract_2_1L (Y,gN,Yt,Yl);
  
  PGL_multiply_3_3L (Z,gx,ty,Zt,xt,yt,Zl,xl,yl);
  PGL_multiply_3_3L (gN,tx,gy,Zt,xt,yt,Zl,xl,yl);  
  PGL_subtract_2_1L (Z,gN,Zt,Zl);

  CM_deallocate_stack_through(gx);
}


PGL_spawn_frame_t PGL_open_surface (dest,format,size1,size2,flags)
     PGL_buffer_t dest;
     PGL_format_t format;
     unsigned size1,size2;
     unsigned flags;
{
  PGL_format_t i_f=PGL_generate_transfer_format(dest,format,flags);
  unsigned point_length;
  PGL_spawn_frame_t frame;
  point_length = PGL_point_length(i_f);
  frame=PGL_open_spawn(dest,PGL_fill_triangle,i_f,size1,3*point_length,flags);
  
  if (flags & PGL_use_scan_lines)
    frame->next=PGL_open_scan_line(dest,i_f,2*point_length,flags);
  else
    frame->next=PGL_open_spawn(dest,PGL_fill_line,i_f,size2,
			       2*point_length,flags);
  return (frame);
}


void PGL_write_surface (frame,format,p1)
     PGL_spawn_frame_t frame;
     PGL_format_t format;
     CM_field_id_t p1;
{
  unsigned point_length = PGL_point_length (frame->format);
  CM_field_id_t x1=CM_allocate_stack_field (4*point_length);
  CM_field_id_t x2=CM_add_offset_to_field_id(x1,point_length);
  CM_field_id_t x3=CM_add_offset_to_field_id(x2,point_length);
  CM_field_id_t x4=CM_add_offset_to_field_id(x3,point_length);
  CM_field_id_t entry_context = CM_allocate_stack_field(1);
  int a1,a2;

  CM_store_context(entry_context);
  CM_set_context();   

  if (frame->flags & PGL_fortran_geometry) {
    a1=1; a2=2;
  } else {
    a1=0; a2=1;
  }

  PGL_generate_transfer_field(x1,p1,frame->dest,frame->format,format,
			      frame->flags);
  
  if ((frame->dest->lights)  && (!(PGL_find_name(format,PGL_X)||
				   PGL_find_name(format,PGL_Y)||
				   PGL_find_name(format,PGL_Z)))){
    PGL_news_normals (x1,p1,frame->format,format,a1,a2);
    CM_f_divide_constant_2_1L (PGL_extract(x1,frame->format,PGL_X),
			       (float)frame->dest->width,23,8);
    CM_f_divide_constant_2_1L (PGL_extract(x1,frame->format,PGL_Y),
			       (float)frame->dest->height,23,8);
    CM_f_divide_constant_2_1L (PGL_extract(x1,frame->format,PGL_Z),
			       (float)frame->dest->depth,23,8);
  }
  
  CM_get_from_news_always_1L (x2,x1,a1,CM_upward,point_length);
  CM_get_from_news_always_1L (x3,x1,a2,CM_upward,2*point_length);
  
  CM_load_context(entry_context);

  if (frame->dest->combiner != PGL_sgi_render){
    PGL_plot_triangle_internal (frame,frame->dest,frame->format,x1);
    PGL_plot_triangle_internal (frame,frame->dest,frame->format,x2);
  } else {
    PGL_sgi_polygons (3,frame->flags,frame->format,x1);
    PGL_sgi_polygons (3,frame->flags,frame->format,x2);
  }
  CM_deallocate_stack_through (x1);
}


void PGL_close_surface (frame)
     PGL_spawn_frame_t frame;
{
  PGL_flush_spawn (frame);
  if (frame->flags & PGL_use_scan_lines)
    PGL_close_scan_line (frame->next);
  else
    PGL_close_spawn(frame->next);
  PGL_close_spawn(frame);
}


void PGL_plot_surface(dest,flags,format_string,p1)
     PGL_buffer_t dest;
     unsigned flags;
     char *format_string;
     CM_field_id_t p1;
{
  PGL_format_t format=(PGL_format_t)PGL_translate_format(format_string);
  PGL_spawn_frame_t frame;
  
  frame=PGL_open_surface(dest,format,65536,262144,flags);
  PGL_write_surface (frame,format,p1);
  PGL_close_surface(frame);
  PGL_free_format (format);
}

