#include "pgl_internal.h"
#include <math.h>

void PGL_compose (sdest,source2)
     float sdest[4][4],source2[4][4];
{
  float temp[4][4];
  int i,j,ii;
  
  for (i=0;i<4;i++)
    for (j=0;j<4;j++){
      temp[i][j]=sdest[i][j];
      sdest[i][j]=0.0;
    }
  for (i=0;i<4;i++)
    for (j=0;j<4;j++)
      for (ii=0;ii<4;ii++)
	sdest[i][j]+=source2[ii][j]*temp[i][ii];
}


void PGL_print_transform (x)
     float x[4][4];
{
  int i,j;
  for (i=0;i<4;i++){
    for (j=0;j<4;j++)
      printf ("%f ",x[i][j]);
    printf ("\n");
  }
  printf ("\n");
}


/* identity could be a static array*/
void PGL_identity (matrix)
     float matrix[4][4];
{
  int i,j;

  for (i=0;i<4;i++)
    for (j=0;j<4;j++)
      if (i==j)
	matrix[i][j]=1.0;
      else
	matrix[i][j]=0.0;
}


void PGL_rotate(xform,axis,theta)
     float xform[4][4];
     unsigned axis;
     float theta;
{
  float temp[4][4];
  unsigned a1 =((axis-PGL_x)+1)%3;
  unsigned a2 =((axis-PGL_x)+2)%3;
  PGL_identity (temp);
  temp[a2][a2]=temp[a1][a1]=cos(theta);
  temp[a1][a2]=(-(temp[a2][a1]=sin(theta)));
  
  PGL_compose (xform,temp);
}

void PGL_scale(xform,axis,amount)
     float xform[4][4];
     unsigned axis;
     float amount;
{
  float temp[4][4];
  
  PGL_identity (temp);
  temp[axis-PGL_x][axis-PGL_x]=amount;  
  PGL_compose (xform,temp);
}

void PGL_translate(xform,axis,amount)
     float xform[4][4];
     unsigned axis;
     float amount;
{
  float temp[4][4];
  
  PGL_identity (temp);
  temp[3][axis-PGL_x]=amount; 
  PGL_compose (xform,temp);
}


void PGL_perspective (xform,amount)
{
  float temp[4][4];
  
  PGL_identity (temp);
  temp[3][2]=amount;  
  PGL_compose (xform,temp);
}


void PGL_fe_transform (dest,source,xform)
     float dest[3],source[3],xform[4][4];
{
  int i,j;
  float w=xform[3][3];
  float temp[4];

  temp[0]=source[0];
  temp[1]=source[1];
  temp[2]=source[2];
  temp[3]=1.0;
  
  for (i=0;i<4;i++) {
    temp[i]=xform[3][i];
    for (j=0;j<3;j++)
      temp[i]+=xform[j][i]*source[j];
  }
  for (i=0;i<3;i++)
    temp[i]/=temp[3];
  
  dest[0]=temp[0];
  dest[1]=temp[1];
  dest[2]=temp[2];
}

void PGL_transform (dest,source,xform,length)
     CM_field_id_t dest,source;
     float xform[4][4];
{
  int i,j;
  CM_field_id_t temp = CM_allocate_stack_field(length*4);

  for (i=0;i<4;i++) {
    CM_f_move_constant_1L (CM_add_offset_to_field_id(temp,i*length),
			   xform[3][i],23,8);
    for (j=0;j<3;j++)
      CM_f_mult_const_add_1L (CM_add_offset_to_field_id(temp,i*length),
			      CM_add_offset_to_field_id(source,j*length),
			      xform[j][i],
			      CM_add_offset_to_field_id(temp,i*length),
			      S(length),E(length));
  }
  for (i=0;i<3;i++)
    CM_f_divide_2_1L (CM_add_offset_to_field_id(temp,i*length),
		      CM_add_offset_to_field_id(temp,3*length),
		      S(length),E(length));
  CM_u_move_1L (dest,temp,length*3);
  CM_deallocate_stack_through(temp);
}


void PGL_normalize(vector)
     float vector[3];
{
  float norm=sqrt(vector[0]*vector[0]+vector[1]*vector[1]+vector[2]*vector[2]);
  vector[0]/=norm;
  vector[1]/=norm;
  vector[2]/=norm;
}


void PGL_cross(dest,vector1,vector2)
     float dest[3],vector1[3],vector2[2];
{
  dest[0] = vector1[1]*vector2[2] - vector1[2]*vector2[1];
  dest[1] = vector1[2]*vector2[0] - vector1[0]*vector2[2];
  dest[2] = vector1[0]*vector2[1] - vector1[1]*vector2[0];
}

