#include <geodata.h>
#include <geodata_private.h>
#include <time.h>
#include <math.h>


float GS_range(x,x0,x1)
     float x,x0,x1;
{
  int i=0;
  float width=x1-x0;

  if (x < x0)  i = (int)((x-x0)/width)-1;
  else if (x >= x1) 
    i = (int)((x-x1)/width)+1;
  
  if (i != 0)
    return (x-(float)i*width);
  else
    return(x);
}


float GS_same_quadrant(x,y)
     float x,y;
  {
  float temp;
  int quad_x, quad_y;

  temp = GS_range(x,0.0,(2.0*GS_PI));
  quad_x = (int)(temp*0.636619772);
  quad_y = (int)(y*0.636619772);
  return(temp+(float)(quad_y-quad_x)*1.570796327);
}



float GS_g_h_a(t)
     float t;
{
  float temp;
  temp = (float)(int)t;
  return(GS_range(4.894961212+0.017202791*temp+5.0752099e-15*temp*temp+
		  6.300038809*(t-temp),0.0,(2.0*GS_PI)));
}


void GS_solar_position(dest,t)
     float *dest,t;
{

  float mean_long,mean_anomaly,ecliptic_long,obliquity,right_ascen,
  declination,distance,time_offset,gmst,sun_lon,sun_lat;

  mean_long = GS_range(4.894950+0.01720279*t,0.0,(2.0*GS_PI));
  mean_anomaly = GS_range(6.240041+0.01720197*t,0.0,(2.0*GS_PI));
  ecliptic_long = GS_range(mean_long+0.033423*sin(mean_anomaly)+
                         0.000349*sin(2.0*mean_anomaly),0.0,(2.0*GS_PI));
  obliquity = 0.409088-6.0e-9*t;
  right_ascen = GS_same_quadrant(atan(cos(obliquity)*tan(ecliptic_long)),
                               ecliptic_long);
  declination = asin(sin(obliquity)*sin(ecliptic_long));
  distance =(1.00014-0.01671*cos(mean_anomaly)-0.00014*cos(2.0*mean_anomaly));
  time_offset = 0.159154943*(mean_long-right_ascen);
  gmst = GS_g_h_a(t);
  dest[1] = GS_range(right_ascen-gmst,0.0,(2.0*GS_PI));
  dest[0] = GS_SOLAR_DISTANCE*cos(declination)*cos(dest[1]);
  dest[1] = GS_SOLAR_DISTANCE*cos(declination)*sin(dest[1]);
  dest[2] = GS_SOLAR_DISTANCE*sin(declination);
}



float GS_dayssinceJ2000(year,month,day,hour,minute,second)
       int year,month,day,hour,minute,second;
{
  int itemp;
  float rtemp;
  
  if (year > 50) 
    itemp = 365*(year-100)+day+31*(month-1);
  else
    itemp = 365*year+day+31*(month-1);
  
  if (month >= 3) 
    itemp = itemp-(int)(0.4*(float)month+2.3)+
      (year / 4)-(3*((year / 100)+1)) / 4;
  else
    itemp = itemp+((year-1) / 4)-(3*((year-1) / 100)+1) / 4;
  rtemp = (float)itemp+(float)hour/24.0+(float)minute/1440.0+
    (float)second/86400.0;
  return (rtemp-25.5);
}



void GS_light(dest,scene)     
     float *dest;
     GS_scene_t scene;
{
  long tval;
  struct tm *my_time;

  switch(scene->light_type){
  case GS_AMBIENT:
    dest[0]=dest[1]=dest[2]=0.0;
    break;
    
  case GS_SUN:
    GS_solar_position(dest,GS_dayssinceJ2000(scene->year,
					     scene->month,
					     scene->day,
					     scene->hour,
					     scene->minute,
					     scene->second));    
    break;
    
  case GS_CURRENT_SUN:
    tval = time(0);
    my_time = (struct tm *)gmtime(&tval);
    GS_solar_position(dest,GS_dayssinceJ2000(my_time->tm_year-6,
					     my_time->tm_mon,
					     my_time->tm_mday,
					     my_time->tm_hour,
					     my_time->tm_min,
					     my_time->tm_sec)); 
    
    break;
    
  case GS_POINT:
    dest[0]=scene->light_source[0];
    dest[1]=scene->light_source[1];
    dest[2]=scene->light_source[2];
    break;
  }
}
