#include <geodata.h>
#include <geodata_private.h>

static unsigned state;
static unsigned format=0;

void GS_symbol_open (s_dest,s_priority,memory,s_flags)
     CM_field_id_t s_dest;
     unsigned s_priority;     
     int memory;
     unsigned s_flags;
{
  char format_string[255];
    if (format == 0){
      sprintf (format_string,"xf yf zf ru8 gu8 bu8 Tu(%d)",s_priority);
      format = PGL_translate_format (format_string);
    }
  state = PGL_open_triangle (s_dest,format,
			     (1<<(CM_physical_cube_address_length+memory)),
			     (1<<(CM_physical_cube_address_length+1+memory)),
			     s_flags);
}


void GS_symbol_close()
{
 PGL_close_triangle (state);
}


void GS_plot_symbol (symbol,position,rotation,scale)
     GS_symbol_t symbol;
     CM_field_id_t position,rotation,scale;
{
  GS_polygon_t i;
  GS_vertex_t j;
  int k;
  CM_field_id_t triangle=CM_allocate_stack_field(3*128);
  
  for (i=symbol->polygons; i != (GS_polygon_t)0 ; i=i->next){
    for (j=i->vertices,k=0; j!=(GS_vertex_t)0 ; j=j->next,k++){
      CM_f_sin_2_1L (VREF(triangle,(k*4)+3),rotation,23,8);
      CM_f_cos_2_1L (VREF(triangle,(k*4)+1),rotation,23,8);

      CM_f_multiply_constant_3_1L (VREF(triangle,(k*4)),
				   VREF(triangle,(k*4)+1),j->point[0],23,8);
      CM_f_mult_const_add_1L (VREF(triangle,(k*4)),VREF(triangle,(k*4)+3),
			      -(j->point[1]),VREF(triangle,k*4),23,8);
      CM_f_mult_add_1L (VREF(triangle,(k*4)),VREF(triangle,(k*4)),
			scale,position,23,8);
      
      CM_f_multiply_constant_2_1L (VREF(triangle,(k*4)+1),
				   j->point[1],23,8);
      CM_f_mult_const_add_1L (VREF(triangle,(k*4)+1),VREF(triangle,(k*4)+3),
			      j->point[0],VREF(triangle,(k*4)+1),23,8);
      CM_f_mult_add_1L (VREF(triangle,(k*4)+1),VREF(triangle,(k*4)+1),
			scale,VREF(position,1),23,8);
      
      CM_f_mult_const_add_1L (VREF(triangle,(k*4)+2),scale,j->point[2],
			      VREF(position,2),23,8);
      CM_u_move_1L (VREF(triangle,(k*4)+3),VREF(position,3),32);
    }
    PGL_write_triangle (state,format,triangle,VREF(triangle,4),
			VREF(triangle,8));
  }
  CM_deallocate_stack_through(triangle);
}


void GS_symbol_write (data,rep)
     CM_field_id_t data;
     GS_representation_t rep;
{
  GS_table_t k;
  GS_symbol_t symbol;
  
  CM_field_id_t entry_context = CM_allocate_stack_field(1);  
  CM_field_id_t rotation = CM_allocate_stack_field(32);
  CM_field_id_t scale = CM_allocate_stack_field(32);
  CM_field_id_t temp = CM_allocate_stack_field(32);
  

/*  printf("on entry to GS_symbol_write\n");
  GD_transfer_statistics(4,data);*/
  CM_store_context (entry_context);
  CM_u_move_1L(temp,VREF(data,3),32);

  CM_f_lt_constant_1L (temp,rep->tables->value,23,8);
  CM_logand_context_with_test();
  CM_f_move_constant_1L (rotation,rep->tables->rotation,23,8);
  CM_f_move_constant_1L (scale,rep->tables->scale,23,8);

  GS_color_value (VREF(data,3),temp,0,rep->tables,rep->what_varies);
  GS_plot_symbol (GB_get_object(rep->tables->discrete),data,rotation,scale);
  
  for (k=rep->tables;k->next!=(GS_table_t)0;k=k->next){
    CM_load_context (entry_context);
    CM_f_ge_constant_1L (temp,k->value,23,8);
    CM_logand_context_with_test();
    CM_f_lt_constant_1L (temp,k->next->value,23,8);
    CM_logand_context_with_test();

    /*    printf ("%f (%f %f)\n", k->value,CM_global_f_min_1L(temp,23,8),
	    CM_global_f_max_1L(temp,23,8));*/
    /* use PGL_transform*/
    if (rep->what_varies == GS_ROTATION_VARIES){
      CM_f_sub_const_mult_const_1L(rotation,temp,k->value,
				   1.0/(float)((float)k->next->value -
					       (float)k->value),23,8);
      CM_f_mult_const_add_const_1L (rotation,rotation,
				    (float)((float)k->next->rotation -
					    (float)k->rotation),
				    k->rotation,23,8);
/*      printf("interpolate  val/rot  %f %f %f %f \n",k->value,k->rotation,
	     (float)k->next->value,(float)k->next->rotation);
      printf("delta  val/rot   %f %f \n",
	     ((float)k->next->value - (float)k->value),
	     ((float)k->next->rotation - (float)k->rotation));*/
      }  else CM_f_move_constant_1L (rotation,k->rotation,23,8);
    
    if (rep->what_varies == GS_SCALE_VARIES){
      CM_f_sub_const_mult_const_1L(scale,temp,k->value,
				   1.0/(float)(k->value-k->next->value),23,8);
      CM_f_mult_const_add_const_1L (scale,scale,
				    (float)(k->scale-k->next->scale),
				    k->next->scale,23,8);
    } else CM_f_move_constant_1L (scale,k->scale,23,8);

/*    printf ("rotation: %f %f %\n",CM_global_f_min_1L(rotation,23,8),
	    CM_global_f_max_1L(rotation,23,8));*/
    
    GS_color_value (VREF(data,3),temp,k,k->next,rep->what_varies);
    GS_plot_symbol (GB_get_object(k->discrete),data,rotation,scale);
  } 
  CM_load_context(entry_context);

  CM_f_ge_constant_1L (temp,k->value,23,8);
  CM_logand_context_with_test();
/*  printf("baddies\n");
  GD_transfer_statistics(4,data);*/
  CM_f_move_constant_1L (rotation,k->rotation,23,8);
  CM_f_move_constant_1L (scale,k->scale,23,8);
  GS_color_value (VREF(data,3),temp,k,0,rep->what_varies);

  GS_plot_symbol (GB_get_object(k->discrete),data,rotation,scale);
  CM_load_context(entry_context); 
  CM_deallocate_stack_through (entry_context);
}


