#include "xtool.h"

extern Widget widget_array[];
extern Window curr_window;
extern GX_xtool_projection_t curr_proj;
extern GX_scene_list_t scenes;
extern GX_scene_list_t curr_scene;
extern int cont_reset;
extern int file_mode;
extern int act_scales;
extern int cont_updt;
extern int cont_disp;
extern int appl_running;
extern int preserve_scene;
extern char curr_display[];
extern int x_offset;
extern int y_offset;
extern int displays[];

extern XmString GX_string_create();
extern int *GX_get_instance();
extern void GX_output_command();
extern Boolean GX_string_read();
extern void GX_reset_widget_list();
extern void GX_reset_movie_list();
extern char *XmTextGetString();


/**************************************
   Control Panel CallBacks...
 **************************************/

void display_callback(widget, tag, data)
     Widget widget;
     int *tag;
     XmToggleButtonCallbackStruct *data;
{
  char command[GX_COM_LEN], name[GB_LABEL_LENGTH], type[GB_LABEL_LENGTH];
  int index = *tag - k_cp_cmfb_tb;
  int size = 1024;

  if (data->set) {
    switch (*tag) {
    case k_cp_cmfb_tb: size = 1024; sprintf (type, "cmfb");
      sprintf (name, "X_CMFB_DISPLAY"); break;
    case k_cp_smfb_tb: size = 1024; sprintf (type, "smooth_cmfb");
      sprintf (name, "X_SMFB_DISPLAY"); break;
    case k_cp_xwin_tb: size = 512; sprintf (type, "xwindows");
      sprintf (name, "X_XWIN_DISPLAY"); break;
    case k_cp_glwn_tb: size = 512; sprintf (type, "glwindows");
      sprintf (name, "X_GLWN_DISPLAY"); break;
    default: ;
    }
    sprintf (curr_display, "%s", name);
    if (displays[index] == 0) {
      displays[index] = 1;
      sprintf (command, "(display %s (display_type %s) (size %d %d))",
	       name, type, size, size);
      GX_output_command (command,0,0);
    }
  }
}

void offset_callback(widget, tag, data)
     Widget widget;
     int *tag;
     XmScaleCallbackStruct *data;
{
  static Arg args[1];
  static char string[15];

  switch (*tag) {
  case k_cp_xoff_sc:
    x_offset = 16 * data->value;
    sprintf (string, "X Off: %d", x_offset);
    XtSetArg (args[0], XmNvalue, string);
    XtSetValues(widget_array[k_cp_xoff_tx], args, 1);
    if (curr_scene) curr_scene->x_off = x_offset;
    break;
  case k_cp_yoff_sc:
    y_offset = 16 * data->value;
    sprintf (string, "Y Off: %d", y_offset);
    XtSetArg (args[0], XmNvalue, string);
    XtSetValues(widget_array[k_cp_yoff_tx], args, 1);
    if (curr_scene) curr_scene->y_off = y_offset;
    break;
  }
}

void activate_callback (widget, tag, data)
     Widget widget;
     char *tag;
     XmAnyCallbackStruct *data;
{
  static char command[GX_COM_LEN];
  char *text;

  if (data->reason == XmCR_ACTIVATE) {
    text = XmTextGetString (widget);
    sprintf (command, "%s", text);
    XtFree (text);
    GX_output_command (command,0,0);
  }
}

void menu_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmAnyCallbackStruct *data;
{
  char command[GX_COM_LEN];
  int output = 1, quit = 0;

  switch (*tag) {
  case k_mb_load:
    if (!(XtIsManaged(widget_array[k_fil_sel_ls])))
      XtManageChild(widget_array[k_fil_sel_ls]);
    file_mode = LOAD_MODE;
    output = 0;
    break;
  case k_mb_save:
    if (!(XtIsManaged(widget_array[k_fil_sel_ls])))
      XtManageChild(widget_array[k_fil_sel_ls]);
    file_mode = SAVE_MODE;
    output = 0;
    break;
  case k_mb_quit: sprintf (command, "(die Xtool-Quit)"); quit=1; break;
  case k_mb_clear: sprintf (command, ";;; X_Command_Clear"); break;
  case k_mb_cut: sprintf (command, ";;; X_Command_Cut"); break;
  case k_mb_copy: sprintf (command, ";;; X_Command_Copy"); break;
  case k_mb_paste: sprintf (command, ";;; X_Command_Paste"); break;
  case k_mb_select: sprintf (command, ";;; X_Command_Select_All"); break;
  case k_mb_scene:
    if (!(XtIsRealized(widget_array[k_se_shell]))) {
      XtManageChild (widget_array[k_se_form]);
      XtRealizeWidget (widget_array[k_se_shell]);
    }else XtUnrealizeWidget(widget_array[k_se_shell]);
    output = 0;
    break;
  case k_mb_rep:
    if (!(XtIsRealized(widget_array[k_re_shell]))) {
      XtManageChild (widget_array[k_re_form]);
      XtRealizeWidget (widget_array[k_re_shell]);
    } else XtUnrealizeWidget(widget_array[k_re_shell]);
    output = 0;
    break;
  case k_mb_proj:
    if (!(XtIsRealized(widget_array[k_pp_shell]))) {
      XtManageChild (widget_array[k_pp_form]);
      XtRealizeWidget (widget_array[k_pp_shell]);
    }else XtUnrealizeWidget(widget_array[k_pp_shell]);
    output = 0;
    break;
  case k_mb_vid:
    if (!(XtIsRealized(widget_array[k_vp_shell]))) {
      GB_execute_open_video_display((GB_open_video_display_t)0,
				    (GB_parse_frame_t)0);
      if (!(XtIsManaged(widget_array[k_vp_form]))) {
	VP_set_video_list(widget_array[k_vp_vlst_sb]);
	XtManageChild (widget_array[k_vp_form]);
      }
      XtRealizeWidget (widget_array[k_vp_shell]);
    }
    else {
      GB_execute_close_video_display((GB_close_video_display_t)0,
				 (GB_parse_frame_t)0);
      XtUnrealizeWidget(widget_array[k_vp_shell]);
    }
    output = 0;
    break;
  case k_mb_upd_sc: GX_update_scene (command); break;
  case k_cp_updt_pb: GX_update_scene (command); break;
  case k_mb_dis_sc: GX_display_scene (command); break;
  case k_mb_detach: sprintf (command, "(detach Xtool-Detach)"); break;
  default: ;
  }
  if (output != 0) GX_output_command (command,0,1);
  if (quit) { GB_execute_close_video_display
		((GB_close_video_display_t)0,(GB_parse_frame_t)0);
	      exit (1);
	    }
}

void update_all_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmAnyCallbackStruct *data;
{
  char command[GX_COM_LEN];
  GX_scene_list_t temp_scene = curr_scene;

  curr_scene = scenes;
  while (curr_scene != NULL) {
    sprintf (command, "");
    GX_update_scene (command);
    GX_output_command (command,0,1);
    curr_scene = curr_scene->next;
  }
  curr_scene = temp_scene;
}

void file_cancel_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmFileSelectionBoxCallbackStruct *data;
{
  if (XtIsManaged(widget_array[k_fil_sel_ls]))
      XtUnmanageChild(widget_array[k_fil_sel_ls]);
  file_mode = NONE_MODE;
}  

void file_select_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmFileSelectionBoxCallbackStruct *data;
{
  char command[GX_COM_LEN];
  char *file[GD_FILE_NAME_LENGTH];

  GX_string_read (data->value, file);
  switch (file_mode) {
  case LOAD_MODE:
    if (!(cont_reset)) {
      GB_load_object_file (*file);
      GX_reset_widget_list (widget_array[k_se_sclist], GB_SCENE);
      GX_reset_widget_list (widget_array[k_se_replist], GB_REPRESENTATION);
      GX_reset_widget_list (widget_array[k_se_datlist], GB_DATA_DESC);
      GX_reset_movie_list ();
    }
    sprintf (command, "(include X_INCLUDE (file \"%s\"))", *file);
    if (data->reason == XmCR_OK)
      file_cancel_callback (widget, tag, data);
    GX_output_command (command,0,0);
    break;
  case SAVE_MODE:
    sprintf (command, "(recur_print X_SAVE (obj_id %s)\n", curr_scene->label);
    sprintf (command, "%s      (out_file \"%s\"))", command, *file);
    file_cancel_callback (widget, tag, data);
    GX_output_command (command,0,1);
    break;
  default: ;
  }
}

void step_application_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmDrawingAreaCallbackStruct *data;
{
  Arg args[1];
  XmString string;
  char command[GX_COM_LEN];

  switch (*tag) {
  case k_mb_stp_ap:
    if (appl_running)
      sprintf (command, ";;; X Application already running");
    else{
      sprintf (command, ";;; X Step Application\n");
      sprintf (command, "%s\n(handshake handshake-foo)",command);
    }
    break;
  case k_mb_run_ap:
    if (appl_running) {
      appl_running = 0;
      sprintf (command, ";;; Application Paused");
      string = XmStringLtoRCreate ("Continue Application","");
      XtSetArg (args[0], XmNlabelString, string);
      XtSetValues(widget, args, 1);
    }
    else {
      appl_running = 1;
      sprintf (command, ";;; Application Running\n");
      sprintf (command, "%s\n(handshake handshake-foo)",command);
      string = XmStringLtoRCreate ("Pause Application","");
      XtSetArg (args[0], XmNlabelString, string);
      XtSetValues(widget, args, 1);
    }
    break;
  default: ;
  }
  GX_output_command (command,0,0);
}

void continuous_update_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmAnyCallbackStruct *data;
{
  Arg args[2];
  XmString string;

  if (cont_updt) {
    cont_updt = 0;
    string = XmStringLtoRCreate ("Auto Update","");
    XtSetArg (args[0], XmNlabelString, string);
    XtSetValues(widget, args, 1);
  }
  else {
    cont_updt = 1;
    string = XmStringLtoRCreate ("Manual Update","");
    XtSetArg (args[0], XmNlabelString, string);
    XtSetValues(widget, args, 1);
  }
}

void continuous_display_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmAnyCallbackStruct *data;
{
  Arg args[2];
  XmString string;

  if (cont_disp) {
    cont_disp = 0;
    string = XmStringLtoRCreate ("Auto Display","");
    XtSetArg (args[0], XmNlabelString, string);
    XtSetValues(widget, args, 1);
  }
  else {
    cont_disp = 1;
    string = XmStringLtoRCreate ("Manual Display","");
    XtSetArg (args[0], XmNlabelString, string);
    XtSetValues(widget, args, 1);
  }
}

void preserve_scene_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmAnyCallbackStruct *data;
{
  Arg args[2];
  XmString string;

  if (preserve_scene) {
    preserve_scene = 0;
    string = XmStringLtoRCreate ("Preserve Scene","");
    XtSetArg (args[0], XmNlabelString, string);
    XtSetValues(widget, args, 1);
  }
  else {
    preserve_scene = 1;
    string = XmStringLtoRCreate ("Preserve Controls","");
    XtSetArg (args[0], XmNlabelString, string);
    XtSetValues(widget, args, 1);
  }
}

void movie_callback (widget, client, data)
     Widget widget;
     caddr_t client;
     XmAnyCallbackStruct *data;
{
  Arg args[1];
  XmString string;
  char command[GX_COM_LEN], *label[GB_LABEL_LENGTH];

  XtSetArg (args[0], XmNlabelString, &string);
  XtGetValues(widget, args, 1);
  GX_string_read (string, label);
  sprintf (command, "(play_movie X_PLAY_MOV\n\t(movie_id %s)\n", *label);
  sprintf (command, "%s\t(display_id %s))", command, curr_display);
  GX_output_command (command,0,0);
}

void disp_expose_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmDrawingAreaCallbackStruct *data;
{
  char command[GX_COM_LEN];
  if (!curr_window)
    curr_window = data->window;
  sprintf (command, ";;; Expose Display Event");
  GX_output_command (command,0,1);
}

void disp_resize_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmDrawingAreaCallbackStruct *data;
{
  XWindowAttributes attributes;  
  char command[GX_COM_LEN];

  XGetWindowAttributes(XtDisplay(widget), data->window, &attributes);
  sprintf (command, "(scene %s (dimensions %d %d))", curr_scene->label,
	   attributes.width, attributes.height);
  GX_output_command (command,0,0);
}
