#include "xtool.h"

extern Widget widget_array[];
extern Window curr_window;
extern GX_xtool_projection_t curr_proj;
extern int cont_reset;
extern int file_mode;
extern int act_scales;
extern int cont_updt;
extern int cont_disp;
extern int appl_running;
extern int preserve_scene;
extern GX_scene_list_t curr_scene;
extern char curr_display[];
extern int x_offset;
extern int y_offset;
extern int displays[];
float GX_resl = 1.0;

extern int power();
extern XmString GX_string_create();
extern int *GX_get_instance();
extern void GX_output_command();
extern Boolean GX_string_read();
extern void GX_reset_widget_list();
extern void GX_reset_movie_list();
extern char *XmTextGetString();


/**************************************
   Projection Panel CallBacks...
 **************************************/

void change_parameters (code, params)
     int code;
     float params[];
{
  static char command[GX_COM_LEN];
  static GS_scene *scene = NULL;
  int i;

  if (curr_scene) {
    scene = (GS_scene *)
      GX_get_instance ("Change_Parameters", GB_SCENE, curr_scene->label);
    if (code) {
      sprintf (command, "(scene %s\n\t(projection %d)\n\t(parameters",
	       curr_scene->label, code);
      if (scene) scene->projection = curr_proj->GX_proj_code;
    } else
      sprintf (command, "(scene %s\n\t(parameters", curr_scene->label);
    for (i=0; i<NO_PROJECTION_PARAMS; i++) {
      sprintf (command, "%s %g", command, params[i]);
      if (scene) scene->parameters[i] = params[i];
    }
    sprintf (command, "%s))", command);
    GX_output_command (command,0,0);
  }
}

void projection_callback (widget, tag, data)
     Widget widget;
     char *tag;
     XmToggleButtonCallbackStruct *data;
{
  Arg args[10];
  static char string[40];
  int i, j, n, d;
  float params[NO_PROJECTION_PARAMS];

  if (data->set) {
    curr_proj = (GX_xtool_projection_t)
      GX_get_instance ("projection_callback", GB_XTOOL_PROJECTION, tag);

    sprintf (string, "Scale Resolution: 1");
    XtSetArg (args[0], XmNvalue, string);
    XtSetValues(widget_array[k_pp_resl_tx], args, 1);
    GX_resl = 1.0;

    XtSetArg (args[0], XmNvalue, 100);
    XtSetValues(widget_array[k_pp_resl_sc], args, 1);

    for (j=0; j<NO_PROJECTION_PARAMS; j++) {
      params[j] = (curr_proj->GX_defaults[j]) * (curr_proj->GX_convert[j]);
      for (i=0; i<GX_NO_SCALES; i++)
	if (curr_proj->GX_scale_map[i] == j) {
	  curr_proj->GX_decimals[j] = curr_proj->GX_precision[j];
	  sprintf (string, "%s: %.*f", curr_proj->GX_labels[j],
		   curr_proj->GX_decimals[j], curr_proj->GX_defaults[j]);
	  XtSetArg (args[0], XmNvalue, string);
	  XtSetValues(widget_array[i+k_pp_par1_tx], args, 1);
	  n = 0;
	  d = power (10, (curr_proj->GX_decimals[j]));
	  XtSetArg (args[n], XmNminimum,(int)(curr_proj->GX_min[j]*d)); n++;
	  XtSetArg (args[n], XmNmaximum,(int)(curr_proj->GX_max[j]*d)); n++;
	  XtSetArg (args[n], XmNvalue,(int)(curr_proj->GX_defaults[j]*d)); n++;
	  XtSetArg (args[n], XmNscaleMultiple,
		    (int)(((curr_proj->GX_max[j]-curr_proj->GX_min[j])*d)/20));
	  n++;
	  XtSetValues(widget_array[i+k_pp_par1_sc], args, n);
	}
    }
    change_parameters (curr_proj->GX_proj_code, params);
  }
}

void resl_callback(widget, tag, data)
     Widget widget;
     int *tag;
     XmScaleCallbackStruct *data;
{
  static Arg args[1];
  static char string[40];
  int i, j, n, sval, srange, stop, sbottom;
  float val, range, top, bottom;

  sprintf (string, "Scale Resolution: %g",
	   ((data->value / 100.0) * (data->value / 100.0)));
  XtSetArg (args[0], XmNvalue, string);
  XtSetValues(widget_array[k_pp_resl_tx], args, 1);
  if ((data->reason == XmCR_VALUE_CHANGED) && (curr_proj)) {
    GX_resl = ((data->value / 100.0) * (data->value / 100.0));
    for (j=0; j<NO_PROJECTION_PARAMS; j++) {
      for (i=0; i<GX_NO_SCALES; i++)
	if (curr_proj->GX_scale_map[i] == j) {
	  val = curr_proj->GX_defaults[j];
	  range = (curr_proj->GX_max[j] - curr_proj->GX_min[j]) * GX_resl;
	  while ((range * (power(10,(curr_proj->GX_decimals[j])))) < 90)
	    curr_proj->GX_decimals[j] += 1;
	  while ((curr_proj->GX_decimals[j] > curr_proj->GX_precision[j]) &&
		 ((range * (power(10,(curr_proj->GX_decimals[j])))) > 900))
	    curr_proj->GX_decimals[j] -= 1;
	  if ((val + (range / 2)) >= curr_proj->GX_max[j]) {
	    top = curr_proj->GX_max[j];
	    bottom = top - range;
	  } else if ((val - (range / 2)) <= curr_proj->GX_min[j]) {
	    bottom = curr_proj->GX_min[j];
	    top = bottom + range;
	  } else {
	    top = (val + (range / 2));
	    bottom = (val - (range / 2));
	  }
	  sval = (int)(val * (power(10,(curr_proj->GX_decimals[j]))));
	  stop = (int)(top * (power(10,(curr_proj->GX_decimals[j]))));
	  sbottom = (int)(bottom * (power(10,(curr_proj->GX_decimals[j]))));
	  srange = (int)(range * (power(10,(curr_proj->GX_decimals[j]))));
	  if (stop <= sbottom) stop = sbottom + 1;
	  if (sval < sbottom) sval = sbottom;
	  if (sval > stop) sval = stop;
	  n = 0;
	  XtSetArg (args[n], XmNminimum, sbottom); n++;
	  XtSetArg (args[n], XmNmaximum, stop); n++;
	  XtSetArg (args[n], XmNscaleMultiple, (srange / 20)); n++;
	  XtSetArg (args[n], XmNvalue, sval); n++;
	  XtSetValues(widget_array[i+k_pp_par1_sc], args, n);
	}
    }
  }
}

void parameter_callback(widget, tag, data)
     Widget widget;
     int *tag;
     XmScaleCallbackStruct *data;
{
  static Arg args[1];
  static char string[40];
  float params[NO_PROJECTION_PARAMS];
  int i, param, proj_code;

  if (curr_proj) {
    param = curr_proj->GX_scale_map[*tag-k_pp_par1_sc];
    proj_code = curr_proj->GX_proj_code;
    sprintf (string, "%s: %.*f", curr_proj->GX_labels[param],
	     (curr_proj->GX_decimals[param]),
	     ((float)data->value/(power(10,(curr_proj->GX_decimals[param])))));
    XtSetArg (args[0], XmNvalue, string);
    XtSetValues(widget_array[(*tag-k_pp_par1_sc)+k_pp_par1_tx], args, 1);

    if ((data->reason == XmCR_VALUE_CHANGED) || (act_scales)) {
      curr_proj->GX_defaults[param] =
	((float)data->value / (power(10,(curr_proj->GX_decimals[param]))));
      for (i=0; i<NO_PROJECTION_PARAMS; i++)
	params[i] = (curr_proj->GX_defaults[i]) * (curr_proj->GX_convert[i]);
      change_parameters (proj_code, params);
    }
  }
}
