#include "xtool.h"

extern GX_scene_list_t scenes;
extern GX_scene_list_t curr_scene;
extern Widget widget_array[];
extern int act_scales;
extern int preserve_scene;
extern char curr_display[];
extern int x_offset;
extern int y_offset;
extern XmString curr_repdat;
extern char RDformat[];
extern char curr_rep[];
extern char curr_data[];

extern void GX_output_command();
extern Boolean GX_string_read();
extern int *GX_get_instance();
extern void GX_get_label();
extern void GX_clear_list();
extern void GX_add_list_item();
extern void GX_create_object();
extern void GX_reset_object();
extern void GX_reset_controls();
extern void GX_reset_new_scene();

/**************************************
   Scene Editor CallBacks...
 **************************************/

void scene_select_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmListCallbackStruct *data;
{
  char *scene_name;
  GB_id scene_id;

  scene_name = (char *)malloc(GB_LABEL_LENGTH);
  GX_string_read (data->item, &scene_name);
  /* Check if the scene is used before, if not add to scenes list. */
  scene_id = GB_get_id (scene_name, GB_SCENE);
  curr_scene = scenes;
  while ((curr_scene != NULL) && (curr_scene->id != scene_id))
    curr_scene = curr_scene->next;
  if (!curr_scene) {
    curr_scene = (GX_scene_list_t) malloc (sizeof (GX_scene_list));
    curr_scene->id = scene_id;
    sprintf (curr_scene->label, "%s", scene_name);
    sprintf (curr_scene->display, "%s", curr_display);
    curr_scene->x_off = 0;
    curr_scene->y_off = 0;
    curr_scene->next = scenes;
    scenes = curr_scene;
  }
  GX_create_object (curr_scene->label, GB_SCENE);
  GX_reset_object (curr_scene->label);
  if (preserve_scene)
    GX_reset_controls ();
  else
    GX_reset_new_scene ();
}

void scene_create_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmAnyCallbackStruct *data;

{
  char *scene_name[GB_LABEL_LENGTH];
  Arg args[1];

  XtSetArg (args[0], XmNvalue, scene_name);
  XtGetValues(widget_array[k_se_text], args, 1);
  GX_create_object (*scene_name, GB_SCENE);
  GX_clear_list (widget_array[k_se_rdlist]);
}

void scene_size_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmScaleCallbackStruct *data;
{
  static int x, y;
  static Arg args[1];
  static char string[19];
  static char command[GX_COM_LEN];
  static GS_scene *scene = NULL;

  switch (*tag) {
  case k_se_width_sc: x = 16 * data->value;
    sprintf (string, "Scene Width: %d", x);
    XtSetArg (args[0], XmNvalue, string);
    XtSetValues(widget_array[k_se_width_tx], args, 1);
    break;
  case k_se_height_sc: y = 16 * data->value;
    sprintf (string, "Scene Height: %d", y);
    XtSetArg (args[0], XmNvalue, string);
    XtSetValues(widget_array[k_se_height_tx], args, 1);
    break;
  }
  if ((data->reason == XmCR_VALUE_CHANGED) || (act_scales)) {
    switch (*tag) {
    case k_se_width_sc:
      XtSetArg (args[0], XmNvalue, &y);
      XtGetValues(widget_array[k_se_height_sc], args, 1);
      y = 16 * y;
      break;
    case k_se_height_sc:
      XtSetArg (args[0], XmNvalue, &x);
      XtGetValues(widget_array[k_se_width_sc], args, 1);
      x = 16 * x;
      break;
    }
    scene = (GS_scene *)
      GX_get_instance ("Scene_Size_Callback", GB_SCENE, curr_scene->label);
    if (scene != NULL) {
      scene->dimensions[0] = x;
      scene->dimensions[1] = y;
    }
    sprintf (command, "(scene %s (dimensions %d %d))",
	     curr_scene->label, x, y);
    GX_output_command (command,0,0);
  }
}

void zoom_callback(widget, tag, data)
     Widget widget;
     int *tag;
     XmScaleCallbackStruct *data;
{
  static Arg args[1];
  static char string[15];
  static GS_scene *scene = NULL;
  static char command[GX_COM_LEN];

  sprintf (string, "Scene Zoom: %g", ((data->value)/10.0));
  XtSetArg (args[0], XmNvalue, string);
  XtSetValues(widget_array[k_se_zoom_tx], args, 1);
  if ((data->reason == XmCR_VALUE_CHANGED) || (act_scales)) {
    scene = (GS_scene *)
      GX_get_instance ("Zoom_Callback", GB_SCENE, curr_scene->label);
    if (scene != NULL)
      scene->zoom = (data->value) / 10.0;
    sprintf (command, "(scene %s (zoom %g))", curr_scene->label,
	     ((data->value)/10.0));
    GX_output_command (command,0,0);
  }
}

void repdat_select_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmListCallbackStruct *data;
{
  curr_repdat = XmStringCopy (data->item);
}

void repdat_delete_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmAnyCallbackStruct *data;
{
  Widget list = widget_array[k_se_rdlist];
  char *text[3*GB_LABEL_LENGTH];
  char rep[GB_LABEL_LENGTH], dat[GB_LABEL_LENGTH];
  GS_scene *scene = NULL;
  GS_rep_list *rep_list = NULL, *last = NULL;
  GB_id id, rep_id, dat_id, priority;
  char command[GX_COM_LEN];

  if (!(XmStringEmpty (curr_repdat))) {
    GX_string_read (curr_repdat, text);
    sscanf (*text, RDformat, rep, dat);
    rep_id = GB_get_id (rep, GB_REPRESENTATION);
    dat_id = GB_get_id (dat, GB_DATA_DESC);
    scene = (GS_scene *)
      GX_get_instance ("RepDat_Delete_Callback", GB_SCENE, curr_scene->label);
    rep_list = scene->representations;
    sprintf (command, "(scene %s\n\t(representations (", curr_scene->label);
    while (rep_list != NULL) {
      if ((rep_list->rep_id != rep_id) || (rep_list->data_id != dat_id)) {
	id = rep_list->rep_id;
	GX_get_label ("RepDat_Delete_Callback", id, rep);
	id = rep_list->data_id;
	GX_get_label ("RepDat_Delete_Callback", id, dat);
	priority = rep_list->priority;
	GX_output_command (command,1,0);
	sprintf (command, "\t((rep_id %s) (data_id %s) (priority %d))",
		 rep, dat, priority);
      }
      else
	if (rep_list == scene->representations)
	  scene->representations = (scene->representations)->next;
	else
	  last->next = rep_list->next;
      last = rep_list;
      rep_list = rep_list->next;
    }
    XmListDeleteItem (list, curr_repdat);
    XmStringFree (curr_repdat);
    curr_repdat = NULL;
    sprintf (command, "%s)))", command);
    GX_output_command (command,0,0);
  }
}

void rep_select_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmListCallbackStruct *data;
{
  char *text[GB_LABEL_LENGTH];
  GX_string_read (data->item, text);
  sprintf (curr_rep, "%s", *text);
}

void rep_edit_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmAnyCallbackStruct *data;

{
  char *text[GB_LABEL_LENGTH];
  Arg args[1];

  XtSetArg (args[0], XmNvalue, text);
  XtGetValues(widget_array[k_se_text], args, 1);
  if (!(XtIsManaged(widget_array[k_re_form])))
    XtManageChild(widget_array[k_re_form]);
}

void data_select_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmListCallbackStruct *data;
{
  char *text[GB_LABEL_LENGTH];
  GX_string_read (data->item, text);
  sprintf (curr_data, "%s", *text);
}

void data_add_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmAnyCallbackStruct *data;
{
  char text[3*GB_LABEL_LENGTH];
  char rep[GB_LABEL_LENGTH], dat[GB_LABEL_LENGTH];
  Widget rdlist;
  GS_scene *scene = NULL;
  GS_rep_list *rep_list = NULL;
  GB_id id;
  int priority = 0;
  char command[GX_COM_LEN];

  if ((curr_data != NULL) && (curr_rep != NULL)) {
    scene = (GS_scene *)
      GX_get_instance ("Data_Add_Callback", GB_SCENE, curr_scene->label);
    rep_list = scene->representations;
    rdlist = widget_array[k_se_rdlist];
    sprintf (command, "(scene %s (representations (((rep_id %s) (data_id %s))",
	     curr_scene->label, curr_rep, curr_data);
    while (rep_list != NULL) {
      id = rep_list->rep_id;
      GX_get_label ("Data_Add_Callback", id, rep);
      id = rep_list->data_id;
      GX_get_label ("Data_Add_Callback", id, dat);
      priority = rep_list->priority;
      GX_output_command (command,1,0);
      sprintf (command, "\t((rep_id %s) (data_id %s) (priority %d))",
	       rep, dat, priority);
      rep_list = rep_list->next;
    }
    rep_list = (GS_rep_list *) malloc (sizeof (GS_rep_list));
    id = GB_get_id (curr_rep, GB_REPRESENTATION);
    rep_list->rep_id = id;
    id = GB_get_id (curr_data, GB_DATA_DESC);
    rep_list->data_id = id;
    rep_list->priority = 1;
    rep_list->next = scene->representations;
    scene->representations = rep_list;
    sprintf (text, RDformat, curr_rep, curr_data);
    GX_add_list_item (rdlist, text);
    sprintf (command, "%s)))", command);
    GX_output_command (command,0,0);
  }
}
