#include <stdio.h>
#include <sys/signal.h>
#include <math.h>
#include <sys/time.h>
#ifdef ultrix
#include <sys/ioctl.h>
#else
#include <sys/filio.h>
#endif
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Mrm/MrmPublic.h>
#include <fcntl.h>
#include <stropts.h> 
#include "xtool.h"


#define PANEL_UID_FILE "xtool.uid"
#define GEO_APP_NAME "GeoData Xtool"
#define GEO_APP_CLASS "xtool"
  
  /**************************************
    Initial Values for some global (yeeach) variables.
 **************************************/

Widget widget_array[k_last_constant];
GX_widget_list_t movies = NULL;
GX_scene_list_t scenes = NULL;
GX_scene_list_t curr_scene = NULL;
Window curr_window = 0;
GX_xtool_projection_t curr_proj = NULL;
int cont_reset = 0;
int file_based = 1;
int act_scales = 0;
int cont_updt = 0;
int cont_disp = 0;
int file_mode = NONE_MODE;
int appl_running = 0;
int preserve_scene = 1;
XmString curr_repdat;
char RDformat[GB_LABEL_LENGTH] = "%s  /  %s";
char curr_rep[GB_LABEL_LENGTH] = "";
char curr_data[GB_LABEL_LENGTH] = "";
char curr_display[GB_LABEL_LENGTH] = "";
int x_offset = 0;
int y_offset = 0;
int displays[5] = {0,0,0,0,0};
FILE *x_in = stdin;
FILE *x_out = stdout;
FILE *x_error = stderr;

/**************************************
 * Demo Init Definitions and functions.  Begin
 **************************************/

/* note from djones: will need to add 2nd bb (24,648) 
   and arrange for 3 beams from each array 
   (can still span same area) */

/* This is wendall`s version.  I need a new version for a movie. */
GX_scene_list demo_ss = {0,"ship_scene",24,24,"",NULL};
GX_scene_list demo_bb = {0,"broad_band",24,504,"",NULL};
GX_scene_list demo_b0 = {0,"beam0",512,24,"",NULL};
GX_scene_list demo_b3 = {0,"beam3",512,168,"",NULL};
GX_scene_list demo_b7 = {0,"beam7",512,312,"",NULL};
GX_scene_list demo_b11 = {0,"beam11",512,456,"",NULL};
GX_scene_list demo_b14 = {0,"beam14",512,600,"",NULL};

void demo_init_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmAnyCallbackStruct *data;
{
  GX_scene_list_t temp;

  demo_ss.id = GB_get_id (demo_ss.label, GB_SCENE);
  demo_bb.id = GB_get_id (demo_bb.label, GB_SCENE);
  demo_b0.id = GB_get_id (demo_b0.label, GB_SCENE);
  demo_b3.id = GB_get_id (demo_b3.label, GB_SCENE);
  demo_b7.id = GB_get_id (demo_b7.label, GB_SCENE);
  demo_b11.id = GB_get_id (demo_b11.label, GB_SCENE);
  demo_b14.id = GB_get_id (demo_b14.label, GB_SCENE);

  demo_ss.next = &demo_bb;
  demo_bb.next = &demo_b0;
  demo_b0.next = &demo_b3;
  demo_b3.next = &demo_b7;
  demo_b7.next = &demo_b11;
  demo_b11.next = &demo_b14;

  temp = scenes;
  while (scenes != NULL) {
    temp = scenes;
    scenes = scenes->next;
    free (temp);
  }
  scenes = &demo_ss;
  curr_scene = scenes;
}

/* My version for working only on X stations.
GX_scene_list demo_b3 = {0,"beam3",0,0,"",NULL};
GX_scene_list demo_b7 = {0,"beam7",0,128,"",NULL};
GX_scene_list demo_b11 = {0,"beam11",0,256,"",NULL};
GX_scene_list demo_b14 = {0,"beam14",0,384,"",NULL};

void demo_init_callback (widget, tag, data)
     Widget widget;
     int *tag;
     XmAnyCallbackStruct *data;
{
  GX_scene_list_t temp;

  demo_b3.id = GB_get_id (demo_b3.label, GB_SCENE);
  demo_b7.id = GB_get_id (demo_b7.label, GB_SCENE);
  demo_b11.id = GB_get_id (demo_b11.label, GB_SCENE);
  demo_b14.id = GB_get_id (demo_b14.label, GB_SCENE);

  demo_b3.next = &demo_b7;
  demo_b7.next = &demo_b11;
  demo_b11.next = &demo_b14;

  temp = scenes;
  while (scenes != NULL) {
    temp = scenes;
    scenes = scenes->next;
    free (temp);
  }
  scenes = &demo_b3;
  curr_scene = scenes;
}
*/

/**************************************
 * Demo Init Definitions and functions.  End
 **************************************/



/**************************************
 * Define CALLBACKs and Mrm Registration list of callbacks.
 **************************************/

/* Control Panel Callbacks... */
void create_callback();
void display_callback();
void offset_callback();
void activate_callback();
void update_all_callback();
void menu_callback();
void file_select_callback();
void file_cancel_callback();
void step_application_callback();
void continuous_update_callback();
void continuous_display_callback();
void preserve_scene_callback();
void movie_callback();
void disp_expose_callback();
void disp_resize_callback();

/* Scene Editor Callbacks... */
void scene_select_callback();
void scene_create_callback();
void scene_size_callback();
void repdat_select_callback();
void repdat_delete_callback();
void rep_select_callback();
void rep_edit_callback();
void data_select_callback();
void data_add_callback();
void zoom_callback();

/* Projection Panel Callbacks... */
void projection_callback();
void parameter_callback();
void resl_callback();

/* Video Panel Callbacks... */
void cancel_video_box();
void play_video();
void VP_update_list();
void cancel_msg_box();
void msg_rescan_list();


MrmRegisterArg reglist[] = {
  {"demo_init_callback", (caddr_t) demo_init_callback},
  {"create_callback", (caddr_t) create_callback},
  {"display_callback", (caddr_t) display_callback},
  {"offset_callback", (caddr_t) offset_callback},
  {"activate_callback", (caddr_t) activate_callback},
  {"update_all_callback", (caddr_t) update_all_callback},
  {"menu_callback", (caddr_t) menu_callback},
  {"file_select_callback", (caddr_t) file_select_callback},
  {"file_cancel_callback", (caddr_t) file_cancel_callback},
  {"step_application_callback", (caddr_t) step_application_callback},
  {"continuous_update_callback", (caddr_t) continuous_update_callback},
  {"continuous_display_callback", (caddr_t) continuous_display_callback},
  {"preserve_scene_callback", (caddr_t) preserve_scene_callback},
  {"movie_callback", (caddr_t) movie_callback},
  {"disp_expose_callback", (caddr_t) disp_expose_callback},
  {"disp_resize_callback", (caddr_t) disp_resize_callback},
  {"scene_select_callback", (caddr_t) scene_select_callback},
  {"scene_create_callback", (caddr_t) scene_create_callback},
  {"scene_size_callback", (caddr_t) scene_size_callback},
  {"repdat_select_callback", (caddr_t) repdat_select_callback},
  {"repdat_delete_callback", (caddr_t) repdat_delete_callback},
  {"rep_select_callback", (caddr_t) rep_select_callback},
  {"rep_edit_callback", (caddr_t) rep_edit_callback},
  {"data_select_callback", (caddr_t) data_select_callback},
  {"data_add_callback", (caddr_t) data_add_callback},
  {"zoom_callback", (caddr_t) zoom_callback},
  {"projection_callback", (caddr_t) projection_callback},
  {"parameter_callback", (caddr_t) parameter_callback},
  {"resl_callback", (caddr_t) resl_callback},
  {"cancel_video_box", (caddr_t) cancel_video_box},
  {"play_video", (caddr_t) play_video},
  {"VP_update_list", (caddr_t) VP_update_list},
  {"cancel_msg_box", (caddr_t) cancel_msg_box},
  {"msg_rescan_list", (caddr_t) msg_rescan_list}
};

int reglist_num = ((sizeof reglist) / (sizeof reglist[0]));


void GX_xtool_step_handler ()
{
  char command[GX_COM_LEN];
  fd_set fset;
  struct timeval zero;

  zero.tv_sec=0;
  zero.tv_usec=0;

  FD_ZERO (&fset);
  FD_SET (x_in->_file,&fset);
  select(getdtablesize(),&fset,0,0,&zero);
/*  fprintf (stderr, ";;; Entering Xtool Step Handler");*/
  if (FD_ISSET(x_in->_file,&fset)){
    /*    sprintf (command, ";;; Entering Xtool Step Handler");
    GX_output_command (command,0,1);*/
    GB_parse_file (x_in, x_out, x_error);
/*    GX_reset_widget_list (widget_array[k_se_sclist], GB_SCENE);
      GX_reset_widget_list (widget_array[k_se_replist], GB_REPRESENTATION);
      GX_reset_widget_list (widget_array[k_se_datlist], GB_DATA_DESC);
      GX_reset_movie_list ();*/
    if (appl_running) {
      sprintf (command, ";;; Xtool Step Handler:  Application Running\n");
      sprintf (command, "%s\n(handshake handshake-foo)",command);
    }
    else {
      sprintf (command, ";;; Xtool Step Handler:  Application Paused");
    }
    GX_output_command (command,0,0);
  }
}

void create_app_shell (width, height, k_sh, k_fm, title, uname, hier, disp)
     int width, height, k_sh, k_fm;
     char title[], uname[];
     MrmHierarchy hier;
     Display *disp;

{
  Widget *shell, *form;
  MrmCode class;
  Arg arglist[5];
  int nargs;

  nargs = 0;
  XtSetArg (arglist[nargs], XmNwidth, width); nargs++;
  XtSetArg (arglist[nargs], XmNheight, height); nargs++;
  shell = &(widget_array[k_sh]);
  form = &(widget_array[k_fm]);
  *shell = XtAppCreateShell (title, "xtool", applicationShellWidgetClass,
			     disp, arglist, nargs);
  if (MrmFetchWidget (hier, uname, *shell, form, &class) != MrmSUCCESS) {
    fprintf (stderr, "Can't fetch %s widget\n", uname);
    exit (1);
  }
}

/**************************************
   Main Program
 **************************************/

main(argc,argv)
     int argc;
     char **argv;
{
  Widget *shell, *form;
  MrmHierarchy hierarchy;
  MrmCode class;
  Arg arglist[5];
  Display *display;
  XtAppContext context;
  char pathname[192];
  char *pathname_foo;
  int argc_foo=argc, nargs;
  struct itimerval this;
  int i;

  GX_init_data ();

  MrmInitialize();

  XtToolkitInitialize ();
  context = XtCreateApplicationContext ();
  display = XtOpenDisplay (context, 0, GEO_APP_NAME, GEO_APP_CLASS, 0, 0,
			   &argc, argv);
  if (display == NULL) {
    XtWarning ("cannot open display");
    exit (1);
  }


/*  sprintf (pathname,"./%s",PANEL_UID_FILE);*/
/*  pathname_foo = pathname;*/
/*  if (MrmOpenHierarchy (1,&pathname_foo, NULL, &hierarchy) != MrmSUCCESS) {*/
    sprintf (pathname,"%s/%s",GEODATA_LIB,PANEL_UID_FILE);
    pathname_foo = pathname;
    if (MrmOpenHierarchy (1,&pathname_foo, NULL, &hierarchy) != MrmSUCCESS) {
      fprintf (stderr, "Can't open Mrm Hierarchy\n");
      exit (1);
    }
/*  }*/

  if (MrmRegisterNames (reglist, reglist_num) != MrmSUCCESS) {
    fprintf (stderr, "Can't register names\n");
    exit (1);
  }

  create_app_shell (475, 185, k_geodata_shell, k_geodata_form, "GeoData Xtool",
		    "geodata_main", hierarchy, display);
  create_app_shell (450, 350, k_se_shell, k_se_form, "Scene Editor",
		    "se_form", hierarchy, display);
  create_app_shell (520, 255, k_pp_shell, k_pp_form, "Projection Panel",
		    "pp_form", hierarchy, display);
  create_app_shell (400, 300, k_re_shell, k_re_form, "Rep Editor",
		    "re_form", hierarchy, display);
  create_app_shell (230, 280, k_vp_shell, k_vp_form, "Video Disk Movies",
		    "vp_form", hierarchy, display);

  if (MrmFetchWidget (hierarchy, "dis_win_fm", widget_array[k_geodata_shell],
		      &widget_array[k_dis_win_fm], &class) != MrmSUCCESS) {
    fprintf (stderr, "Can't fetch display window widget\n");
    exit (1);
  }

  if (MrmFetchWidget (hierarchy, "fil_sel_ls", widget_array[k_geodata_shell],
		      &widget_array[k_fil_sel_ls], &class) != MrmSUCCESS) {
    fprintf (stderr, "Can't fetch file selection widget\n");
    exit (1);
  }

  if (MrmFetchWidget (hierarchy, "vp_msg_box", widget_array[k_geodata_shell],
		      &widget_array[k_vp_msg_box], &class) != MrmSUCCESS) {
    fprintf (stderr, "Can't fetch warning box widget\n");
    exit (1);
  }

  MrmCloseHierarchy (hierarchy);

  XtManageChild (widget_array[k_geodata_form]);

  XtRealizeWidget (widget_array[k_geodata_shell]);
/*  {
    struct sigvec hval;
    sigvec(SIGIO,0,hval);
    fprintf (stderr,"%x %x\n",hval.sv_handler,GX_xtool_step_handler);
  }
  signal (SIGIO, GX_xtool_step_handler);
  {
    struct sigvec hval;
    sigvec(SIGIO,0,hval);
    fprintf (stderr,"%x %x\n",hval.sv_handler,GX_xtool_step_handler);
  }
*/
  this.it_interval.tv_sec = 1;
  this.it_interval.tv_usec = 0;
  this.it_value.tv_sec = 1;
  this.it_value.tv_usec = 0;
  signal (SIGALRM, GX_xtool_step_handler);
  if (setitimer (ITIMER_REAL,&this,NULL) == -1) perror("");
  signal (SIGIO, GX_xtool_step_handler);
  fcntl(0,F_SETFL,FASYNC|FNDELAY);
  ioctl(0,I_SETSIG,S_INPUT);
  XtAppMainLoop (context);
}
